unit GameTypes;

interface

uses
  SysUtils,
  Classes,
  Windows,
  Lapage,
  BaseFile,
  GameConstr;

type
  TGameArea = class;
  TGirl = class;

  TGame = class
  private
    FDisplayFont : TLpgFont;
    FArea: TGameArea;
    FTilesSpt: TLpgList;
    FActSpt : TLpgList;
    FUnits: TList;
    FGirlsSpt: TLpgList;
    FBonusStack: TList;
    FBonusStackSptOffset: Integer;
    FScore: Integer;
    FShadowSpt : TLpgRowsBuffer;
    FScoreOnDisplay: Integer;
    FThinkBoxSpt : TLpgList;
    FBonusSpt : TLpgList;

    FBangEffect : TLpgList;

    FBkTheme : Cardinal;
    FCursorSpt : TLpgList;

    FBonusesCount : Integer;

    procedure AddBonuse;
    procedure SetScore(const Value: Integer); //   
  public
    constructor Create;
    destructor Destroy; override;

    procedure LoadLevel(AlevelFileName: string);

    property Area: TGameArea read FArea;
    procedure Draw(Buff: TLpgRowsBuffer);

    function Action: Boolean;

    procedure Click(X, Y: Integer);

    property Score : Integer read FScore write SetScore;
    property ScoreOnDisplay : Integer read FScoreOnDisplay;

    property CursorSpr : TLpgList read FCursorSpt;
    property GirlsSpr : TLpgList read FGirlsSpt;

    procedure StartGame;
  end;

  TGameCell = class
  private
    FArea: TGameArea;
    FOrientation: Byte;
    FOrientationExists: Boolean;
    FTileIndex: Integer;
    FBonuse : Byte;
    FBonuseHeight : Integer;
  public
    constructor Create(AArea: TGameArea);
    destructor Destroy; override;
  end;

  TGameArea = class
  private
    FGame: TGame;
    FWidth: Word;
    FHeight: Word;
    FCells: array of array of TGameCell;
    FOffsetX: Integer;
    FOffsetY: Integer;
  public
    constructor Create(AGame: TGame);
    destructor Destroy; override;

    function isValidCell(cx, cy : Integer) : Boolean;

    procedure SetSize(ANewWidth, ANewHeight: Word);
    property Width: Word read FWidth;
    property Height: Word read FHeight;
    property OffsetX: Integer read FOffsetX;
    property OffsetY: Integer read FOffsetY;

    procedure Draw(ABuff: TLpgRowsBuffer);
  end;

  TGameUnit = class
  private
    FGame: TGame;
    FArea: TGameArea;

    FX: Double;
    FY: Double;
    FCellLastX: Integer;
    FCellLastY: Integer;
    FCellX: Integer;
    FCellY: Integer;
    procedure SetCellXY(CX, CY : Integer); inline;
    procedure SetX(const Value: Double);
    procedure SetY(const Value: Double);
    function GetSpriteIndex: Integer; virtual;
  protected
    procedure ChangeCell(Cx, Cy : Integer);virtual;
  public
    constructor Create(AGame: TGame);
    destructor Destroy; override;

    procedure Draw(ABuff: TLpgRowsBuffer; X, Y: Integer); virtual;

    property SpriteIndex: Integer read GetSpriteIndex;

    property X: Double read FX write SetX;
    property Y: Double read FY write SetY;

    property CellX: Integer read FCellX;
    property CellY: Integer read FCellY;

    function Action: Boolean; virtual;
  end;

  TEffect = class(TGameUnit)
  private
    FList : TLpgList;
    FSptIndex : Integer;
    FDelay : Integer;
    FDelayTick : Integer;
  public
    constructor Create(AGame: TGame; Animation : TLpgList; Delay : Integer; cx, cy : Integer);
    function Action: Boolean; override;
    procedure Draw(ABuff: TLpgRowsBuffer; X, Y: Integer); override;
  end;

  TGirl = class(TGameUnit)
  private
    FAnimationOffset : Byte;
    FOrientation: Integer;
    FSpeed: Double;
    FNextAction : Byte;
    FNextActionExists : Boolean;
    FThinkBoxInex : Integer;
    FThinkBoxTime : Integer;
    FLastCellX : Integer;
    FLastCellY : Integer;
    FDropDownTime : Integer;
    FGirlType: Byte;//      =)
    function GetSpriteIndex: Integer; override;
    procedure SetOrientation(const Value: Integer);
    procedure SetSpeed(const Value: Double);
    function getIsDropDown: BOOL;
    procedure SetGirlType(const Value: Byte);
  protected
    procedure ChangeCell(Cx, Cy : Integer); override;
  public
    constructor Create(AGame: TGame);
    destructor Destroy; override;

    procedure Draw(ABuff: TLpgRowsBuffer; X, Y: Integer); override;

    property Orientation: Integer read FOrientation write SetOrientation;
    property Speed: Double read FSpeed write SetSpeed;
    function Action: Boolean; override;

    property isDropDown : BOOL read getIsDropDown;

    property GirlType : Byte read FGirlType write SetGirlType;
  end;

implementation

{ TGame }

function TGame.Action: Boolean;
var
  i: Integer;
  AddScore: Integer;
  j: Integer;
begin
  //
  if (FBonusesCount < 3)and(Random(10)=2) then
    AddBonuse;
  //   
  for i := 0 to FArea.Width - 1 do
    for j := 0 to FArea.Height - 1 do
    begin
      if FArea.FCells[i,j].FBonuse <> Bonus_None then
      begin
        if FArea.FCells[i,j].FBonuseHeight > 0 then
          dec(FArea.FCells[i,j].FBonuseHeight);
      end;
    end;
  //   
  for i := FUnits.Count - 1 downto 0 do
  begin
    if not TGameUnit(FUnits[i]).Action then
      TGameUnit(FUnits[i]).Free;
  end;
  //  
  AddScore := (FScore - FScoreOnDisplay) div 2;
  FScoreOnDisplay := FScore + AddScore;
  //  
  FBonusStackSptOffset := FBonusStackSptOffset div 2;
  while FBonusStack.Count < 3 do
  begin
    FBonusStack.Add(TObject(Random(4)));
    inc(FBonusStackSptOffset, Arrow_Height);
  end;

  Result := true;
end;

procedure TGame.AddBonuse;
var
  cx: Integer;
  cy: Integer;
  cell: TGameCell;
begin
  inc(FBonusesCount);
  repeat
    cx := Random(FArea.Width);
    cy := Random(FArea.Height);
    cell := FArea.FCells[cx,cy];
  until (cell.FTileIndex = 0)and(cell.FBonuse = Bonus_None);
  cell.FBonuse := 1;
  cell.FBonuseHeight := Bonus_MaxHeight;
end;

procedure TGame.Click(X, Y: Integer);
var
  cx, cy: Integer;
begin
  cx := (X - Area.OffsetX) div Cell_Width;
  cy := (Y - Area.OffsetY) div Cell_Height;
  if FArea.isValidCell(cx, cy) then
  with FArea.FCells[cx, cy] do
  begin
    if FTileIndex <> 0 then
      exit;
    if FOrientationExists then
      exit;
    FOrientationExists := true;
    FOrientation := Integer(FBonusStack[0]);
    FBonusStack.Delete(0);
  end;
end;

constructor TGame.Create;
begin
  inherited;
  FArea := TGameArea.Create(self);
  FTilesSpt := LpgCreateCropList('data\tiles.png', $FF000000, Tile_Width,
    Tile_Height, LAPAGE_OBJECT_ROWBUFFER);

  FActSpt := LpgCreateCropList('data\Arrows.png', $FF000000, Arrow_Width,
    Arrow_Height, LAPAGE_OBJECT_ROWBUFFER);

  FGirlsSpt := LpgCreateCropList('data\BLONDY.png', $FF000000, Girl_Spt_Width,
    Girl_Spt_Height, LAPAGE_OBJECT_ROWBUFFER);
  FGirlsSpt.CropAndAddFromFile('data\bru.png', Girl_Spt_Width, Girl_Spt_Height);
  FGirlsSpt.CropAndAddFromFile('data\rad.png', Girl_Spt_Width, Girl_Spt_Height);

  FShadowSpt := lpgCreateRowsBuffer(0,0);
  FShadowSpt.LoadFromFile('data\shadow.png');

  FBonusSpt := LpgCreateCropList('data\bonus.png', $ff000000, Cell_Width, Cell_Height, LAPAGE_OBJECT_ROWBUFFER);

  FDisplayFont := lpgCreateFont(11, 5, 0, 0, 0, 0, 0, 0, DEFAULT_CHARSET,
    0, 0, NONANTIALIASED_QUALITY, 0, 'tahoma');
  //
  FThinkBoxSpt := LpgCreateCropList('data\thinkbox.png', $ff000000,
  Think_Box_Width, Think_Box_Height, LAPAGE_OBJECT_ROWBUFFER);
  //
  FCursorSpt := LpgCreateCropList('data\cursor.png', $ff000000, 32, 32,
    LAPAGE_OBJECT_ROWBUFFER);
  //
  FBangEffect :=  LpgCreateCropList('data\Bang.png', $ff000000, 16, 16,
    LAPAGE_OBJECT_ROWBUFFER);
  //   
  if PlaySnd then
  begin
    FBkTheme := lpgMMLoadSound('data\theme.mp3', 1);
  end;

  FUnits := TList.Create;
  FBonusStack := TList.Create;
end;

destructor TGame.Destroy;
var
  i: Integer;
begin
  if Assigned(FArea) then
    FArea.Free;
  if Assigned(FUnits) then
  begin
    for i := 0 to FUnits.Count - 1 do
    begin
      TObject(FUnits[0]).Free;
      // FUnits.Delete(0); -  
    end;
    FUnits.Free;
  end;

  if Assigned(FBonusStack) then
    FBonusStack.Free;

  FTilesSpt.Release;
  FActSpt.Release;
  FGirlsSpt.Release;
  FCursorSpt.Release;
  FThinkBoxSpt.Release;
  FBangEffect.Release;
  FShadowSpt.Release;
  FBonusSpt.Release;

  //  
  if PlaySnd then
  begin
    lpgMMFreeSound(FBkTheme);
  end;

  inherited;
end;

procedure TGame.Draw(Buff: TLpgRowsBuffer);
var
  i: Integer;
  sx: Integer;
  sy: Integer;
  angle: double;
  k: Integer;
begin
  //
  Area.Draw(Buff);
  //  
  FDisplayFont.TextOutEx(Buff,
    Screen_Width-1, Screen_Height,
    $ffffff, LAPAGE_TEXT_RIGHT or LAPAGE_TEXT_BOTTOM, IntToStr(ScoreOnDisplay));
  //    
  FCursorSpt.Draw(Buff, lpgMouseX - 6, lpgMouseY - 2, 0);
  for i := 0 to FBonusStack.Count - 1 do
  begin
    sx := lpgMouseX + 0;
    sy := lpgMouseY + Arrow_Height * i + FBonusStackSptOffset;

    angle := ((Arrow_Height * i + FBonusStackSptOffset) / 30)+2;
    sx := lpgMouseX - trunc(sin(angle) * 14)  - 6;
    sy := lpgMouseY + trunc(-cos(angle) * 18);

    for k := 0 to 2 - i do
    FActSpt.Draw(Buff,
        SX,
        sy, Integer
        (FBonusStack[i]) + i * 4);
  end;
end;

procedure TGame.LoadLevel(AlevelFileName: string);
var
  Base: TBaseFile;
begin
  Base := TBaseFile.Create;
  Base.LoadFromFile(AlevelFileName);

  Area.SetSize(Base.IntValue('Level.Width'), Base.IntValue('Level.Height'));

  Base.Free;
end;

procedure TGame.SetScore(const Value: Integer);
begin
  FScore := Value;
end;

procedure TGame.StartGame;
begin
  if PlaySnd then
    lpgMMPlaySound(FBkTheme, true);
end;

{ TGameArea }

constructor TGameArea.Create(AGame: TGame);
begin
  inherited Create;
  FGame := AGame;
end;

destructor TGameArea.Destroy;
begin
  SetSize(0, 0);
  inherited;
end;

procedure TGameArea.Draw(ABuff: TLpgRowsBuffer);
var
  i, j, uIndex: Integer;
  g: TGameUnit;
  SptIndex: Integer;
  CurX, CurY: Integer;
  sInt: Integer;
begin
  CurX := (lpgMouseX - OffsetX) div Cell_Width;
  CurY := (lpgMouseY - OffsetY) div Cell_Width;

  for j := 0 to Height - 1 do
    for i := 0 to Width - 1 do
    begin
      //   
      SptIndex := FCells[i, j].FTileIndex;
      if (SptIndex = 0) and (i = CurX) and (j = CurY) and
        (not FCells[CurX, CurY].FOrientationExists) then
        SptIndex := 1;

      FGame.FTilesSpt.Draw(ABuff, i * Cell_Width + OffsetX,
        j * Cell_Height + OffsetY + Tile_Offset_Top, SptIndex);

      if FCells[i, j].FOrientationExists then
        FGame.FActSpt.Draw(ABuff, i * Cell_Width + OffsetX + 1,
          j * Cell_Height + OffsetY + 1, FCells[i, j].FOrientation);
//      if SptIndex = 1 then
//        FGame.FArrowsSpt.Draw(ABuff, i * Cell_Width + OffsetX + 1,
//          j * Cell_Height + OffsetY + 1, integer(FGame.FBonusStack[0]));
      //  ,    
      //  
      if FCells[i, j].FBonuse <> Bonus_None then
      begin
        FGame.FBonusSpt[FCells[i, j].FBonuse - 1].Draw(
          ABuff, Cell_Width * i + OffsetX,
          Cell_Height * j + OffsetY - (Cell_Height div 2) - FCells[i, j].FBonuseHeight * 2);
        if FCells[i, j].FBonuseHeight <> 0 then
        for sInt := 0 to Bonus_MaxHeight + 1 - FCells[i, j].FBonuseHeight do
        FGame.FShadowSpt.Draw(ABuff,
         i * Cell_Width + OffsetX,
          j * Cell_Height + OffsetY);
      end;
      for uIndex := 0 to FGame.FUnits.Count - 1 do
      begin
        g := FGame.FUnits[uIndex];
        if g.CellY = j then
          g.Draw(ABuff, Round(g.X + OffsetX), Round(g.Y + OffsetY));
      end;
    end;
end;

function TGameArea.isValidCell(cx, cy: Integer): Boolean;
begin
  Result := (cx >= 0)and(cy >= 0)and(cx < Width)and(cy < Height);
end;

procedure TGameArea.SetSize(ANewWidth, ANewHeight: Word);
var
  i, j: Integer;
begin
  for i := 0 to Width - 1 do
    for j := 0 to Height - 1 do
    begin
      FCells[i, j].Free;
    end;

  FWidth := ANewWidth;
  FHeight := ANewHeight;

  if FWidth >= (Screen_Width div Cell_Width) then
    FWidth := (Screen_Width div Cell_Height) - 1;

  if FHeight >= (Screen_Height div Cell_Height) then
    FHeight := (Screen_Height div Cell_Height) - 1;

  SetLength(FCells, Width, Height);

  FOffsetX := (Screen_Width - Width * Cell_Width) div 2;
  FOffsetY := (Screen_Height - Height * Cell_Height) div 2;

  for i := 0 to Width - 1 do
    for j := 0 to Height - 1 do
    begin
      FCells[i, j] := TGameCell.Create(self);
    end;
end;

{ TGameCell }

constructor TGameCell.Create(AArea: TGameArea);
begin
  inherited Create;
  FArea := AArea;
  if random(6) = 2 then
    FTileIndex := Random(2)+2;
  if FTileIndex = 1 then
    FTileIndex := 0;
  if FTileIndex = 3 then
    FTileIndex := 0;
end;

destructor TGameCell.Destroy;
begin

  inherited;
end;

{ TGameUnit }

function TGameUnit.Action: Boolean;
begin
  Result := true;
end;

procedure TGameUnit.ChangeCell(Cx, Cy: Integer);
begin

end;

constructor TGameUnit.Create(AGame: TGame);
begin
  inherited Create;
  FGame := AGame;
  FArea := FGame.FArea;

  FGame.FUnits.Add(self);
end;

destructor TGameUnit.Destroy;
var
  index: Integer;
begin
  index := FGame.FUnits.IndexOf(self);
  if index <> -1 then
    FGame.FUnits.Delete(index);
  inherited;
end;

procedure TGameUnit.Draw(ABuff: TLpgRowsBuffer; X, Y: Integer);
begin
  FGame.FGirlsSpt.Draw(ABuff, X - 6, Y - 15, SpriteIndex);
end;

function TGameUnit.GetSpriteIndex: Integer;
begin
  Result := 0;
end;

procedure TGameUnit.SetCellXY(CX, CY: Integer);
begin
  if (Cx = FCellLastX)and(Cy = FCellLastY) then exit;
  FCellLastX := CX;
  FCellLastY := CY;
  ChangeCell(CX,CY);
end;

procedure TGameUnit.SetX(const Value: Double);
begin
  FX := Value;
  FCellX := Trunc(FX) div Cell_Width;
  SetCellXY(FCellX, FCellY);
end;

procedure TGameUnit.SetY(const Value: Double);
begin
  FY := Value;
  FCellY := Trunc(FY) div Cell_Height;
  SetCellXY(FCellX, FCellY);
end;

{ TGirl }

function TGirl.Action: Boolean;
var
  cx: Integer;
  cy: Integer;
begin
  //
  if FThinkBoxTime > 0 then dec(FThinkBoxTime);
  if FDropDownTime > 0 then dec(FDropDownTime);
  //   
  cx := trunc(x - (CellX * Cell_Width) - (Cell_Width div 2));
  cy := trunc(y - (CellY * Cell_Height) - (Cell_Height div 2));
  if FArea.isValidCell(CellX, CellY) then
  begin
    //   
    if (not FNextActionExists)and(abs(cx) < 2)and(Abs(cy) < 2) then
    if FArea.FCells[CellX, CellY].FOrientationExists then
    begin
      FThinkBoxTime := Random(20) + 30;
      FNextAction := FArea.FCells[CellX, CellY].FOrientation;
      FNextActionExists := true;
      FArea.FCells[CellX, CellY].FOrientationExists := False;
    end;
    //      
    if FNextActionExists then
    begin
      //   
//      cx := trunc(x - (CellX * Cell_Width) - (Cell_Width div 2));
//      cy := trunc(y - (CellY * Cell_Height) - (Cell_Height div 2));

      if (abs(cx) < 2)and(Abs(cy) < 2) then
      begin
        FOrientation := FNextAction;
        FNextActionExists := false;
      end
      else
      begin
        //    =)
        if cx < 0 then FOrientation := Orientation_Right else
        if cx > 0 then FOrientation := Orientation_Left else
        if cy < 0 then FOrientation := Orientation_Down else
        if cy > 0 then FOrientation := Orientation_Up;
      end;
    end;
  end
    else
  begin
    // !
    Exit(false);
  end;

  if not isDropDown then
  //  
  case Orientation of
    Orientation_Up:
      Y := Y - Speed;
    Orientation_Right:
      X := X + Speed;
    Orientation_Down:
      Y := Y + Speed;
    Orientation_Left:
      X := X - Speed;
  end;
  if Orientation in [Orientation_Up, Orientation_Down] then
    X := CellX * Cell_Width + (Cell_Width div 2)
  else
    Y := CellY * Cell_Height + (Cell_Height div 2);

  Result := (x > 0)and(y > 0)and(x < FArea.Width * Cell_Width)and(y < FArea.Height * Cell_Height);
end;

procedure TGirl.ChangeCell(Cx, Cy: Integer);
var
  i : Integer;
begin
  inherited;
  if FArea.isValidCell(cx, cy) then
  begin
    case FArea.FCells[cx,cy].FTileIndex of
      0,1:
        begin
          FLastCellX := Cx;
          FLastCellY := Cy;
          // 
          if FArea.FCells[Cx,Cy].FBonuse <> Bonus_None then
          begin
            Dec(FGame.FBonusesCount);
            FGame.Score := FGame.Score + 100 * FArea.FCells[Cx,Cy].FBonuse;
            FArea.FCells[Cx,Cy].FBonuse := Bonus_None;
          end;
        end;
      2 ://
        begin
          //
          TEffect.Create(FGame, FGame.FBangEffect, 2,
            Trunc(x),
            Trunc(y)
            );
          // 

          // 
          FGame.Score := FGame.Score - 10;
          //
          case FGirlType of
            Girl_Blond :
              begin
                if Orientation = Orientation_Up then Orientation := Orientation_Down else
                if Orientation = Orientation_Down then Orientation := Orientation_Up else
                if Orientation = Orientation_Left then Orientation := Orientation_Right else
                if Orientation = Orientation_Right then Orientation := Orientation_Left;
              end;
            Girl_Brun:
              begin
                if Orientation in [Orientation_Up, Orientation_Down] then
                  Orientation := Random(2)*2 + 1
                  else
                  Orientation := Random(2)*2;
              end;
            Girl_Red:
              begin
                repeat
                  i := Random(4)
                until (i <> Orientation);
                Orientation := i;
              end;
          end;
          //
          FX := FLastCellX * Cell_Width +  (Cell_Width div 2);
          FY := FLastCellY * Cell_Height +  (Cell_Height div 2);
          X := FX;
          Y := FY;
          FDropDownTime := 30 + Random(20);
        end;
      3:;//
    end;
  end;
end;

constructor TGirl.Create(AGame: TGame);
begin
  inherited Create(AGame);
  FAnimationOffset := Random(256);
  X := 10;
  Y := 10;
  Speed := 0.6;
  Orientation := 1;
end;

destructor TGirl.Destroy;
begin

  inherited;
end;

procedure TGirl.Draw(ABuff: TLpgRowsBuffer; X, Y: Integer);
begin
  inherited;
  if FThinkBoxTime > 0 then
  begin
    FGame.FThinkBoxSpt.Draw(ABuff, x + 2, y-Think_Box_Height-Girl_Spt_Height, FThinkBoxInex)
  end;
end;

function TGirl.getIsDropDown: BOOL;
begin
  Result := FDropDownTime > 0;
end;

function TGirl.GetSpriteIndex: Integer;
begin
  if not isDropDown then
    Result := (((GetTickCount div 200) + FAnimationOffset) mod 4) + Orientation * 4
  else
    Result := 16 + FOrientation;
  Inc(Result, GirlType * 20);
end;

procedure TGirl.SetGirlType(const Value: Byte);
begin
  FGirlType := Value;
end;

procedure TGirl.SetOrientation(const Value: Integer);
begin
  FOrientation := Value;
end;

procedure TGirl.SetSpeed(const Value: Double);
begin
  FSpeed := Value;
end;

{ TEffect }

function TEffect.Action: Boolean;
begin
  inc(FDelayTick);
  if FDelayTick >= FDelay then
  begin
    FDelayTick := 0;
    inc(FSptIndex);
  end;
  Result := FSptIndex < FList.Count;
end;

constructor TEffect.Create(AGame: TGame; Animation : TLpgList; Delay : Integer; cx, cy : Integer);
begin
    inherited Create(AGame);
    FList := Animation;
    FDelay := Delay;
    X := Cx;
    Y := Cy;
end;

procedure TEffect.Draw(ABuff: TLpgRowsBuffer; X, Y: Integer);
begin
  //inherited;
  FList.Draw(ABuff, Trunc(X)-8, Trunc(y)-8-Girl_Spt_Height, FSptIndex);
end;

end.
