unit uGUI;
// Andru
//     ))

interface

uses zglHeader;

{$REGION '  CONSTS  '}
const
  // 
  GUI_NULL      = 0;
  GUI_BUTTON    = 1;
  GUI_LABEL     = 2;
  GUI_CHECKBOX  = 3;
  GUI_LISTBOX   = 4;
  GUI_SCROLLBAR = 5;

  //
  GUI_DEFOULT_COLOR = $828282;
  GUI_FOCUS_COLOR   = $FFC56B;
  GUI_MOUSEIN_COLOR = $AEAEAE;
  GUI_PRESSED_COLOR = $4A4A4A;
//  GUI_DISABLE_COLOR = $222222;
{$ENDREGION}

{$REGION '  TYPES  '}
type  pGUIEvents = ^tGUIEvents;
      pGUIObject = ^tGUIObject;
      pGUIManager = ^tGUIManager;

      tGUIEvents = record
        OnCreate     : procedure( GUIObj : pGUIObject );
        OnSetFocus   : procedure( GUIObj : pGUIObject; const Focus : Boolean );
        OnClick      : procedure( GUIObj : pGUIObject );
        OnStartDrag  : procedure( GUIObj : pGUIObject );
        OnDrag       : procedure( GUIObj : pGUIObject; const X, Y : Single );
        OnEndDrag    : procedure( GUIObj : pGUIObject; const X, Y : Single );
        OnMouseUp    : procedure( GUIObj : pGUIObject );
        OnMouseMove  : procedure( GUIObj : pGUIObject; const X, Y : Single );
        OnMouseEnter : procedure( GUIObj : pGUIObject );
        OnMouseLeave : procedure( GUIObj : pGUIObject );
        OnMouseWheel : procedure( GUIObj : pGUIObject; const Axis : Byte );
        OnKeyDown    : procedure( GUIObj : pGUIObject; const KeyCode : Byte );
        OnKeyUp      : procedure( GUIObj : pGUIObject; const KeyCode : Byte );
        OnChange     : procedure( GUIObj : pGUIObject; const Value, Change : boolean );
      end;

      tGUIObject = record
        _type      : Integer;

        Desc       : Pointer;
        Data       : Pointer;
        Rect       : zglTRect;

        Layer      : Integer;
        Visible    : Boolean;
        Focus      : Boolean;
        MouseIn    : Boolean;

        OnDraw     : procedure( const GUIObj : pGUIObject );
        OnUpdt     : procedure( const GUIObj : pGUIObject );
        Events     : tGUIEvents;

        GUIMan     : pGUIManager;
        next_obj   : pGUIObject;
        prew_obj   : pGUIObject;
        _next, _prew : pGUIObject
      end;

      //GUI Manager
      tGUIManager = record
        NowLayer    : integer;
        LastFocused : pGUIObject;

        Count  : word;
        GUIObj : pGUIObject;
      end;

//////Descs
      pGUIButtonDesc = ^tGUIButtonDesc;
      tGUIButtonDesc = record
        Font    : zglPFont;
        Caption : String;
        Pressed : Boolean;
      end;

      pGUICheckBoxDesc = ^tGUICheckBoxDesc;
      tGUICheckBoxDesc = record
        Font    : zglPFont;
        Caption : String;
        Checked : Boolean;
      end;

      pGUILabelDesc = ^tGUILabelDesc;
      tGUILabelDesc = record
        Font    : zglPFont;
        Caption : String;
      end;

      pGUIListBoxDesc = ^tGUIListBoxDesc;
      tGUIListBoxDesc = record
        Font       : zglPFont;
        List       : zglTStringList;
        ItemIndex  : Integer;
        ItemHeight : Integer;
        SelectMode : Integer;
      end;

      pGUIScrollBarDesc = ^tGUIScrollBarDesc;
      tGUIScrollBarDesc = record
        Kind     : Byte;
        Step     : Integer;
        Position : Integer;
        PageSize : Integer;
        Max      : Integer;

        uPressed : Boolean;
        dPressed : Boolean;
        sDraged  : Boolean;
      end;
//////
{$ENDREGION}

//  
procedure GUI_SetLayer( Layer : integer );

// 
procedure mGUI_Init;
// 
procedure mGUI_Draw;
//  
procedure mGUI_Update;

//    
function  GUI_AddObject( const _type : Integer; const X, Y, W, H : Single;
                         const Focus, Visible : Boolean; const Desc, Data : Pointer;
                         const next, prew : pGUIObject ) : pGUIObject;

implementation

var managerGUI : tGUIManager;

procedure GUI_SetLayer( Layer : integer );
begin
  managerGUI.NowLayer := Layer;
end;

procedure DefDraw( const GUIObj : pGUIObject );
begin
  with GUIObj^ do begin
    if not Visible then Exit;
    case _type of
      GUI_NULL     : ;
      GUI_BUTTON   : begin
        if MouseIn then begin
          if pGUIButtonDesc( Desc ).Pressed then PR2D_Rect( Rect.X, Rect.Y, Rect.W, Rect.H, GUI_PRESSED_COLOR, 255, PR2D_FILL )
            else PR2D_Rect( Rect.X, Rect.Y, Rect.W, Rect.H, GUI_MOUSEIN_COLOR, 255, PR2D_FILL );
        end else PR2D_Rect( Rect.X, Rect.Y, Rect.W, Rect.H, GUI_DEFOULT_COLOR, 255, PR2D_FILL );
        if Focus   then PR2D_Rect( Rect.X, Rect.Y, Rect.W, Rect.H, GUI_FOCUS_COLOR, 255 );
        Text_Draw( pGUIButtonDesc( Desc ).Font,
                   Rect.X + Rect.W/2, Rect.Y + Rect.H/2 -pGUIButtonDesc( Desc ).Font.MaxHeight/2,
                   pGUIButtonDesc( Desc ).Caption,
                   TEXT_HALIGN_CENTER );
      end;
      GUI_LABEL    : begin

      end;
      GUI_CHECKBOX : begin

      end;
    end;
  end;
end;

procedure DefUpdt( const GUIObj : pGUIObject );
begin
  with GUIObj^ do begin
    if not Visible then Exit;
    case GUIObj._type of
      GUI_NULL     : ;
      GUI_BUTTON   : begin
        if col2d_PointInRect( Mouse_X, Mouse_Y, Rect ) then begin
          if not MouseIn then begin
            if @Events.OnMouseEnter <> nil then
              Events.OnMouseEnter( GUIObj );
            MouseIn := true
          end;
          if Mouse_Down( M_BLEFT ) then begin
            pGUIButtonDesc( Desc ).Pressed := true;
          end else begin
            pGUIButtonDesc( Desc ).Pressed := false;
          end;
          if Mouse_Click( M_BLEFT ) then begin
            if @Events.OnClick <> nil then
              Events.OnClick( GUIObj );
            if @Events.OnSetFocus <> nil then
              Events.OnSetFocus( GUIObj, true );
            GUIMan.LastFocused.Focus := false;
            if @GUIMan.LastFocused.Events.OnSetFocus <> nil then
              GUIMan.LastFocused.Events.OnSetFocus( GUIMan.LastFocused, false );
            GUIMan.LastFocused := GUIObj;
            Focus := true;
          end;
        end else begin
          if MouseIn then begin
            if @Events.OnMouseLeave <> nil then
              Events.OnMouseLeave( GUIObj );
            MouseIn := false;
          end;
        end;
      end;
      GUI_LABEL    : begin
      end;
      GUI_CHECKBOX : begin
      end;
    end;
  end;
end;

procedure mGUI_Init;
begin
  managerGUI.NowLayer    := 0;
  managerGUI.LastFocused := nil;
  managerGUI.Count       := 0;
  managerGUI.GUIObj      := nil;
end;

procedure mGUI_Draw;
var i : word;
    p : pGUIObject;
begin
  with managerGUI do begin
    p := GUIObj;
    while p <> nil do begin
      if p.Layer = NowLayer then
        p.OnDraw( p );
      p := p._prew;
    end;
  end;
end;

procedure mGUI_Update;
var i : word;
    p : pGUIObject;
begin
  with managerGUI do begin
    p := GUIObj;
    while p <> nil do begin
      if p.Layer = NowLayer then
        p.OnUpdt( p );
      p := p._prew;
    end;
  end;
end;

function  GUI_AddObject( const _type : Integer; const X, Y, W, H : Single;
                         const Focus, Visible : Boolean; const Desc, Data : Pointer;
                         const next, prew : pGUIObject ) : pGUIObject;
var buf : pGUIObject;
begin
  with managerGUI do begin
    Inc( Count );
    buf := New( pGUIObject );
    buf._type    := _type;
    buf.Desc     := Desc;
    buf.Data     := Data;
    buf.Layer    := NowLayer;
    buf.Rect.X   := X;
    buf.Rect.Y   := Y;
    buf.Rect.W   := W;
    buf.Rect.H   := H;
    buf.Visible  := Visible;
    buf.Focus    := Focus;
    buf.MouseIn  := false;
    buf.OnDraw   := @DefDraw;
    buf.OnUpdt   := @DefUpdt;
    buf.GUIMan   := @managerGUI;
    buf.next_obj := next;
    buf.prew_obj := prew;
    buf.Events.OnSetFocus   := nil;
    buf.Events.OnClick      := nil;
    buf.Events.OnMouseEnter := nil;
    buf.Events.OnMouseLeave := nil;
    buf.Events.OnKeyDown    := nil;
    buf.Events.OnKeyUp      := nil;
    buf.Events.OnChange     := nil;
    if Focus then LastFocused := buf;
    buf._next := nil;
    buf._prew := GUIObj;
    GUIObj := buf;

    result := buf;
  end;
end;

end.
