unit uGame_Enemies;

interface

uses  uGlobalData,
      zglHeader,
      uGame_Map;

type  pRasp = ^tRasp;
      tRasp = class ( tMapElement )
        public
          b_point : zglTPoint2D;
          last_tm_b : single;
          frenc_b : integer;

          max_p : integer;
          nmax_p : integer;
          last_tm_mp : single;

          constructor Create ( param : pointer ); override;

          procedure   Updt ( param, param2 : pointer ); override;
        protected
      end;

type  pBullet = ^tBullet;
      tBullet = class
        tex     : zglPTexture;
        damage  : integer;
        last_tm : single;
        lx, ly  : single;
        x, y    : single;
        vx, vy  : single;
        side : boolean;
        kill : boolean;

        constructor Create ( px, py, pvx, pvy : single; dm : integer; textr : zglPTexture; sd : boolean );
        procedure Draw;
        procedure Updt;
      end;

type  pEnemy = ^tEnemy;
      tEnemy = class
        public
          kill : boolean;
          x, y: single;
          health : integer;
          last_tm : single;

          constructor Create ( px, py: single ); virtual;

          procedure   Draw; virtual;
          procedure   Updt ( param, param2: pointer ); virtual;
        protected
      end;

type  pRaspCh = ^tRaspCh;
      tRaspCh = record
        data : pRasp;
        next : pointer;
        prew : pointer;
      end;

type  pEnemyCh = ^tEnemyCh;
      tEnemyCh = record
        data : pEnemy;
        next : pointer;
        prew : pointer;
      end;

type  pBulletCh = ^tBulletCh;
      tBulletCh = record
        data : pBullet;
        next : pointer;
        prew : pointer;
      end;

implementation

uses  uGame_Buildings,
      uGame1;

////////////////////////////////////////////////////////////////////////////////

constructor tRasp.Create(param: Pointer);
begin
  b_point.X := pEmptyElemParam( param ).x;
  b_point.y := pEmptyElemParam( param ).y;

  max_p := 0;
  nmax_p := 8;

  frenc_b := round( pEmptyElemParam( param ).r );
  last_tm_b := timer_GetTicks;
end;

procedure tRasp.Updt(param, param2: Pointer);
var dt : single;
    p : pointer;
begin
  if max_p = 6 then begin
    last_tm_mp := timer_GetTicks;
    max_p := -1;
  end;

  if max_p = -1 then begin
    dt := timer_GetTicks - last_tm_mp;
    if dt > (PROD_DT*1.5) then max_p := 0;
    exit;
  end;   

  dt := timer_GetTicks - last_tm_b;
  if dt > (PROD_DT/frenc_b) then begin
    last_tm_b := timer_GetTicks;

    if param = nil then begin
    end else begin
      p := new ( pEnemyCh );
      pEnemyCh( p ).data := @tEnemy.Create;
      max_p := max_p +1;
      pEnemyCh( p ).prew := pEnemyCh(param^);
      pEnemyCh( param^ ).next := p;
      pEnemyCh( p ).next := nil;
      pEnemyCh(param^) := p;
    end;
  end;

end;

////////////////////////////////////////////////////////////////////////////////

constructor tBullet.Create( px, py, pvx, pvy : single; dm : integer; textr : zglPTexture; sd : boolean );
begin
  last_tm := timer_getTicks;

  lx := px;
  ly := py;
  x := lx;
  y := ly;
  vx := pvx;
  vy := pvy;

  damage := dm;
  tex := textr;

  side := sd;
  kill := false;
end;

procedure tBullet.Draw;
begin
{  SSprite2D_Draw( tex,
    SetX_ByCenter( x, 8 ),
    SetY_ByCenter( y, 8 ),
    8, 8, 0 );
  SSprite2D_Draw( tex,
    SetX_ByCenter( lx, 4 ),
    SetY_ByCenter( ly, 4 ),
    4, 4, 0, 96 );}
  if side then begin
    PR2D_Circle( x, y, 6, $ffffff, 255, 6, PR2D_FILL );
  end else begin
    PR2D_Circle( x, y, 6, $FF8800, 255, 6, PR2D_FILL );
  end;
end;

procedure tBullet.Updt;
var dt : single;
begin
  dt := timer_getTicks - last_tm;
  lx := x;
  ly := y;
  x := x + vx * dt/1000;
  y := y + vy * dt/1000;
end;

////////////////////////////////////////////////////////////////////////////////

constructor tEnemy.Create;
begin
  kill := false;
  health := 20;
  last_tm := timer_GetTicks;
  x:= px;
  y := py;
end;

procedure tEnemy.Draw;
begin
  PR2D_Circle( x, y, 16, $ff00ff, 255, 5, PR2D_FILL );
end;

procedure tEnemy.Updt( param, param2: pointer );
var ptr, ptr2: pointer;
    mx, my : single;
    nx, ny : single;
    mr, nr : single;
    dt : single;
    blt : zglTLine;
    col : zglTCircle;
begin

  ptr := param;
  mx := 100500;
  my := 100500;
  while ptr <> nil do begin
    nx := pBuildCh( ptr ).data.col.cX;
    ny := pBuildCh( ptr ).data.col.cY;
    nr := sqrt( sqr(x-nx) + sqr(y-ny) );
    mr := sqrt( sqr(x-mx) + sqr(y-my) );
    if nr < mr then begin
      mx := nx;
      my := ny;
    end;
    ptr := pBuildCh( ptr ).prew;
  end;

  mr := sqrt( sqr(x-mx) + sqr(y-my) );
  if mr > (ME_SIZE * 6) then begin
    nx := 1/mr;
    x := x + (mx-x)*nx*1.3;
    y := y + (my-y)*nx*1.3;
  end;

  dt := timer_GetTicks - last_tm;
  mr := sqrt( sqr(x-mx) + sqr(y-my) );
  if (dt > 2000) and (mr<(ME_SIZE * 18)) then begin
    last_tm := timer_GetTicks;
    map.AddBullet( x, y, mx, my, 4.5, 6, false );
  end;

  ptr := param2;
  col.cX := x;
  col.cY := y;
  col.Radius := 14;
  while ptr <> nil do begin
    blt.x0 := pBulletCh( ptr ).data.lx;
    blt.x1 := pBulletCh( ptr ).data.x;
    blt.y0 := pBulletCh( ptr ).data.ly;
    blt.y1 := pBulletCh( ptr ).data.y;
    if col2d_LineVsCircle( blt, col ) and
       pBulletCh(ptr).data.side then begin
      pBulletCh( ptr ).data.kill := true;
      health := health - pBulletCh( ptr ).data.damage;
      if health <= 0 then kill := true;
    end;
    ptr := pBulletCh( ptr ).prew;
  end;

end;

////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////

end.
