unit uGame_Map;

interface

uses  zglHeader,
      uGlobalData;

const   ME_SIZE = 32;
const   PROD_DT = 20000;

const   cMove : array [ 0 .. 6, 1 .. 2 ] of shortint =
          ( ( 0, 0 ), ( 0, -1 ), ( 1, -1 ), ( 1, 0 ), ( 0, 1 ), ( -1, 1 ), ( -1, 0 ) );

type  pBuildStructure = ^tBuildStructure;
      tBuildStructure = record
        count : byte;
        points : array [ 1..7, 1..2 ] of shortint;
      end;
type  tMapElementType =
        ( ME_NILL, ME_EMPTY, ME_BUILD, ME_ENEMY, ME_DEC, ME_RASP );
type  pMapElemDrawParam = ^tMapElemDrawParam;
      tMapElemDrawParam = record
        x, y : single;
        xp, yp : integer;
        w, h : single;
        alpha: byte;
        angle: single;
      end;

type  pMapElement = ^tMapElement;
      tMapElement =   class
        public
          col_crcl: zglTCircle;

          constructor Create ( param : pointer ); virtual;
          procedure   Draw ( param : pMapElemDrawParam ); virtual;
          procedure   Updt ( param, param2 : pointer ); virtual;

          function    GetType : tMapElementType;
          function    GetCols : zglPCircle;
        protected
          el_type : tMapElementType;

      end;

type  pEmptyElemParam = ^tEmptyElemParam;
      tEmptyElemParam = record
        x, y, r: single;
        tex : zglPTexture;
      end;
      pEmptyElement = ^tEmptyElement;
      tEmptyElement = class ( tMapElement )
        public
          constructor Create ( param : pointer ); override;
          procedure   Draw ( param : pMapElemDrawParam ); override;
          procedure   Updt ( param, param2 : pointer ); override;
        protected
          tex : zglPTexture;
      end;

type  pMap = ^tMap;
      tMap = class
        public
          constructor Create ( pch : string );

          procedure   Draw;
          procedure   Updt;

          procedure   SetAddMode ( struct : pBuildStructure );
          procedure   AddRasp ( ptr : pMapElement );
          procedure   AddBase ( tp : byte );
          procedure   AddEng ( tp : byte );
          procedure   AddShaxt ( tp : byte );
          procedure   AddMing ( tp : byte );
          procedure   AddRocg ( tp : byte );
          procedure   AddLazr ( tp : byte );

          procedure   AddBullet ( px,py,ptx,pty,v: single; dm:integer; sd: boolean );

          procedure   Greed ( onoff : boolean );
          procedure   ShiftMap ( dir, delta : shortint );
          procedure   SetCamera ( onoff : boolean );
          procedure   ZoomCamera ( updown : boolean );
          procedure   DefaultCamera;
          function    GetAddBuildFocused : boolean;

          function    GetTotalMet : single;
          function    GetTotalEng : single;
          function    GetBaseHealth : integer;
          function    GetProdMet : integer;
          function    PosbAddStruct ( i, j : integer ) : boolean;
        protected
          Camera      : zglTCamera2D;
          Max_X, Max_Y: single;
          Min_X, Min_Y: single;
          DMax_X, DMax_Y: single;
          DMin_X, DMin_Y: single;
          ColRect     : zglTRect;

          Elements    : array of array of tMapElement;
          Rasps       : pointer;
          Builds      : pointer;
          Enemies     : pointer;
          Bullets     : pointer;

          Width       : integer;
          FullHeight  : integer;
          Height      : integer;

          DrawGreed   : boolean;

          BStruct     : pBuildStructure;
          BS_X, BS_Y  : integer;
          AddMode     : boolean;

          Base        : pMapElement;
          TotalMet    : single;
          ProdMet     : integer;
          TotalEng    : single;
          LastTime    : single;

          kill : boolean;

          procedure   DrawStruct ( i, j : integer );
      end;

implementation

uses  uGame_Buildings,
      uGame_Enemies;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

constructor tMapElement.Create(param: Pointer);
begin
  if param = NIL then el_type := ME_NILL;
end;

procedure tMapElement.Draw(param: pMapElemDrawParam);
begin
end;

procedure tMapElement.Updt(param, param2: Pointer);
begin
end;

function tMapElement.GetType : tMapElementType;
begin
  result := el_type;
end;

function tMapElement.GetCols : zglPCircle;
begin
  result := @col_crcl;
end;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

constructor tEmptyElement.Create(param: Pointer);
begin
  el_type := ME_EMPTY;
  tex := pEmptyElemParam( param ).tex;

  col_crcl.cX := pEmptyElemParam( param ).x;
  col_crcl.cY := pEmptyElemParam( param ).y;
  col_crcl.Radius := pEmptyElemParam( param ).r;
end;

procedure tEmptyElement.Draw(param: pMapElemDrawParam);
begin
  if param = nil then exit;

  SSprite2D_Draw( tex,
    SetX_ByCenter( param.x, param.w ),
    SetY_ByCenter( param.y, param.h ),
    param.w,
    param.h,
    param.angle,
    param.alpha );

  //text_draw( font2, param.x, param.y, 'Gr' );
end;

procedure tEmptyElement.Updt(param, param2: Pointer);
begin
end;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

constructor tMap.Create;
type  tParam = record
        el_tp : tMapElementType;
        tex_nm: string;
        pod_tp: byte;
      end;
var DF, DFEX : textfile;
    str : string;
    i, j : integer;
    k, l : integer;
    params : array of tParam;

    sm_param_1 : tEmptyElemParam;
begin

  AssignFile ( DF, pch );
  Reset ( DF );

  Readln ( DF, str );
    AssignFile ( DFEX, DataFolder + str );
    Reset ( DFEX );
    Readln ( DFEX, i );
    SetLength ( params, i );
    for j := 0 to i - 1 do begin
      Readln( DFEX, k, l );
      case k of
        0: params[ j ].el_tp := ME_NILL;
        1: params[ j ].el_tp := ME_EMPTY;
        2: params[ j ].el_tp := ME_BUILD;
        3: params[ j ].el_tp := ME_ENEMY;
        4: params[ j ].el_tp := ME_DEC;
        5: params[ j ].el_tp := ME_RASP;
      end;
      params[ j ].pod_tp := l;
      Readln ( DFEX, str );
      params[ j ].tex_nm := str;
    end;
  CloseFile ( DFEX );

  Readln ( DF, Width, FullHeight );
  Height := FullHeight - ( Width - 1 ) div 2;
  SetLength ( Elements, Width, FullHeight );

  for j := 0 to FullHeight - 1 do begin
    for i := 0 to Width - 1 do begin
      Read ( DF, k );
      case params[ k ].el_tp of
        ME_NILL : begin
          Elements [ i, j ] := tMapElement.Create( nil );
        end;
        ME_EMPTY : begin
          sm_param_1.tex := Resources.GetTex( params[ k ].tex_nm );
          sm_param_1.x := ME_SIZE * i * 3/2;
          sm_param_1.y := ME_SIZE * ( j * sqrt( 3 ) + i * sqrt( 3 )/2 );
          sm_param_1.r := ME_SIZE * sqrt( 3 )/2;
          Elements [ i, j ] := tEmptyElement.Create( @sm_param_1 );
        end;
        ME_BUILD : ;
        ME_ENEMY : ;
        ME_DEC : ;
        ME_RASP : begin
          sm_param_1.x := ME_SIZE * i * 3/2;
          sm_param_1.y := ME_SIZE * ( j * sqrt( 3 ) + i * sqrt( 3 )/2 );
          sm_param_1.r := params[ k ].pod_tp;
          Elements [ i, j ] := tRasp.Create( @sm_param_1 );
          AddRasp( @Elements[ i, j ] );
        end;
      end;
    end;
    Readln ( DF );
  end;

  CloseFile ( DF );

  DMin_X := - ME_SIZE * 3;
  DMin_Y := ME_SIZE * ( FullHeight - Height ) * sqrt( 3 ) - ME_SIZE * 3;
  DMax_X := DMin_X + ( ME_SIZE * ( Width - 1 ) * 3 )/2 + ME_SIZE * 6 - DefScrW;
  DMax_Y := DMin_Y + ME_SIZE * Height * sqrt( 3 ) + ME_SIZE * 6 - DefScrH;

  DefaultCamera;

  AddMode := false;
  DrawGreed := false;

  TotalMet := 220;
  TotalEng := 0;

  Builds := nil;
//  Rasps := nil;
  Enemies := nil;
  Bullets := nil;

  kill := false;
end;

procedure tMap.Draw;
var i, j   : integer;
    xh, yh : single;
    param  : tMapElemDrawParam;
    ptr, ptr2 : pointer;
begin
  for i := 0 to Width - 1 do begin
    for j := 0 to FullHeight - 1 do begin
      if Elements [ i, j ].GetType = ME_NILL then continue;

      xh := ME_SIZE * i * 3/2;
      yh := ME_SIZE * ( j * sqrt( 3 ) + i * sqrt( 3 )/2 );

      param.x := xh;
      param.y := yh;
      param.xp := i;
      param.yp := j;
      param.w := ME_SIZE * 2;
      param.h := ME_SIZE * 2;
      param.alpha := 255;
      param.angle := 0;

      if col2d_PointInRect( xh, yh, ColRect ) then begin
        Elements [ i, j ].Draw( @param );
      end;
    end;
  end;

  for i := 0 to Width - 1 do begin
    for j := 0 to FullHeight - 1 do begin
      if Elements [ i, j ].GetType = ME_NILL then continue;
      xh := ME_SIZE * i * 3/2;
      yh := ME_SIZE * ( j * sqrt( 3 ) + i * sqrt( 3 )/2 );
      if DrawGreed and col2d_PointInRect( xh, yh, ColRect )
        then PR2D_Circle ( xh, yh, ME_SIZE, $FFFFFF, 255, 6 )
        else PR2D_Circle ( xh, yh, ME_SIZE, $000000, 32, 6 );
    end;
  end;

  if (BS_X <> -1) and AddMode then begin
    DrawStruct ( BS_X, BS_Y );
  end;

  ptr := Builds;
  while ptr <> nil do begin
    pBuild( pBuildCh( ptr ).data ).DrawInf;
    ptr := pBuildCh( ptr ).prew;
  end;

  ptr := Enemies;
  while ptr <> nil do begin
    if col2d_PointInRect( pEnemyCh( ptr ).data.x, pEnemyCh( ptr ).data.y, ColRect ) then
      pEnemy( pEnemyCh( ptr ).data ).Draw;
    ptr := pEnemyCh( ptr ).prew;
  end;

  ptr := Bullets;
  while ptr <> nil do begin
    if col2d_PointInRect( pBulletCh( ptr ).data.x, pBulletCh( ptr ).data.y, ColRect ) then
      pBullet( pBulletCh( ptr ).data ).Draw;
    ptr := pBulletCh( ptr ).prew;
  end;

  if kill then begin
    PR2D_Rect( camera.X, camera.Y + DefScrH/2-20, DefScrW, 40, $000000, 128, PR2D_FILL );
    text_Draw( font2, camera.X + DefScrW/2, camera.Y + DefScrH/2, ' !', TEXT_HALIGN_CENTER );
  end;
end;

procedure tMap.Updt;
var i, j : integer;
    ptr, ptr2 : pointer;
    dt, dt2 : single;
    enm : pEnemy;
    k : integer;
    tx, ty : integer;
begin
  ColRect.X := Camera.X - ME_SIZE*3 - 15;
  ColRect.Y := Camera.Y - ME_SIZE*3 - 15;
  ColRect.W := DefScrW + ME_SIZE*6 + 30;
  ColRect.H := DefScrH + ME_SIZE*6 + 30;

  if AddMode then begin
    BS_X := -1;
    BS_Y := -1;

    For i := 0 to Width - 1 do begin
      For j := 0 to FullHeight - 1 do begin
        if Elements [ i, j ].GetType = ME_EMPTY then begin
          if col2d_PointInCircle ( Camera.X + Mouse_X * Camera.Zoom.X,
                                   Camera.Y + Mouse_Y * Camera.Zoom.X,
                                   Elements [ i, j ].GetCols^ )
          then begin
            if PosbAddStruct( i, j ) then begin
              BS_X := i;
              BS_Y := j;
            end;
          end;
        end;
      end;
    end;

  end;

  if (Base <> nil) and not kill then begin
    {ptr := Builds;
    while ptr <> nil do begin
      case pBuild( pBuildCh( ptr ).data ).GetBuildType of
        BD_BASE: begin
          pBuildBase( pBuildCh( ptr ).data ).Updt( nil, nil );
        end;
        BD_ENG: begin
          pBuildEng( pBuildCh( ptr ).data ).Updt( nil, nil );
        end;
        BD_SHAXT: begin
          pBuildShaxt( pBuildCh( ptr ).data ).Updt( nil, nil );
        end;
        BD_MING: begin
          pBuildMing( pBuildCh( ptr ).data ).Updt( Bullets, Enemies );
        end;
        BD_ROCG: begin
          pBuildRocg( pBuildCh( ptr ).data ).Updt( Bullets, Enemies );
        end;
        BD_LAZR: begin
          pBuildLazr( pBuildCh( ptr ).data ).Updt( Bullets, Enemies );
        end;
      end;
      ptr := pBuildCh( ptr ).prew;
    end; }

    if pBuild( Base ).GetHealth <= 0 then begin
      kill := true;
    end;

    dt := timer_GetTicks - LastTime;
    LastTime := timer_GetTicks;

    ProdMet := 0;

    ptr := Rasps;
    while ptr <> nil do begin
      if enemies = nil then begin
        enemies := new ( pEnemyCh );
        enm := new( pEnemy );
        enm^ := tEnemy.Create( pRaspCh(ptr).data.b_point.X, pRaspCh(ptr).data.b_point.Y );
        inc( pRaspCh(ptr).data.max_p );
        pEnemyCh( enemies ).data := enm;
        pEnemyCh( enemies ).prew := nil;
        pEnemyCh( enemies ).next := nil;
      end else begin
        if pRaspCh(ptr).data.max_p = 9 then begin
          pRaspCh(ptr).data.last_tm_mp := timer_GetTicks;
          pRaspCh(ptr).data.max_p := -1;
        end;
        if pRaspCh(ptr).data.max_p = -1 then begin
          dt2 := timer_GetTicks - pRaspCh(ptr).data.last_tm_mp;
          if dt2 > (PROD_DT*3) then begin
            pRaspCh(ptr).data.max_p := 0;
            inc( pRaspCh(ptr).data.nmax_p, 2 );
          end;
          ptr := pRaspCh( ptr ).prew;
          continue;
        end;

        dt2 := timer_GetTicks - pRaspCh(ptr).data.last_tm_b;
        if dt2 > (PROD_DT/pRaspCh(ptr).data.frenc_b) then begin
          pRaspCh(ptr).data.last_tm_b := timer_GetTicks;

          ptr2 := new ( pEnemyCh );
          enm := new( pEnemy );
          enm^ := tEnemy.Create( pRaspCh(ptr).data.b_point.X, pRaspCh(ptr).data.b_point.Y );
          inc( pRaspCh(ptr).data.max_p );
          pEnemyCh( ptr2 ).data := enm;
          pEnemyCh( ptr2 ).prew := enemies;
          pEnemyCh( enemies ).next := ptr2;
          pEnemyCh( ptr2 ).next := nil;
          enemies := ptr2;
        end;
      end;
      ptr := pRaspCh( ptr ).prew;
    end;

    ptr := Builds;
    while ptr <> nil do begin
      if pBuild( pBuildCh( ptr ).data ).GetUseMet <> 0 then begin
        TotalMet := totalMet - pBuild( pBuildCh( ptr ).data ).GetUseMet * dt / PROD_DT;
        ProdMet := ProdMet - pBuild( pBuildCh( ptr ).data ).GetUseMet;
      end;
      pBuildCh( ptr ).data.Updt( bullets, enemies );
      if pBuildCh( ptr ).data.kill then begin
        if pBuildCh( ptr ).prew <> nil
          then pBuildCh( pBuildCh( ptr ).prew ).next := pBuildCh( ptr ).next;
        if pBuildCh( ptr ).next <> nil
          then pBuildCh( pBuildCh( ptr ).next ).prew := pBuildCh( ptr ).prew;
        if ptr = Builds then Builds := pBuildCh( ptr ).prew;

        for k := 1 to pBuildCh( ptr ).data.structure.count do begin
          tx :=    pBuildCh(ptr).data.mx;
          ty :=    pBuildCh(ptr).data.my;
          tx :=    tx+ pBuildCh( ptr ).data.structure.points[ k, 1 ];
          ty :=    ty+ pBuildCh( ptr ).data.structure.points[ k, 2 ];          

          Elements[ tx, ty ] := Elements[ 0, 0 ];
          Elements[ tx, ty ].col_crcl.cX := ME_SIZE * tx * 3/2;
          Elements[ tx, ty ].col_crcl.cY := ME_SIZE * ( ty * sqrt( 3 ) + tx * sqrt( 3 )/2 );
          Elements[ tx, ty ].col_crcl.Radius := ME_SIZE * sqrt( 3 )/2;
        end;

        ptr2:= ptr;
        ptr := pBuildCh( ptr ).prew;
        Dispose( ptr2 );
      end else ptr := pBuildCh( ptr ).prew;
    end;

    ptr := Enemies;
    while ptr <> nil do begin
      pEnemy( pEnemyCh( ptr ).data ).Updt( Builds, Bullets );
      if pEnemyCh( ptr ).data.kill then begin
        if pEnemyCh( ptr ).prew <> nil
          then pEnemyCh( pEnemyCh( ptr ).prew ).next := pEnemyCh( ptr ).next;
        if pEnemyCh( ptr ).next <> nil
          then pEnemyCh( pEnemyCh( ptr ).next ).prew := pEnemyCh( ptr ).prew;
        if ptr = Enemies then Enemies := pBuildCh( ptr ).prew;
        ptr2:= ptr;
        ptr := pEnemyCh( ptr ).prew;
        Dispose( ptr2 );
      end else ptr := pEnemyCh( ptr ).prew;
    end;

    ptr := Bullets;
    while ptr <> nil do begin
      pBullet( pBulletCh( ptr ).data ).Updt;
      if pBulletCh( ptr ).data.kill then begin
        if pBulletCh( ptr ).prew <> nil
          then pBulletCh( pBulletCh( ptr ).prew ).next := pBulletCh( ptr ).next;
        if pBulletCh( ptr ).next <> nil
          then pBulletCh( pBulletCh( ptr ).next ).prew := pBulletCh( ptr ).prew;
        if ptr = Bullets then Bullets := pBuildCh( ptr ).prew;          
        ptr2:= ptr;
        ptr := pBulletCh( ptr ).prew;
        Dispose( ptr2 );
      end else ptr := pBulletCh( ptr ).prew;
    end;

  end;

end;

procedure tMap.ShiftMap( dir, delta: shortint );
begin
  case dir of
    1: Camera.X := Camera.X - delta;
    2: Camera.Y := Camera.Y - delta;
    3: Camera.X := Camera.X + delta;
    4: Camera.Y := Camera.Y + delta;
  end;

  If Camera.X < Min_X then Camera.X := Min_X;
  If Camera.X > Max_X then Camera.X := Max_X;
  If Camera.Y < Min_Y then Camera.Y := Min_Y;
  If Camera.Y > Max_Y then Camera.Y := Max_Y;
end;

procedure tMap.Greed(onoff: Boolean);
begin
  DrawGreed := onoff;
end;

procedure tMap.SetCamera( onoff: boolean );
begin
  if onoff then cam2d_set ( @camera )
    else cam2d_set ( nil );
end;

procedure tMap.ZoomCamera(updown: Boolean);
begin
  if updown then begin
    camera.Zoom.X := camera.Zoom.X * 1.1;
    camera.Zoom.Y := camera.Zoom.Y * 1.1;
  end else begin
    camera.Zoom.X := camera.Zoom.X * 0.9;
    camera.Zoom.Y := camera.Zoom.Y * 0.9;
  end;
  if camera.Zoom.X > 2 then begin
    camera.Zoom.X := 2;
    camera.Zoom.Y := 2;
  end;
  if camera.Zoom.X < 0.3 then begin
    camera.Zoom.X := 0.3;
    camera.Zoom.Y := 0.3;
  end;
end;

procedure tMap.DefaultCamera;
begin
  Camera.X := ME_SIZE * ( Width - 1 ) * 3/4 - DefScrW/2;
  Camera.Y := ME_SIZE * ( FullHeight - Height/2 ) * sqrt( 3 ) - DefScrH/2;
  Camera.Angle := 0;
  Camera.Zoom.X := 1;
  Camera.Zoom.Y := 1;
  cam2d_Set( @Camera );

  Min_X := DMin_X;
  Min_Y := DMin_Y;
  Max_X := DMax_X;
  Max_Y := DMax_Y;
end;

procedure tMap.SetAddMode(struct: pBuildStructure);
begin
  if struct = nil then begin
    AddMode := false;
    exit;
  end;
  BStruct := struct;
  AddMode := true;
  BS_X := -1;
  BS_Y := -1;
end;

procedure tMap.AddBase(tp: Byte);
var param : tBuildBaseParam;
begin
  AddMode := false;
  case tp of
    1: begin
      param.x := BS_X;
      param.y := BS_Y;
      param.r := ME_SIZE;
      param.tp := tp;
      param.tex1 := 'elem_bld_bs1';
      param.str := cPosBuilds [ 1 ];

      Elements [ BS_X, BS_Y ] := tBuildBase.Create( @param );
      Base := @Elements [ BS_X, BS_Y ];
      Totalmet := totalmet - pBuild( base ).GetPrice;
      Totaleng := totaleng - pBuild( base ).GetUseEng;
    end;
    2: begin
      param.x := BS_X;
      param.y := BS_Y;
      param.r := ME_SIZE;
      param.tp := tp;
      param.tex1 := 'elem_bld_bs2';
      param.tex2 := 'elem_bld_bs3';
      param.tex3 := 'elem_bld_bs4';
      param.str := cPosBuilds [ 4 ];

      Elements [ BS_X, BS_Y ] := tBuildBase.Create( @param );
      Elements [ BS_X + cPosBuilds [ 4 ].points[ 2, 1 ],
                 BS_Y + cPosBuilds [ 4 ].points[ 2, 2 ] ] := tBuild.Create( nil );
      Elements [ BS_X + cPosBuilds [ 4 ].points[ 3, 1 ],
                 BS_Y + cPosBuilds [ 4 ].points[ 3, 2 ] ] := tBuild.Create( nil );
      Base := @Elements [ BS_X, BS_Y ];
      Totalmet := totalmet - pBuild( base ).GetPrice;
      Totaleng := totaleng - pBuild( base ).GetUseEng;
    end;
    3: begin
      param.x := BS_X;
      param.y := BS_Y;
      param.r := ME_SIZE;
      param.tp := tp;
      param.tex1 := 'elem_bld_bs2';
      param.tex2 := 'elem_bld_bs3';
      param.tex3 := 'elem_bld_bs4';
      param.tex4 := 'elem_bld_bs5';
      param.str := cPosBuilds [ 5 ];

      Elements [ BS_X, BS_Y ] := tBuildBase.Create( @param );
      Elements [ BS_X + cPosBuilds [ 5 ].points[ 2, 1 ],
                 BS_Y + cPosBuilds [ 5 ].points[ 2, 2 ] ] := tBuild.Create( nil );
      Elements [ BS_X + cPosBuilds [ 5 ].points[ 3, 1 ],
                 BS_Y + cPosBuilds [ 5 ].points[ 3, 2 ] ] := tBuild.Create( nil );
      Elements [ BS_X + cPosBuilds [ 5 ].points[ 4, 1 ],
                 BS_Y + cPosBuilds [ 5 ].points[ 4, 2 ] ] := tBuild.Create( nil );
      Base := @Elements [ BS_X, BS_Y ];
      Totalmet := totalmet - pBuild( base ).GetPrice;
      Totaleng := totaleng - pBuild( base ).GetUseEng;
    end;
  end;

  Builds := new ( pBuildCh );
  pBuildCh( Builds ).data := @Elements [ BS_X, BS_Y ];
  pBuildCh( Builds ).prew := nil;
  pBuildCh( Builds ).next := nil;

  LastTime := timer_GetTicks;
end;

procedure tMap.AddEng(tp: Byte);
var param : tBuildEngParam;
var ptr : pointer;
begin
  AddMode := false;
  case tp of
    1: begin
      param.x := BS_X;
      param.y := BS_Y;
      param.r := ME_SIZE;
      param.tp := tp;
      param.tex1 := 'elem_bld_eng1';
      param.str := cPosBuilds [ 1 ];

      Elements [ BS_X, BS_Y ] := tBuildEng.Create( @param );
    end;
    2: begin
      param.x := BS_X;
      param.y := BS_Y;
      param.r := ME_SIZE;
      param.tp := tp;
      param.tex1 := 'elem_bld_eng2';
      param.tex2 := 'elem_bld_eng3';
      param.tex3 := 'elem_bld_eng4';
      param.str := cPosBuilds [ 4 ];

      Elements [ BS_X, BS_Y ] := tBuildEng.Create( @param );
      Elements [ BS_X + cPosBuilds [ 4 ].points[ 2, 1 ],
                 BS_Y + cPosBuilds [ 4 ].points[ 2, 2 ] ] := tBuild.Create( nil );
      Elements [ BS_X + cPosBuilds [ 4 ].points[ 3, 1 ],
                 BS_Y + cPosBuilds [ 4 ].points[ 3, 2 ] ] := tBuild.Create( nil );
    end;
    3: begin
      param.x := BS_X;
      param.y := BS_Y;
      param.r := ME_SIZE;
      param.tp := tp;
      param.tex1 := 'elem_bld_eng5';
      param.tex2 := 'elem_bld_eng6';
      param.tex3 := 'elem_bld_eng7';
      param.tex4 := 'elem_bld_eng8';
      param.tex5 := 'elem_bld_eng9';
      param.tex6 := 'elem_bld_eng10';
      param.tex7 := 'elem_bld_eng11';
      param.str := cPosBuilds [ 6 ];

      Elements [ BS_X, BS_Y ] := tBuildEng.Create( @param );
      Elements [ BS_X + cPosBuilds [ 6 ].points[ 2, 1 ],
                 BS_Y + cPosBuilds [ 6 ].points[ 2, 2 ] ] := tBuild.Create( nil );
      Elements [ BS_X + cPosBuilds [ 6 ].points[ 3, 1 ],
                 BS_Y + cPosBuilds [ 6 ].points[ 3, 2 ] ] := tBuild.Create( nil );
      Elements [ BS_X + cPosBuilds [ 6 ].points[ 4, 1 ],
                 BS_Y + cPosBuilds [ 6 ].points[ 4, 2 ] ] := tBuild.Create( nil );
      Elements [ BS_X + cPosBuilds [ 6 ].points[ 5, 1 ],
                 BS_Y + cPosBuilds [ 6 ].points[ 5, 2 ] ] := tBuild.Create( nil );
      Elements [ BS_X + cPosBuilds [ 6 ].points[ 6, 1 ],
                 BS_Y + cPosBuilds [ 6 ].points[ 6, 2 ] ] := tBuild.Create( nil );
      Elements [ BS_X + cPosBuilds [ 6 ].points[ 7, 1 ],
                 BS_Y + cPosBuilds [ 6 ].points[ 7, 2 ] ] := tBuild.Create( nil );
    end;
  end;

  Totalmet := totalmet - tBuild( Elements [ BS_X, BS_Y ] ).GetPrice;
  Totaleng := totaleng - tBuild( Elements [ BS_X, BS_Y ] ).GetUseEng;

  ptr := new ( pBuildCh );
  pBuildCh( ptr ).data := @Elements [ BS_X, BS_Y ];
  pBuildCh( ptr ).prew := Builds;
  pBuildCh( builds ).next := ptr;
  pBuildCh( ptr ).next := nil;
  builds := ptr;
end;

procedure tMap.AddShaxt(tp: Byte);
var param : tBuildShaxtParam;
var ptr : pointer;
begin
  AddMode := false;
  case tp of
    1: begin
      param.x := BS_X;
      param.y := BS_Y;
      param.r := ME_SIZE;
      param.tp := tp;
      param.tex1 := 'elem_bld_shaxt1';
      param.str := cPosBuilds [ 1 ];

      Elements [ BS_X, BS_Y ] := tBuildShaxt.Create( @param );
    end;
    2: begin
      param.x := BS_X;
      param.y := BS_Y;
      param.r := ME_SIZE;
      param.tp := tp;
      param.tex1 := 'elem_bld_shaxt2';
      param.tex2 := 'elem_bld_shaxt3';
      param.tex3 := 'elem_bld_shaxt4';
      param.str := cPosBuilds [ 4 ];

      Elements [ BS_X, BS_Y ] := tBuildShaxt.Create( @param );
      Elements [ BS_X + cPosBuilds [ 4 ].points[ 2, 1 ],
                 BS_Y + cPosBuilds [ 4 ].points[ 2, 2 ] ] := tBuild.Create( nil );
      Elements [ BS_X + cPosBuilds [ 4 ].points[ 3, 1 ],
                 BS_Y + cPosBuilds [ 4 ].points[ 3, 2 ] ] := tBuild.Create( nil );
    end;
    3: begin
      param.x := BS_X;
      param.y := BS_Y;
      param.r := ME_SIZE;
      param.tp := tp;
      param.tex1 := 'elem_bld_shaxt5';
      param.tex2 := 'elem_bld_shaxt6';
      param.tex3 := 'elem_bld_shaxt7';
      param.tex4 := 'elem_bld_shaxt8';
      param.tex5 := 'elem_bld_shaxt9';
      param.tex6 := 'elem_bld_shaxt10';
      param.str := cPosBuilds [ 7 ];

      Elements [ BS_X, BS_Y ] := tBuildShaxt.Create( @param );
      Elements [ BS_X + cPosBuilds [ 7 ].points[ 2, 1 ],
                 BS_Y + cPosBuilds [ 7 ].points[ 2, 2 ] ] := tBuild.Create( nil );
      Elements [ BS_X + cPosBuilds [ 7 ].points[ 3, 1 ],
                 BS_Y + cPosBuilds [ 7 ].points[ 3, 2 ] ] := tBuild.Create( nil );
      Elements [ BS_X + cPosBuilds [ 7 ].points[ 4, 1 ],
                 BS_Y + cPosBuilds [ 7 ].points[ 4, 2 ] ] := tBuild.Create( nil );
      Elements [ BS_X + cPosBuilds [ 7 ].points[ 5, 1 ],
                 BS_Y + cPosBuilds [ 7 ].points[ 5, 2 ] ] := tBuild.Create( nil );
      Elements [ BS_X + cPosBuilds [ 7 ].points[ 6, 1 ],
                 BS_Y + cPosBuilds [ 7 ].points[ 6, 2 ] ] := tBuild.Create( nil );
    end;
  end;

  Totalmet := totalmet - tBuild( Elements [ BS_X, BS_Y ] ).GetPrice;
  Totaleng := totaleng - tBuild( Elements [ BS_X, BS_Y ] ).GetUseEng;

  ptr := new ( pBuildCh );
  pBuildCh( ptr ).data := @Elements [ BS_X, BS_Y ];
  pBuildCh( ptr ).prew := Builds;
  pBuildCh( builds ).next := ptr;
  pBuildCh( ptr ).next := nil;
  builds := ptr;
end;

procedure tMap.AddMing(tp: Byte);
var param : tBuildMingParam;
var ptr : pointer;
begin
  AddMode := false;
  case tp of
    1: begin
      param.x := BS_X;
      param.y := BS_Y;
      param.r := ME_SIZE;
      param.tp := tp;
      param.tex1 := 'elem_bld_ming';
      param.str := cPosBuilds [ 1 ];

      Elements [ BS_X, BS_Y ] := tBuildMing.Create( @param );
    end;
    2: begin
      param.x := BS_X;
      param.y := BS_Y;
      param.r := ME_SIZE;
      param.tp := tp;
      param.tex1 := 'elem_bld_ming';
      param.str := cPosBuilds [ 1 ];

      Elements [ BS_X, BS_Y ] := tBuildMing.Create( @param );
    end;
    3: begin
      param.x := BS_X;
      param.y := BS_Y;
      param.r := ME_SIZE;
      param.tp := tp;
      param.tex1 := 'elem_bld_ming';
      param.tex2 := 'elem_bld_ming';
      param.tex3 := 'elem_bld_ming';
      param.str := cPosBuilds [ 4 ];

      Elements [ BS_X, BS_Y ] := tBuildMing.Create( @param );
      Elements [ BS_X + cPosBuilds [ 4 ].points[ 2, 1 ],
                 BS_Y + cPosBuilds [ 4 ].points[ 2, 2 ] ] := tBuild.Create( nil );
      Elements [ BS_X + cPosBuilds [ 4 ].points[ 3, 1 ],
                 BS_Y + cPosBuilds [ 4 ].points[ 3, 2 ] ] := tBuild.Create( nil );
    end;
  end;

  Totalmet := totalmet - tBuild( Elements [ BS_X, BS_Y ] ).GetPrice;
  Totaleng := totaleng - tBuild( Elements [ BS_X, BS_Y ] ).GetUseEng;

  ptr := new ( pBuildCh );
  pBuildCh( ptr ).data := @Elements [ BS_X, BS_Y ];
  pBuildCh( ptr ).prew := Builds;
  pBuildCh( builds ).next := ptr;
  pBuildCh( ptr ).next := nil;
  builds := ptr;
end;

procedure tMap.AddRocg(tp: Byte);
var param : tBuildRocgParam;
var ptr : pointer;
begin
  AddMode := false;
  case tp of
    1: begin
      param.x := BS_X;
      param.y := BS_Y;
      param.r := ME_SIZE;
      param.tp := tp;
      param.tex1 := 'elem_bld_rocg';
      param.str := cPosBuilds [ 1 ];

      Elements [ BS_X, BS_Y ] := tBuildRocg.Create( @param );
    end;
    2: begin
      param.x := BS_X;
      param.y := BS_Y;
      param.r := ME_SIZE;
      param.tp := tp;
      param.tex1 := 'elem_bld_rocg';
      param.tex2 := 'elem_bld_rocg';
      param.str := cPosBuilds [ 2 ];

      Elements [ BS_X, BS_Y ] := tBuildRocg.Create( @param );
      Elements [ BS_X + cPosBuilds [ 2 ].points[ 2, 1 ],
                 BS_Y + cPosBuilds [ 2 ].points[ 2, 2 ] ] := tBuild.Create( nil );
    end;
    3: begin
      param.x := BS_X;
      param.y := BS_Y;
      param.r := ME_SIZE;
      param.tp := tp;
      param.tex1 := 'elem_bld_rocg';
      param.tex2 := 'elem_bld_rocg';
      param.tex3 := 'elem_bld_rocg';
      param.str := cPosBuilds [ 3 ];

      Elements [ BS_X, BS_Y ] := tBuildRocg.Create( @param );
      Elements [ BS_X + cPosBuilds [ 3 ].points[ 2, 1 ],
                 BS_Y + cPosBuilds [ 3 ].points[ 2, 2 ] ] := tBuild.Create( nil );
      Elements [ BS_X + cPosBuilds [ 3 ].points[ 3, 1 ],
                 BS_Y + cPosBuilds [ 3 ].points[ 3, 2 ] ] := tBuild.Create( nil );
    end;
  end;

  Totalmet := totalmet - tBuild( Elements [ BS_X, BS_Y ] ).GetPrice;
  Totaleng := totaleng - tBuild( Elements [ BS_X, BS_Y ] ).GetUseEng;

  ptr := new ( pBuildCh );
  pBuildCh( ptr ).data := @Elements [ BS_X, BS_Y ];
  pBuildCh( ptr ).prew := Builds;
  pBuildCh( builds ).next := ptr;
  pBuildCh( ptr ).next := nil;
  builds := ptr;
end;

procedure tMap.AddLazr(tp: Byte);
var param : tBuildLazrParam;
var ptr : pointer;
begin
  AddMode := false;
  case tp of
    1: begin
      param.x := BS_X;
      param.y := BS_Y;
      param.r := ME_SIZE;
      param.tp := tp;
      param.tex1 := 'elem_bld_lazr';
      param.str := cPosBuilds [ 1 ];

      Elements [ BS_X, BS_Y ] := tBuildLazr.Create( @param );
    end;
    2: begin
      param.x := BS_X;
      param.y := BS_Y;
      param.r := ME_SIZE;
      param.tp := tp;
      param.tex1 := 'elem_bld_lazr';
      param.str := cPosBuilds [ 1 ];

      Elements [ BS_X, BS_Y ] := tBuildLazr.Create( @param );
    end;
    3: begin
      param.x := BS_X;
      param.y := BS_Y;
      param.r := ME_SIZE;
      param.tp := tp;
      param.tex1 := 'elem_bld_lazr';
      param.str := cPosBuilds [ 1 ];

      Elements [ BS_X, BS_Y ] := tBuildLazr.Create( @param );
    end;
  end;

  Totalmet := totalmet - tBuild( Elements [ BS_X, BS_Y ] ).GetPrice;
  Totaleng := totaleng - tBuild( Elements [ BS_X, BS_Y ] ).GetUseEng;

  ptr := new ( pBuildCh );
  pBuildCh( ptr ).data := @Elements [ BS_X, BS_Y ];
  pBuildCh( ptr ).prew := Builds;
  pBuildCh( builds ).next := ptr;
  pBuildCh( ptr ).next := nil;
  builds := ptr;
end;

procedure tMap.DrawStruct(i: Integer; j: Integer);
var k : integer;
    xh, yh : single;
begin
  For k := 1 to BStruct.count do begin
    xh := ME_SIZE * (i + BStruct.points[ k, 1 ]) * 3/2;
    yh := ME_SIZE * ( (j + BStruct.points[ k, 2 ]) * sqrt( 3 ) + (i + BStruct.points[ k, 1 ]) * sqrt( 3 )/2 );

    PR2D_Circle ( xh, yh, ME_SIZE, $FF0000, 128, 6, PR2D_FILL );
  end;
end;

function tMap.PosbAddStruct(i: Integer; j: Integer): boolean;
var k : integer;
begin
  result := true;
  for k := 1 to BStruct.count do begin
    if ( (BStruct.points[ k, 1 ] + i) > -1 ) and
       ( (BStruct.points[ k, 1 ] + i) < Width ) and
       ( (BStruct.points[ k, 2 ] + j) > -1 ) and
       ( (BStruct.points[ k, 2 ] + j) < FullHeight ) and
       ( Elements [ BStruct.points[ k, 1 ] + i, BStruct.points[ k, 2 ] + j ].GetType = ME_EMPTY ) 
       then continue
       else begin
         result := false;
         exit;
       end;    
  end;      
end;

function tMap.GetTotalMet : single;
begin
  result := TotalMet;
end;

function tMap.GetTotalEng : single;
begin
  result := totalEng;
end;

function tMap.GetBaseHealth : integer;
begin
  result := pBuild( base ).GetHealth;
end;

function tMap.GetAddBuildFocused;
begin
  if BS_X <> -1 then result := true
    else result := false;
end;

function tMap.GetProdMet;
begin
  result := ProdMet;
end;

procedure tMap.AddRasp(ptr: pMapElement);
var p : pointer;
begin
  if Rasps = nil then begin
    Rasps := new ( pRaspCh );
    pRaspCh( Rasps ).data := pRasp( ptr );
    pRaspCh( Rasps ).prew := nil;
    pRaspCh( Rasps ).next := nil;
  end else begin
    p := new ( pBuildCh );
    pRaspCh( p ).data := pRasp( ptr );
    pRaspCh( p ).prew := Rasps;
    pRaspCh( rasps ).next := p;
    pRaspCh( p ).next := nil;
    rasps := p;
  end;
end;

procedure tMap.AddBullet(px: Single; py: Single; ptx: Single; pty: Single; v: Single; dm : integer; sd: Boolean);
var blt : pBullet;
    p : pointer;
    dr : single;
    pvx, pvy : single;
begin
  if Bullets = nil then begin
    Bullets := new ( pBulletCh );
    blt := new( pBullet );
    dr := sqrt( sqr(ptx-px)+sqr(pty-py) );
    dr := v*1/dr;
    pvx := dr*(ptx-px);
    pvy := dr*(pty-py);
    blt^ := tBullet.Create( px, py, pvx, pvy, dm, nil, sd );
    pBulletCh( Bullets ).data := blt;
    pBulletCh( Bullets ).prew := nil;
    pBulletCh( Bullets ).next := nil;
  end else begin
    p := new ( pBulletCh );
    blt := new( pBullet );
    dr := sqrt( sqr(ptx-px)+sqr(pty-py) );
    dr := v*1/dr;
    pvx := dr*(ptx-px);
    pvy := dr*(pty-py);
    blt^ := tBullet.Create( px, py, pvx, pvy, dm, nil, sd );
    pBulletCh( p ).data := blt;
    pBulletCh( p ).prew := Bullets;
    pBulletCh( Bullets ).next := p;
    pBulletCh( p ).next := nil;
    Bullets := p;
  end;
end;

end.
