unit uINI_Files;
//    INI 
//      
//   
//    
//     [ _ ]
//  : _ = 
//       .ini

interface
//    ,   )

type  pFile = ^textfile;

      tKey = record
        Name: String;
        Value: String;
      end;

      tSection = record
        Name: String;
        CountKeys: integer;
        Keys: array of tKey;
      end;

      pINI = ^tINI;
      tINI = class
        public
          constructor Create;

          procedure INI_LoadFromFile( _FileName: string );
          procedure INI_SaveToFile  ( FileName: string );
          procedure INI_Free;
          procedure INI_Add          ( Section, Key: string );
          function  INI_ReadKeyInt   ( Section, Key: string ):integer;
          function  INI_ReadKeyStr   ( Section, Key: string ):string;
          function  INI_ReadKeyFloat ( Section, Key: string ):real;
          function  INI_ReadKeyBol   ( Section, Key: string ):boolean;
          procedure INI_WriteKeyInt  ( Section, Key: string; Value: integer );
          procedure INI_WriteKeyStr  ( Section, Key: string; Value: string );
          procedure INI_WriteKeyFloat( Section, Key: string; Value: real );
          procedure INI_WriteKeyBol  ( Section, Key: string; Value: boolean );
        protected
          FileName     : String;
          CountSections: integer;
          Sections     : array of tSection;
          DF: textfile;

          procedure UpCaseAll;
      end;

//var INIFile: tINI;

implementation

uses SysUtils;

//var DF: textfile;

//       Symbol   
function ReadUp( Symbol: char; DF: pFile ) : string;
var Str : string;
    Symb: string[1];
begin
 Str :='';
 Symb:='';
 repeat
  Str:=Str+Symb;
  if (not EoLn( DF^ )) or (not EoF( DF^ )) then Read( DF^, Symb )
   else Break;
 until Symb = Symbol;
 result:=Str;
end;

//           
//    
function ReadToEnd( DF : pFile ) : string;
var Str : string;
    Symb: string[1];
begin
 Str :='';
 while not EoLn( DF^ ) do begin
  Read( DF^, Symb );
  Str:=Str+Symb;
 end;
 if not EoF( DF^ ) then Readln( DF^ );
 result:=Str;
end;

//    :
// Cond(  ) = true,       Symbol,
////       Symbol
// Cond(  ) = false,       Symbol,
////      
function ReadWhile( Cond: boolean; Symbol: char; DF : pFile ) : string;
var Str : string;
    Symb: string[1];
begin
 If Cond then begin
  Symb:='';
  repeat
   Read( DF^, Symb );
  until (Symb <> Symbol) or EoLn( DF^ ) or EoF( DF^ );
  Result:=Symb;
  if (EoLn( DF^ ) or EoF( DF^ )) and (Symb=Symbol) then result:='';
 end else begin
  Str :='';
  Symb:='';
  repeat
   Str:=Str+Symb;
   Read( DF^, Symb );
  until (Symb = Symbol) or EoLn( DF^ ) or EoF( DF^ );
  Result:=Str;
 end;
end;

function UpCaseStr( Str: string ) : string;
var k: integer;
begin
 For k:=1 to Length( Str ) do
  Str[ k ]:=UpCase( Str[ k ] );
 Result:=Str;
end;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

constructor tINI.Create;
begin
  FileName := '';
  CountSections:= 0;
  SetLength( Sections, CountSections );
end;

procedure tINI.INI_LoadFromFile( _FileName: string );
label L1;
var Str : string;
    Symb: string[1];
begin
//  
 INI_Free;
 FileName := _FileName;
// 
 AssignFile( DF, FileName );
 Reset( DF );
 Str:=''; Symb:='';
 While true do begin
//  
  if Symb='' then ReadWhile( false, '[', @DF );
  Str:=ReadWhile( true, ' ', @DF );
  Symb:='';
  repeat
   Str:=Str+Symb;
   Read( DF, Symb );
  until (Symb=' ') or (Symb=']');
//  
  Inc( CountSections );
  SetLength( Sections, CountSections );
  Sections[ CountSections-1 ].CountKeys:=0;
  Sections[ CountSections-1 ].Name:=Str;
  If EoF( DF ) then Break else Readln( DF );
// 
  While true do begin
   Str:=''; Symb:='';
   if EoF( DF ) then Break else Read( DF, Symb );
   if Symb='[' then Break;
   if Symb='' then begin
    Readln( DF );
    Continue;
   end;
   if Symb=' ' then begin
    Symb:=ReadWhile( true, ' ', @DF );
    if Symb='' then Readln( DF ) else GoTo L1;
   end else
    L1: With Sections[ CountSections-1 ] do begin
     Str:=Symb;
     Symb:='';
     repeat
      Str:=Str+Symb;
      Read( DF, Symb );
     until (Symb=' ') or (Symb='=');
     INC( CountKeys );
     SetLength( Keys, CountKeys );
     Keys[ CountKeys-1 ].Name:=Str;

     if Symb=' ' then ReadUp( '=', @DF );
     Str:=''; Symb:='';
     Symb:=ReadWhile( true, ' ', @DF );
     Str:=Symb+ReadToEnd( @DF );
     Keys[ CountKeys-1 ].Value:=Str;
    end;
  end;
  If EoF( DF ) then Break;
 end;
 CloseFile( DF );
 UpCaseAll;
end;

procedure tINI.INI_SaveToFile( FileName: string );
var i, j: integer;
begin
 AssignFile( DF, FileName );
 Rewrite( DF );
 For i:=0 to CountSections-1 do begin
  Writeln( DF, '[ '+Sections[ i ].Name+' ]' );
  For j:=0 to Sections[ i ].CountKeys-1 do
   Writeln( DF, Sections[ i ].Keys[ j ].Name+' = '+Sections[ i ].Keys[ j ].Value );
  Writeln( DF );
 end;
 CloseFile( DF );
end;

procedure tINI.INI_Free;
begin
 FileName:='';
 CountSections:=0;
 SetLength( Sections, CountSections );
end;

procedure tINI.INI_Add( Section, Key: string );
var i, j: integer;
    tb  : boolean;
begin
 tb:=false;
 Section:=UpCaseStr( Section );
 Key:=UpCaseStr( Key );
 For i:=0 to CountSections-1 do
  if Sections[ i ].Name=Section then begin
   For j:=0 to Sections[ i ].CountKeys-1 do
    if Sections[ i ].Keys[ j ].Name=Key then tb:=true;
   if not tb then begin
    INC( Sections[ i ].CountKeys );
    SetLength( Sections[ i ].Keys, Sections[ i ].CountKeys );
    Sections[ i ].Keys[ Sections[ i ].CountKeys-1 ].Name:=Key;
    tb:=true;
   end;
 end;
 if not tb then begin
  INC( CountSections);
  SetLength( Sections, CountSections );
  Sections[ CountSections-1 ].Name:=Section;
  Sections[ CountSections-1 ].CountKeys:=1;
  SetLength( Sections[ CountSections-1 ].Keys, 1 );
  Sections[ CountSections-1 ].Keys[ 0 ].Name:=Key;
 end;
end;

function  tINI.INI_ReadKeyInt( Section, Key: string ):integer;
label L1;
const Chisla : set of char = [ '0', '1', '2', '3', '4', '5', '6', '7', '8', '9' ];
var i, j, k: integer;
    Str: string;
begin
 Result:=0;
 Section:=UpCaseStr( Section );
 Key:=UpCaseStr( Key );
 For i:=0 to CountSections-1 do begin
  If Sections[ i ].Name=Section then
   For j:=0 to Sections[ i ].CountKeys-1 do
    If Sections[ i ].Keys[ j ].Name=Key then begin
     Str:=Sections[ i ].Keys[ j ].Value;
     if Str[1]='-' then begin
      for k:=2 to Length( Str ) do begin
       if Str[k] in Chisla then Result:=Result*10+StrToInt( Str[k] )
        else begin
         Result:=0;
         Break;
        end;
      end;
      Result:=-1*Result;
     end else begin
      for k:=1 to Length( Str ) do begin
       if Str[k] in Chisla then Result:=Result*10+StrToInt( Str[k] )
        else begin
         Result:=0;
         Break;
        end;
      end;
     end;
     GoTo L1;
    end;
 end;
 L1: ;
end;

function  tINI.INI_ReadKeyStr( Section, Key: string ):string;
label L1;
var i, j: integer;
begin
 Result:='';
 Section:=UpCaseStr( Section );
 Key:=UpCaseStr( Key );
 For i:=0 to CountSections-1 do begin
  If Sections[ i ].Name=Section then
   For j:=0 to Sections[ i ].CountKeys-1 do
    If Sections[ i ].Keys[ j ].Name=Key then begin
     Result:=Sections[ i ].Keys[ j ].Value;
     GoTo L1;
    end;
 end;
 L1: ;
end;

function  tINI.INI_ReadKeyFloat( Section, Key: string ):real;
label L1;
const Chisla : set of char = [ '0', '1', '2', '3', '4', '5', '6', '7', '8', '9' ];
var i, j, k: integer;
    c_drob : byte;
    drob   : real;
    tb_drob: boolean;
    Str    : string;
begin

Result:=0;
drob  :=0;
tb_drob:= false;
Section:=UpCaseStr( Section );
Key:=UpCaseStr( Key );
For i:=0 to CountSections-1 do If Sections[ i ].Name=Section then
 For j:=0 to Sections[ i ].CountKeys-1 do If Sections[ i ].Keys[ j ].Name=Key then begin
  Str:=Sections[ i ].Keys[ j ].Value;
  if Str[1]='-' then begin

   for k:=2 to Length( Str ) do begin
    if Str[k] in Chisla then
     if tb_drob then begin
      drob:=drob*10+StrToInt( Str[k] );
      inc( c_drob );
     end else Result:=Result*10+StrToInt( Str[k] )
    else
     if ((Str[k]='.') or (Str[k]=',')) and (not tb_drob)
      then tb_drob:=true
      else begin
       Result:=0;
       Break;
      end;
   end;
   if tb_drob then begin
    for k:=1 to c_drob do drob:=drob/10;
    Result:=Result+drob;
   end;
   Result:=-1*Result;

  end else begin

   for k:=1 to Length( Str ) do begin
    if Str[k] in Chisla then
     if tb_drob then begin
      drob:=drob*10+StrToInt( Str[k] );
      inc( c_drob );
     end else Result:=Result*10+StrToInt( Str[k] )
    else
     if ((Str[k]='.') or (Str[k]=',')) and (not tb_drob)
      then tb_drob:=true
      else begin
       Result:=0;
       Break;
      end;
   end;
   if tb_drob then begin
    for k:=1 to c_drob do drob:=drob/10;
    Result:=Result+drob;
   end;

  end;
  GoTo L1;
 end;

 L1: ;
end;

function  tINI.INI_ReadKeyBol( Section, Key: string ) : boolean;
label L1;
var i, j, k: integer;
    Str    : string;
begin
 Result:=false;
 Section:=UpCaseStr( Section );
 Key:=UpCaseStr( Key );
 For i:=0 to CountSections-1 do begin
  If Sections[ i ].Name=Section then
   For j:=0 to Sections[ i ].CountKeys-1 do
    If Sections[ i ].Keys[ j ].Name=Key then begin
     Str:=Sections[ i ].Keys[ j ].Value;
     for k:=1 to Length( Str ) do Str[ k ]:=UpCase( Str[ k ] );
     if Str='TRUE' then Result:=true;
     GoTo L1;
    end;
 end;
 L1: ;
end;

procedure tINI.INI_WriteKeyInt( Section, Key: string; Value: integer );
label L1;
var i, j: integer;
begin
 Section:=UpCaseStr( Section );
 Key:=UpCaseStr( Key );
 For i:=0 to CountSections-1 do begin
  If Sections[ i ].Name=Section then
   For j:=0 to Sections[ i ].CountKeys-1 do
    If Sections[ i ].Keys[ j ].Name=Key then begin
     Sections[ i ].Keys[ j ].Value:=IntToStr( Value );
     GoTo L1;
    end;
 end;
 L1: ;
end;

procedure tINI.INI_WriteKeyStr( Section, Key: string; Value: string );
label L1;
var i, j: integer;
begin
 Section:=UpCaseStr( Section );
 Key:=UpCaseStr( Key );
 For i:=0 to CountSections-1 do begin
  If Sections[ i ].Name=Section then
   For j:=0 to Sections[ i ].CountKeys-1 do
    If Sections[ i ].Keys[ j ].Name=Key then begin
     Sections[ i ].Keys[ j ].Value:=Value;
     GoTo L1;
    end;
 end;
 L1: ;
end;

procedure tINI.INI_WriteKeyFloat( Section, Key: string; Value: real );
label L1;
var i, j: integer;
begin
 Section:=UpCaseStr( Section );
 Key:=UpCaseStr( Key );
 For i:=0 to CountSections-1 do begin
  If Sections[ i ].Name=Section then
   For j:=0 to Sections[ i ].CountKeys-1 do
    If Sections[ i ].Keys[ j ].Name=Key then begin
     Sections[ i ].Keys[ j ].Value:=FloatToStr( Value );
     GoTo L1;
    end;
 end;
 L1: ;
end;

procedure tINI.INI_WriteKeyBol( Section, Key: string; Value: boolean );
label L1;
var i, j: integer;
begin
 Section:=UpCaseStr( Section );
 Key:=UpCaseStr( Key );
 For i:=0 to CountSections-1 do begin
  If Sections[ i ].Name=Section then
   For j:=0 to Sections[ i ].CountKeys-1 do
    If Sections[ i ].Keys[ j ].Name=Key then begin
     if Value then Sections[ i ].Keys[ j ].Value:='TRUE'
      else Sections[ i ].Keys[ j ].Value:='FALSE';
     GoTo L1;
    end;
 end;
 L1: ;
end;

procedure tINI.UpCaseAll;
var i, j: integer;
begin
 For i:=0 to CountSections-1 do begin
  Sections[ i ].Name:=UpCaseStr( Sections[ i ].Name );
  For j:=0 to Sections[ i ].CountKeys-1 do
   Sections[ i ].Keys[ j ].Name:=UpCaseStr( Sections[ i ].Keys[ j ].Name );
 end;
end;

end.
