unit uMainMenuGUI;

interface

uses  zglHeader,
      uGlobalData,
      uGUI;

type  pMMGData = ^tMMGData;     //   
      tMMGData = record
        Bn_Image : zglPTexture; //
        Bn_Glass : zglPTexture; // 
        Bn_Border: zglPTexture; //
        RectEx   : zglTRect;      //
      end;

var   Bn_NewGame_Data, Bn_Continue_Data,
      Bn_Options_Data, Bn_About_Data,
      Bn_Exit_Data  : tMMGData;
var   Bn_Allocation : zglPTexture; //  
var   Bn_NewGame, Bn_ContinueG,
      Bn_Options, Bn_Help,
      Bn_Exit     : pGUIObject;        //   
var   MenuGUICreated : boolean = false;
var   MenuButtonText : string;           //   

var   Points : array [ 1 .. 5 ] of single; //       6   
var   MoveButtons   : boolean;  //  
var   MoveDirection : boolean; // - ,  - 
var   MoveDelta     : byte;     //   
var   MoveProgress  : single;   //    

const SMALL_BUTTON_SZ = 0.36;          //  
      BIG_BUTTON_SZ   = 0.6;           //  ( ) 
      MOVE_ITER       = 12;            //   ()
      MOVE_SPD        = 1 / MOVE_ITER; //    
      MOVE_SZ_SPD     = ( BIG_BUTTON_SZ - SMALL_BUTTON_SZ ) / MOVE_ITER;  // .  
var   DPOINTS         : single = 0;
var   Now_Move_Sz_Spd : single = 0;

var GoGame,
    GoOptions,
    GoHelp     : boolean; //      (    )

procedure InitMMGData;
procedure InitMenuGUI;
procedure MMGButtonDraw( const GUIObj : pGUIObject );
procedure MMGButtonUpdt( const GUIObj : pGUIObject );
procedure MMGDraw;
procedure MMGUpdt;

implementation

uses  //uOptions,
      uGame;

procedure ShiftButtons;    //        ( )
var d : integer;
begin
  if MoveDirection then d := -1 else d := 1;
  /////////////
  Bn_NewGame.Rect.X := Bn_NewGame.Rect.X + d;
  if Bn_NewGame.Rect.X = 0 then Bn_NewGame.Rect.X := 5;
  if Bn_NewGame.Rect.X = 6 then Bn_NewGame.Rect.X := 1;
  /////////////
  Bn_ContinueG.Rect.X := Bn_ContinueG.Rect.X + d;
  if Bn_ContinueG.Rect.X = 0 then Bn_ContinueG.Rect.X := 5;
  if Bn_ContinueG.Rect.X = 6 then Bn_ContinueG.Rect.X := 1;
  /////////////
  Bn_Options.Rect.X := Bn_Options.Rect.X + d;
  if Bn_Options.Rect.X = 0 then Bn_Options.Rect.X := 5;
  if Bn_Options.Rect.X = 6 then Bn_Options.Rect.X := 1;
  /////////////
  Bn_Help.Rect.X := Bn_Help.Rect.X + d;
  if Bn_Help.Rect.X = 0 then Bn_Help.Rect.X := 5;
  if Bn_Help.Rect.X = 6 then Bn_Help.Rect.X := 1;
  /////////////
  Bn_Exit.Rect.X := Bn_Exit.Rect.X + d;
  if Bn_Exit.Rect.X = 0 then Bn_Exit.Rect.X := 5;
  if Bn_Exit.Rect.X = 6 then Bn_Exit.Rect.X := 1;
end;

{$REGION '  EVENTS  '}

procedure Bn_NewGame_Click ( GUIObj : pGUIObject );
begin
  GoGame    := true;
  GoOptions := false;
  GoHelp    := false;

  Scene := sGame;
  StartNewGame;
end;

procedure Bn_ContinueG_Click ( GUIObj : pGUIObject );
begin
  GoGame    := true;
  GoOptions := false;
  GoHelp    := false;

  Scene := sGame;
  ContinueGame;
end;

procedure Bn_Options_Click ( GUIObj : pGUIObject );
begin
  GoGame    := false;
  GoOptions := true;
  GoHelp    := false;

//  Scene := sOptions;
//  OptionsInit;
end;

procedure Bn_Help_Click ( GUIObj : pGUIObject );
begin
  GoGame    := false;
  GoOptions := false;
  GoHelp    := true;
end;

procedure Bn_Exit_Click ( GUIObj : pGUIObject );
begin
  zgl_Exit;
end;

procedure Bn_MenuButton_KeyDown ( GUIObj : pGUIObject; const KeyCode : Byte );
begin
  with GUIObj^ do begin
    if ( KeyCode = K_LEFT ) then begin
      if MoveButtons then Exit;

      Focus := false;
      prew_obj.Focus := true;
      GUIMan.LastFocused := prew_obj;
      MenuButtonText := pGUIButtonDesc( GUIMan.LastFocused.Desc ).Caption;

      MoveButtons   := true;
      MoveDirection := false;
      Inc( MoveDelta );
      Now_Move_Sz_Spd := MOVE_SZ_SPD / MoveDelta;
      if MoveProgress >= 1 then MoveProgress := 0;
    end;
    if ( KeyCode = K_RIGHT ) then begin
      if MoveButtons then Exit;

      Focus := false;
      next_obj.Focus := true;
      GUIMan.LastFocused := next_obj;
      MenuButtonText := pGUIButtonDesc( GUIMan.LastFocused.Desc ).Caption;

      MoveButtons   := true;
      MoveDirection := true;
      Inc( MoveDelta );
      Now_Move_Sz_Spd := MOVE_SZ_SPD / MoveDelta;
      if MoveProgress >= 1 then MoveProgress := 0;
    end;
  end;
  key_ClearState;
end;

procedure Bn_MenuButton_KeyUp ( GUIObj : pGUIObject; const KeyCode : Byte );
begin
  with GUIObj^ do begin
    if ( KeyCode = K_ENTER ) then begin
      if @Events.OnClick <> nil then Events.OnClick( GUIObj );
    end;
  end;
  key_ClearState;
end;

procedure Bn_MenuButton_MouseEnter( GUIObj : pGUIObject );
begin
  //    
  MenuButtonText := pGUIButtonDesc( GUIObj.Desc ).Caption;
end;

procedure Bn_MenuButton_MouseLeave( GUIObj : pGUIObject );
begin
  //    
  MenuButtonText := pGUIButtonDesc( GUIObj.GUIMan.LastFocused.Desc ).Caption;
end;

procedure Bn_MenuButton_SetFocus( GUIObj : pGUIObject; const Focus : Boolean );
begin
  if Focus and not GUIObj.Focus then begin
    MoveButtons := true;
    MoveProgress:= 0;
    MoveDelta   := round( abs( GUIObj.Rect.X - 3 ) );
    Now_Move_Sz_Spd := MOVE_SZ_SPD / MoveDelta;
    MoveDirection   := ( GUIObj.Rect.X - 3 ) > 0;
  end;
end;

{$ENDREGION}

procedure InitMenuGUI;
var button : pGUIButtonDesc;
    dw : single;
    i  : byte;
begin
/////////////////////////////////////////////////////
  GoGame    := false;
  GoOptions := false;
  GoHelp    := false;
/////////////////////////////////////////////////////
  GUI_SetLayer( MainMenu_GUI );

  DPOINTS := DefScrW / 5;
  dw := DPOINTS;
  Points[ 1 ] := dw/2;
  for i:=2 to 5 do begin
    Points[ i ] := Points[ i - 1 ] + dw;
  end;
  ///////////
  button := New( pGUIButtonDesc );
  button.Font := font2;
  button.Caption := ' ';
  button.Pressed := false;
  Bn_NewGame := GUI_AddObject( GUI_BUTTON, 3, DefScrH/2,
                               BIG_BUTTON_SZ, BIG_BUTTON_SZ,
                               true, true, button, @Bn_NewGame_Data, nil, nil );
  Bn_NewGame.OnDraw := @MMGButtonDraw;
  Bn_NewGame.OnUpdt := @MMGButtonUpdt;
  Bn_NewGame.Events.OnClick      := @Bn_NewGame_Click;
  Bn_NewGame.Events.OnKeyDown    := @Bn_MenuButton_KeyDown;
  Bn_NewGame.Events.OnKeyUp      := @Bn_MenuButton_KeyUp;
  Bn_NewGame.Events.OnMouseEnter := @Bn_MenuButton_MouseEnter;
  Bn_NewGame.Events.OnMouseLeave := @Bn_MenuButton_MouseLeave;
  Bn_NewGame.Events.OnSetFocus   := @Bn_MenuButton_SetFocus;
  ///////////
  button := New( pGUIButtonDesc );
  button.Font := font2;
  button.Caption := '';
  button.Pressed := false;
  Bn_ContinueG := GUI_AddObject( GUI_BUTTON, 4, DefScrH/2,
                                 SMALL_BUTTON_SZ, SMALL_BUTTON_SZ,
                                 false, true, button, @Bn_Continue_Data, nil, Bn_NewGame );
  Bn_ContinueG.OnDraw := @MMGButtonDraw;
  Bn_ContinueG.OnUpdt := @MMGButtonUpdt;
  //Bn_ContinueG.Events.OnClick      := @Bn_ContinueG_Click;
  Bn_ContinueG.Events.OnKeyDown    := @Bn_MenuBUtton_KeyDown;
  Bn_ContinueG.Events.OnKeyUp      := @Bn_MenuBUtton_KeyUp;
  Bn_ContinueG.Events.OnMouseEnter := @Bn_MenuButton_MouseEnter;
  Bn_ContinueG.Events.OnMouseLeave := @Bn_MenuButton_MouseLeave;
  Bn_ContinueG.Events.OnSetFocus   := @Bn_MenuButton_SetFocus;
  ///////////
  button := New( pGUIButtonDesc );
  button.Font := font2;
  button.Caption := '';
  button.Pressed := false;
  Bn_Options := GUI_AddObject( GUI_BUTTON, 5, DefScrH/2,
                               SMALL_BUTTON_SZ, SMALL_BUTTON_SZ,
                               false, true, button, @Bn_Options_Data, nil, Bn_ContinueG );
  Bn_Options.OnDraw := @MMGButtonDraw;
  Bn_Options.OnUpdt := @MMGButtonUpdt;
  Bn_Options.Events.OnClick      := @Bn_Options_Click;
  Bn_Options.Events.OnKeyDown    := @Bn_MenuBUtton_KeyDown;
  Bn_Options.Events.OnKeyUp      := @Bn_MenuBUtton_KeyUp;
  Bn_Options.Events.OnMouseEnter := @Bn_MenuButton_MouseEnter;
  Bn_Options.Events.OnMouseLeave := @Bn_MenuButton_MouseLeave;
  Bn_Options.Events.OnSetFocus   := @Bn_MenuButton_SetFocus;
  ///////////
  button := New( pGUIButtonDesc );
  button.Font := font2;
  button.Caption := '';
  button.Pressed := false;
  Bn_Help := GUI_AddObject( GUI_BUTTON, 1, DefScrH/2,
                            SMALL_BUTTON_SZ, SMALL_BUTTON_SZ,
                            false, true, button, @Bn_About_Data, nil, Bn_Options );
  Bn_Help.OnDraw := @MMGButtonDraw;
  Bn_Help.OnUpdt := @MMGButtonUpdt;
  Bn_Help.Events.OnClick      := @Bn_Help_Click;
  Bn_Help.Events.OnKeyDown    := @Bn_MenuBUtton_KeyDown;
  Bn_Help.Events.OnKeyUp      := @Bn_MenuBUtton_KeyUp;
  Bn_Help.Events.OnMouseEnter := @Bn_MenuButton_MouseEnter;
  Bn_Help.Events.OnMouseLeave := @Bn_MenuButton_MouseLeave;
  Bn_Help.Events.OnSetFocus   := @Bn_MenuButton_SetFocus;
  ///////////
  button := New( pGUIButtonDesc );
  button.Font := font2;
  button.Caption := '';
  button.Pressed := false;
  Bn_Exit := GUI_AddObject( GUI_BUTTON, 2, DefScrH/2,
                            SMALL_BUTTON_SZ, SMALL_BUTTON_SZ,
                            false, true, button, @Bn_Exit_Data, Bn_NewGame, Bn_Help );
  Bn_Exit.OnDraw := @MMGButtonDraw;
  Bn_Exit.OnUpdt := @MMGButtonUpdt;
  Bn_Exit.Events.OnClick      := @Bn_Exit_Click;
  Bn_Exit.Events.OnKeyDown    := @Bn_MenuBUtton_KeyDown;
  Bn_Exit.Events.OnKeyUp      := @Bn_MenuBUtton_KeyUp;
  Bn_Exit.Events.OnMouseEnter := @Bn_MenuButton_MouseEnter;
  Bn_Exit.Events.OnMouseLeave := @Bn_MenuButton_MouseLeave;
  Bn_Exit.Events.OnSetFocus   := @Bn_MenuButton_SetFocus;
  ///////////
  Bn_NewGame.next_obj := Bn_ContinueG;
  Bn_NewGame.prew_obj := Bn_Exit;
  ///////////
  Bn_ContinueG.next_obj := Bn_Options;
  Bn_ContinueG.prew_obj := Bn_NewGame;
  ///////////
  Bn_Options.next_obj := Bn_Help;
  Bn_Options.prew_obj := Bn_ContinueG;
  ///////////
  Bn_Help.next_obj := Bn_Exit;
  Bn_Help.prew_obj := Bn_Options;
  ///////////
  Bn_Exit.next_obj := Bn_NewGame;
  Bn_Exit.prew_obj := Bn_Help;
  ///////////
  MenuButtonText := pGUIButtonDesc( Bn_NewGame.GUIMan.LastFocused.Desc ).Caption;
end;

procedure InitMMGData;
begin
  Bn_NewGame_Data.Bn_Image  := Resources.GetTex( 'menu_button_image_new_game' );
  Bn_NewGame_Data.Bn_Glass  := Resources.GetTex( 'menu_button_glass' );
  Bn_NewGame_Data.Bn_Border := Resources.GetTex( 'menu_button_border' );

  Bn_Continue_Data.Bn_Image := Resources.GetTex( 'menu_button_image_continue' );
  Bn_Continue_Data.Bn_Glass := Bn_NewGame_Data.Bn_Glass;
  Bn_Continue_Data.Bn_Border:= Bn_NewGame_Data.Bn_Border;

  Bn_Options_Data.Bn_Image  := Resources.GetTex( 'menu_button_image_options' );
  Bn_Options_Data.Bn_Glass  := Bn_NewGame_Data.Bn_Glass;
  Bn_Options_Data.Bn_Border := Bn_NewGame_Data.Bn_Border;

  Bn_About_Data.Bn_Image    := Resources.GetTex( 'menu_button_image_help' );
  Bn_About_Data.Bn_Glass    := Bn_NewGame_Data.Bn_Glass;
  Bn_About_Data.Bn_Border   := Bn_NewGame_Data.Bn_Border;

  Bn_Exit_Data.Bn_Image     := Resources.GetTex( 'menu_button_image_exit' );
  Bn_Exit_Data.Bn_Glass     := Bn_NewGame_Data.Bn_Glass;
  Bn_Exit_Data.Bn_Border    := Bn_NewGame_Data.Bn_Border;

  Bn_Allocation := Resources.GetTex( 'menu_button_allocation' );
end;

procedure MMGButtonDraw( const GUIObj : pGUIObject );
var tex   : zglPTexture;
    d, dx : single;
    alph  : byte;
begin
  with GUIObj^ do begin
    if not Visible then Exit;
    if MoveButtons then begin
      dx := MoveProgress * ( Points[ 2 ] - Points[ 1 ] );
      if MoveDirection then dx := -1 * dx;
    end else dx := 0;
    //   -  -
    // . ,     .
    d := 512 * Rect.W;
    alph := 255;
    if ( Rect.X = 1 ) and MoveDirection and MoveButtons     then alph := round( 255 * ( 1 - MoveProgress ) );
    if ( Rect.X = 5 ) and not MoveDirection and MoveButtons then alph := round( 255 * ( 1 - MoveProgress ) );
    //  
    tex := pMMGData( Data ).Bn_Image;
    SSprite2D_Draw( tex,
                    SetX_ByCenter( dx + Points[ round( Rect.X ) ], d ),
                    SetY_ByCenter( Rect.Y, d ),
                    d, d, 0, alph );
    if Rect.X = 1 then
      SSprite2D_Draw( tex,
                      SetX_ByCenter( dx + Points[ 5 ] + DPOINTS, d ),
                      SetY_ByCenter( Rect.Y, d ),
                      d, d, 0, round( 255 * MoveProgress ) );
    if Rect.X = 5 then
      SSprite2D_Draw( tex,
                      SetX_ByCenter( dx + Points[ 1 ] - DPOINTS, d ),
                      SetY_ByCenter( Rect.Y, d ),
                      d, d, 0, round( 255 * MoveProgress ) );
    //  
    tex := pMMGData( Data ).Bn_Glass;
    SSprite2D_Draw( tex,
                    SetX_ByCenter( dx + Points[ round( Rect.X ) ], d ),
                    SetY_ByCenter( Rect.Y, d ),
                    d, d, 0, alph );
    if Rect.X = 1 then
      SSprite2D_Draw( tex,
                      SetX_ByCenter( dx + Points[ 5 ] + DPOINTS, d ),
                      SetY_ByCenter( Rect.Y, d ),
                      d, d, 0, round( 255 * MoveProgress ) );
    if Rect.X = 5 then
      SSprite2D_Draw( tex,
                      SetX_ByCenter( dx + Points[ 1 ] - DPOINTS, d ),
                      SetY_ByCenter( Rect.Y, d ),
                      d, d, 0, round( 255 * MoveProgress ) );
    //  
    tex := pMMGData( Data ).Bn_Border;
    SSprite2D_Draw( tex,
                    SetX_ByCenter( dx + Points[ round( Rect.X ) ], d ),
                    SetY_ByCenter( Rect.Y, d ),
                    d, d, 0, alph );
    if Rect.X = 1 then
      SSprite2D_Draw( tex,
                      SetX_ByCenter( dx + Points[ 5 ] + DPOINTS, d ),
                      SetY_ByCenter( Rect.Y, d ),
                      d, d, 0, round( 255 * MoveProgress ) );
    if Rect.X = 5 then
      SSprite2D_Draw( tex,
                      SetX_ByCenter( dx + Points[ 1 ] - DPOINTS, d ),
                      SetY_ByCenter( Rect.Y, d ),
                      d, d, 0, round( 255 * MoveProgress ) );

    if MouseIn and not MoveButtons then begin
      fx_SetColorMode( FX_COLOR_SET );
      fx2d_SetColor( $FFFFFF );
      SSprite2D_Draw( Bn_Allocation,
                      SetX_ByCenter( dx + Points[ round( Rect.X ) ], d ),
                      SetY_ByCenter( Rect.Y, d ),
                      d, d, 0, 160, FX_BLEND or FX_COLOR );
      fx_SetColorMode( FX_COLOR_MIX );
    end;
  end;

end;

procedure MMGButtonUpdt( const GUIObj : pGUIObject );
var dx : single;
begin
  with GUIObj^ do begin
    if not Visible then Exit;

    if ( not Focus ) and ( Rect.W > SMALL_BUTTON_SZ ) then begin
      Rect.W := Rect.W - Now_MOVE_SZ_SPD;
    end;
    if ( not Focus ) and ( Rect.W < SMALL_BUTTON_SZ ) then begin
      Rect.W := SMALL_BUTTON_SZ;
    end;

    if Focus and ( Rect.W < BIG_BUTTON_SZ ) then begin
      Rect.W := Rect.W + Now_MOVE_SZ_SPD;
    end;
    if Focus and ( Rect.W > BIG_BUTTON_SZ ) then begin
      Rect.W := BIG_BUTTON_SZ;
    end;

    with pMMGData( Data ).RectEx do begin
      if MoveButtons then begin
        dx := MoveProgress * ( Points[ 2 ] - Points[ 1 ] );
        if MoveDirection then dx := -1 * dx;
      end else dx := 0;
      X := Points[ round( Rect.X ) ] - 128 * Rect.W + dx; { 128 - 256/2 }
      Y := Rect.Y - 144 * Rect.W;                         { 144 - 288/2 }
      W := 256 * Rect.W;
      H := 288 * Rect.W;
    end;
    //   
    if col2d_PointInRect( Mouse_X, Mouse_Y, pMMGData( Data ).RectEx ) then begin
      //   
      if not MouseIn then begin
        //  
        if @Events.OnMouseEnter <> nil then Events.OnMouseEnter( GUIObj );
        MouseIn := true
      end;
      //   
      if Mouse_Up( M_BLEFT ) then begin
        // 
        // ,        
        if Focus and ( @Events.OnClick <> nil ) then Events.OnClick( GUIObj );             ///!!!!!!!!!!!!!!
        if @Events.OnSetFocus <> nil then Events.OnSetFocus( GUIObj, true );
        if GUIMan.LastFocused <> GUIObj then begin
          GUIMan.LastFocused.Focus := false;
          if @GUIMan.LastFocused.Events.OnSetFocus <> nil then GUIMan.LastFocused.Events.OnSetFocus( GUIMan.LastFocused, false );
          GUIMan.LastFocused := GUIObj;
        end;
        Focus := true;
      end;
    end else begin
      //    
      if MouseIn then begin
        // 
        if @Events.OnMouseLeave <> nil then Events.OnMouseLeave( GUIObj );
        MouseIn := false;
      end;
    end;

    //      
    if Focus then begin
      if key_Last( KA_DOWN ) <> 0 then
        if @Events.OnKeyDown <> nil then
          Events.OnKeyDown( GUIObj, key_Last( KA_DOWN ) );
      if key_Last( KA_UP ) <> 0 then
        if @Events.OnKeyUp <> nil then
          Events.OnKeyUp( GUIObj, key_Last( KA_Up ) );
    end;
  end;
end;

procedure MMGDraw;
var d : single;
    a : byte;
begin
  // 
  PR2D_Rect( 0, DefScrH/2 - 512 * BIG_BUTTON_SZ/2 - 40,
             DefScrW, 512 * BIG_BUTTON_SZ + 80, $000000,
             128, PR2D_FILL );
  PR2D_Rect( 0, DefScrH/2 - 512 * BIG_BUTTON_SZ/2 - 40,
             DefScrW, 512 * BIG_BUTTON_SZ + 80, $000000 );
  // 
  mGUI_Draw;
  //    
  text_DrawEx( Font2, points[ 3 ], DefScrH/2 + 144*BIG_BUTTON_SZ + 40,
               1.7, 0, MenuButtonText, 255, $ffffff,
               TEXT_HALIGN_CENTER );
end;

procedure MMGUpdt;
begin
  if MoveButtons then begin
    MoveProgress := MoveProgress + MOVE_SPD;
    if MoveProgress >= 1 then begin
      Dec( MoveDelta );
      if MoveDelta > 0 then begin
        MoveProgress := 0;
      end else begin
        MoveProgress := 1;
        MoveButtons  := false;
      end;
      ShiftButtons;
    end;
  end;
end;

end.
