{
 *  Copyright © Kemka Andrey aka Andru
 *  mail: dr.andru@gmail.com
 *  site: http://andru-kun.inf.ua
 *
 *  This file is part of ZenGL.
 *
 *  ZenGL is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as
 *  published by the Free Software Foundation, either version 3 of
 *  the License, or (at your option) any later version.
 *
 *  ZenGL is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with ZenGL. If not, see http://www.gnu.org/licenses/
}
unit zgl_sengine_2d;

{$I zgl_config.cfg}

interface

uses
  zgl_textures;

type
  zglPSprite2D = ^zglTSprite2D;
  zglPSEngine2D = ^zglTSEngine2D;

  zglTSEngine2D = record
    Count : LongWord;
    List  : array of zglPSprite2D;
  end;

  zglTSprite2D = record
    ID      : Integer;
    Manager : zglPSEngine2D;
    Texture : zglPTexture;
    Destroy : Boolean;
    Layer   : Integer;
    X, Y    : Single;
    W, H    : Single;
    Angle   : Single;
    Frame   : Single;
    Alpha   : Integer;
    FxFlags : LongWord;
    Data    : Pointer;

    OnInit  : procedure( const Sprite : zglPSprite2D );
    OnDraw  : procedure( const Sprite : zglPSprite2D );
    OnProc  : procedure( const Sprite : zglPSprite2D );
    OnFree  : procedure( const Sprite : zglPSprite2D );
  end;

function  sengine2d_AddSprite( const Texture : zglPTexture; const Layer : Integer; const OnInit, OnDraw, OnProc, OnFree : Pointer ) : zglPSprite2D;
procedure sengine2d_DelSprite( const ID : Integer );
procedure sengine2d_ClearAll;

procedure sengine2d_Set( const SEngine : zglPSEngine2D );
procedure sengine2d_Draw;
procedure sengine2d_Proc;
procedure sengine2d_Sort( iLo, iHi : Integer );
procedure sengine2d_SortID( iLo, iHi : Integer );

implementation
uses
  zgl_main,
  zgl_fx,
  zgl_sprite_2d;

var
  _sengine  : zglTSEngine2D;
  sengine2d : zglPSEngine2D;

function sengine2d_AddSprite;
  var
    new : zglPSprite2D;
begin
  if sengine2d.Count + 1 > length( sengine2d.List ) Then
    SetLength( sengine2d.List, length( sengine2d.List ) + 16384 );

  zgl_GetMem( Pointer( new ), SizeOf( zglTSprite2D ) );
  sengine2d.List[ sengine2d.Count ] := new;
  INC( sengine2d.Count );

  new.ID      := sengine2d.Count - 1;
  new.Manager := sengine2d;
  new.Texture := Texture;
  new.Layer   := Layer;
  new.X       := 0;
  new.Y       := 0;
  new.W       := Texture.Width div Texture.FramesX;
  new.H       := Texture.Height div Texture.FramesY;
  new.Angle   := 0;
  new.Frame   := 1;
  new.Alpha   := 255;
  new.FxFlags := FX_BLEND;
  new.Data    := nil;
  new.OnInit  := OnInit;
  new.OnDraw  := OnDraw;
  new.OnProc  := OnProc;
  new.OnFree  := OnFree;
  Result      := new;
  if Assigned( Result.OnInit ) Then
    Result.OnInit( Result );
end;

procedure sengine2d_DelSprite;
  var
    i : Integer;
begin
  if ( ID < 0 ) or ( ID > sengine2d.Count - 1 ) or ( sengine2d.Count = 0 ) Then exit;

  if Assigned( sengine2d.List[ ID ].OnFree ) Then
    sengine2d.List[ ID ].OnFree( sengine2d.List[ ID ] );

  FreeMemory( sengine2d.List[ ID ] );
  for i := ID to sengine2d.Count - 2 do
    begin
      sengine2d.List[ i ]    := sengine2d.List[ i + 1 ];
      sengine2d.List[ i ].ID := i;
    end;

  DEC( sengine2d.Count );
end;

procedure sengine2d_ClearAll;
  var
    i : Integer;
    s : zglPSprite2D;
begin
  for i := 0 to sengine2d.Count - 1 do
    begin
      s := sengine2d.List[ i ];
      if Assigned( s.OnFree ) Then
        sengine2d.List[ i ].OnFree( s );
      FreeMemory( s );
    end;
  SetLength( sengine2d.List, 0 );
  sengine2d.Count := 0;
end;

procedure sengine2d_Set;
begin
  if Assigned( SEngine ) Then
    sengine2d := SEngine
  else
    sengine2d := @_sengine;
end;

procedure sengine2d_Draw;
  var
    i, a, b, l : Integer;
    s : zglPSprite2D;
begin
  if sengine2d.Count > 1 Then
    begin
      l := 0;
      for i := 0 to sengine2d.Count - 1 do
        begin
          s := sengine2d.List[ i ];
          if s.Layer > l Then l := s.Layer;
          if s.Layer < l Then
            begin
              sengine2d_Sort( 0, sengine2d.Count - 1 );
              // TODO: наверное сделать выбор вкл./выкл. устойчивой сортировки
              l := sengine2d.List[ 0 ].Layer;
              a := 0;
              for b := 0 to sengine2d.Count - 1 do
                begin
                  s := sengine2d.List[ b ];
                  if ( l <> s.Layer ) Then
                    begin
                      sengine2d_SortID( a, b - 1 );
                      a := b;
                      l := s.Layer;
                    end;
                  if b = sengine2d.Count - 1 Then
                    sengine2d_SortID( a, b );
                end;
              for a := 0 to sengine2d.Count - 1 do
                sengine2d.List[ a ].ID := a;
              break;
            end;
        end;
    end;

  i := 0;
  while i < sengine2d.Count do
    begin
      s := sengine2d.List[ i ];
      if Assigned( s.OnDraw ) Then
        s.OnDraw( s )
      else
        asprite2d_Draw( s.Texture, s.X, s.Y, s.W, s.H, s.Angle, Round( s.Frame ), s.Alpha, s.FxFlags );

      if Assigned( s ) Then
        begin
          if s.Destroy Then
            sengine2d_DelSprite( s.ID )
          else
            INC( i );
        end;
    end;
end;

procedure sengine2d_Proc;
  var
    i : Integer;
    s : zglPSprite2D;
begin
  i := 0;
  while i < sengine2d.Count do
    begin
      s := sengine2d.List[ i ];
      if Assigned( s.OnProc ) Then
        s.OnProc( s );

      if Assigned( s ) Then
        begin
          if s.Destroy Then
            sengine2d_DelSprite( s.ID )
          else
            INC( i );
        end;
    end;
end;

procedure sengine2d_Sort;
  var
    lo, hi, mid : Integer;
    t : zglPSprite2D;
begin
  lo   := iLo;
  hi   := iHi;
  mid  := sengine2d.List[ ( lo + hi ) shr 1 ].Layer;

  with sengine2d^ do
  repeat
    while List[ lo ].Layer < mid do INC( lo );
    while List[ hi ].Layer > mid do DEC( hi );
    if lo <= hi then
      begin
        t          := List[ lo ];
        List[ lo ] := List[ hi ];
        List[ hi ] := t;
        INC( lo );
        DEC( hi );
      end;
  until lo > hi;

  if hi > iLo Then sengine2d_Sort( iLo, hi );
  if lo < iHi Then sengine2d_Sort( lo, iHi );
end;

procedure sengine2d_SortID;
  var
    lo, hi, mid : Integer;
    t : zglPSprite2D;
begin
  lo   := iLo;
  hi   := iHi;
  mid  := sengine2d.List[ ( lo + hi ) shr 1 ].ID;

  with sengine2d^ do
  repeat
    while List[ lo ].ID < mid do INC( lo );
    while List[ hi ].ID > mid do DEC( hi );
    if lo <= hi then
      begin
        t          := List[ lo ];
        List[ lo ] := List[ hi ];
        List[ hi ] := t;
        INC( lo );
        DEC( hi );
      end;
  until lo > hi;

  if hi > iLo Then sengine2d_SortID( iLo, hi );
  if lo < iHi Then sengine2d_SortID( lo, iHi );
end;

initialization
  sengine2d := @_sengine;

end.
