program Alienator;

{$R 'Alienator.res'}

uses
  Windows,
  SysUtils,
  Math,
  DGLE2 in 'DGLE2.pas',
  DGLE2_types in 'DGLE2_types.pas',
  Levels in 'Levels.pas';
  
const

  APP_CAPTION   = 'Alienator';
  DLL_PATH      = 'DGLE2.dll';
	RESOURCE_PATH = 'Data.zip|';
  START_LEVEL   = 1;

var

  pEngineCore : IEngineCore;
  pResMan     : IResourceManager;
  pRender2D   : IRender2D;
  pInput      : IInput;

  mouse       : TMouseStates;

  pTexLogo, pTexFon,
  pTexMan1, pTexCursor,
  pTexHlam, pTexSliz,
  pTexUfo, pTexMan2,
  pTexWoman, pTexCop,
  pTextexUfo1, pTextexUfo2,
  pTexEgg, pTexVdv,
  pTexBullet, pTexBlood,
  pTexCoin, pTexTeam: ITexture;

  LaserArmed : Boolean;
  LaserCounter : Integer;

  pMdlUfo : IMesh;

  pFont : IBitmapFont;

  counter : Integer = 0;

  Score : Integer = 0;
  BestScore : Integer = 0;

  isGame : Boolean = false;
  CurrentLevel : Integer = 1;

  SlizCounter : Integer;

  CurHint : Integer = 0;

  ClickFlagL : Boolean = false;
  ClickFlagR : Boolean = false;

  ufoX, ufoY : Single;
  UfoAllowMove : Boolean;

  SoundsCounter : Integer;

  Loose, Win : Cardinal;
  GameEnd : Boolean;

  pSndChpok, pSndLaser, pSndNetu, pSndTeleport, pSndAlienTalk,
  pSndShot,  pSndAlienPain, pSndAlienDie, pSndAlienBrain,
  pSndManPain, pSndManDie, pSndWomanPain, pSndWomanDie, pSndVdvPain, pSndVdvDie,
  pSndManPanic, pSndWomanPanic, pSndCopNormal, pSndCopPanic, pSndVdvNormal, pSndVdvPanic: ISoundSample;

  pMusic : IMusic;

function GenChelAngle : Single;
begin
  Result := random(360);
  if ((Result >= 270) and (Result <= 305)) or ((Result >= 55) and (Result <= 90)) then Result := random(110) - 55;
  if ((Result >= 90) and (Result <= 125)) or ((Result >= 235) and (Result <= 270)) then Result := random(110) + 125;
end;

/////////////////////////// GAME ///////////////////////////

type
  TState = (WALK, PANIC, DIE, ALIEN);

  TCivilian = record
  x, y, ox, oy : Single;
  angle, aframe : Single;
  state : TState;
  ugol : Boolean;
  chel_type, time, lives, wait_shot: integer;
  end;

  TAlien = record
  x, y, ox, oy : Single;
  ugol : Boolean;
  target, time, lives : Integer;
  angle, aframe : Single;
  end;

  TEgg = record
  x, y : Single;
  ugol : Boolean;
  time, lives : Integer;
  end;

  TBullet = record
  owner : Integer;
  x, y, angle : Single;
  end;

  TBlood = record
  color : TColor4;
  x, y, angle, aframe, scale : Single;
  end;

  TMoney = record
  x, y, aframe : Single;
  end;

var
  money : array of TMoney;
  bullets : array of TBullet;
  aliens : array of TAlien;
  eggs : array of TEgg;
  civis : array of TCivilian;
  cells : array[0..12, 0..9] of Integer;
  blood : array of TBlood;

procedure CreateBlood(x, y : Single; color : TColor4; scale : Single = 1.0);
begin
  SetLength(blood, Length(blood)+1);
  blood[Length(blood) - 1].x := x;
  blood[Length(blood) - 1].y := y;
  blood[Length(blood) - 1].angle := random(360);
  blood[Length(blood) - 1].aframe := 0;
  blood[Length(blood) - 1].color := color;
  blood[Length(blood) - 1].scale := scale;
end;

function SlizFindTarget(idx : Integer) : Integer;
var i : Integer;
MinDist, dist : single;
begin
    MinDist := 99999999;
    Result := 0;
    for i := 0 to Length(civis) - 1 do
    if (civis[i].state = WALK) or (civis[i].state = PANIC) then
    begin
    dist := sqrt(sqr(civis[i].x - aliens[idx].x) + sqr(civis[i].y - aliens[idx].y));
     if MinDist>dist then
     begin
     MinDist := dist;
     Result := i;
     end;
    end;
end;

procedure SortCivArray;
var i, j : integer;  t : TCivilian;
begin

for i := 0 to Length(civis) - 1 do
begin

  if civis[i].state = DIE then
   for j := Length(civis) - 1 downto i do
    if civis[j].state <> DIE then
    begin
     t := civis[j];
     civis[j] := civis[i];
     civis[i] := t;
     break;
    end;

end;

end;

procedure CivilMakeShot(idx : Integer; _x, _y : Single);
var tmp : integer;
begin
with civis[idx] do
begin
if ((angle >= -90) and (angle < 90)) or ((angle > 270) and (angle <= 450)) then tmp := 25 else tmp := -25;
wait_shot := 0;
angle := RadToDeg(ArcTan2((y + 40) - _y, (x + 48 + tmp) - _x) + PI);
pSndShot.Play();
SetLength(bullets, Length(bullets)+1);
bullets[Length(bullets) - 1].x := x + 48 + tmp;
bullets[Length(bullets) - 1].y := y + 40;
bullets[Length(bullets) - 1].angle := angle;
bullets[Length(bullets) - 1].owner := idx;
end;
end;

procedure CreateSliz(_x, _y : Integer);
begin
  SetLength(aliens, Length(aliens) + 1);

  with aliens[Length(aliens) - 1] do
  begin
  x := _x-16;
  y := _y-16;
  ox := x;
  oy := y;
  aframe := 0;
  target := SlizFindTarget(Length(aliens) - 1);
  angle := 0;
  time := 0;
  lives := 100;
  ugol := false;
  end;
end;

function GetCellRectF(i, j : Integer) : TRectF;
begin
  case cells[i,j] of
  0: result := RectF(8, 13, 16, 14);
  2: result := RectF(8, 15, 18, 13);
  3: result := RectF(3, 3, 29, 24);
  4: result := RectF(2, 0, 28, 27);
  5: result := RectF(6, 11, 20, 17);
  6: result := RectF(2, 0, 28, 29);
  7: result := RectF(6, 6, 23, 25);
  8: result := RectF(0, 13, 30, 20);
  9: result := RectF(1, 1, 31, 31);
  10: result := RectF(0, 1, 31, 32);
  11: result := RectF(0, 6, 32, 26);
  12: result := RectF(1, 0, 3, 32);
  13: result := RectF(28, 0, 3, 32);
  14: result := RectF(2, 2, 29, 30);
  16: result := RectF(8, 13, 16, 14);
  18: result := RectF(8, 15, 18, 13);
  19: result := RectF(3, 3, 29, 24);
  20: result := RectF(2, 0, 28, 27);
  21: result := RectF(6, 11, 20, 17);
  22: result := RectF(2, 0, 28, 29);
  23: result := RectF(6, 6, 23, 25);
  24: result := RectF(0, 13, 30, 20);
  25: result := RectF(1, 1, 31, 31);
  26: result := RectF(0, 1, 31, 32);
  end;

  result.x := result.x*2 + i*64;
  result.y := result.y*2 + j*64;
  result.width := result.width*2;
  result.height := result.height*2;
end;

procedure GenGame(map : Cardinal = 0);
var
i, j : Integer;
begin
CurHint := 0;
SlizCounter := 2 + trunc(map/3);
counter := 0;
SoundsCounter := 0;
ufoX := -1000;
ufoY := 200;
UfoAllowMove := false;
LaserArmed := true;
LaserCounter := 200;
Loose := 0;
Win := 0;

SetLength(civis, 0);
SetLength(aliens, 0);
SetLength(eggs, 0);
SetLength(bullets, 0);
SetLength(blood, 0);
SetLength(money, 0);

if map = 0 then
begin
 for i := 0 to 12 do
  for j := 0 to 9 do
    case random(50) of
    0: cells[i, j] := 0;
    1: cells[i, j] := 1;
    2: cells[i, j] := 2;
    3: cells[i, j] := 3;
    4: cells[i, j] := 4;
    5: cells[i, j] := 5;
    6: cells[i, j] := 6;
    7: cells[i, j] := 7;
    8: cells[i, j] := 8;
    else
    begin
      cells[i, j] := -1;
      if (random(10) = 5) and ( (j > 0) and (cells[i, j-1] = -1)) then
      begin

        SetLength(civis, Length(civis) + 1);

        with civis[Length(civis) - 1] do
        begin
        x := i*64 - 16;
        y := j*64 - 32;
        ox := x;
        oy := y;
        angle := GenChelAngle;
        aframe := random(4);
        chel_type:= random(5);
        time := 0;
        lives := 100;
        wait_shot := 50;
        state := WALK;
        ugol := false;
        end;

      end;
    end;

    end;//case
end //gen map
else
begin
 if map > Length(LevelData) then
  GameEnd := true else
 begin
      for j := 0 to 9 do
      for i := 0 to 12 do
      if LevelData[map - 1][j*13 + i] >= -1 then
        cells[i, j] := LevelData[map - 1][j*13 + i]
      else
      begin//create civils

        cells[i, j] := -1;

        SetLength(civis, Length(civis) + 1);

        with civis[Length(civis) - 1] do
        begin
        x := i*64 - 16;
        y := j*64 - 32;
        ox := x;
        oy := y;
        angle := GenChelAngle;
        aframe := random(4);
        chel_type:= Abs(LevelData[map - 1][j*13 + i] + 2);
        time := 0;
        if chel_type = 4 then
        lives := 150
        else
        lives := 100;
        wait_shot := 50;
        state := WALK;
        ugol := false;
        end;

      end;
 end;
end;
end;

procedure ProcessGame;
var
i, j, k : Integer;
ufo_angle, ufo_speed, civ_speed, dist, min_dist : Single;
flag : boolean;
begin

  if UfoAllowMove and (LaserCounter > 50) then
  begin
    ufo_angle := ArcTan2((ufoY + 150) - mouse.iY, (ufoX + 64) - mouse.iX) + PI;
    ufo_speed := sqrt(sqr((ufoX + 64)- mouse.iX) + sqr((ufoY + 150) - mouse.iY)) * 0.05;
    ufoX := ufoX + ufo_speed*cos(ufo_angle);
    ufoY := ufoY + ufo_speed*sin(ufo_angle);
  end;

  LaserCounter := LaserCounter + 4;

  k := 0;
  while k < Length(money) do
  with money[k] do
  begin
   aframe := aframe + 0.15;
   if round(aframe) > 3 then aframe := 0;
   if IntersectRect(RectF(mouse.iX - 37, mouse.iY - 15, 75, 30), RectF(round(x), round(y), 16, 16)) then
   begin
      pSndChpok.Play();
      score := score + 10;
      CreateBlood(x, y, Color4(255, 255, 255, 125));
      if Length(money)>1 then
      money[k] := money[Length(money)-1];
      SetLength(money,Length(money)-1);
      k := k - 1;
   end;
   k := k + 1;
  end;

  for k := 0 to Length(eggs) - 1 do
  begin
    eggs[k].time := eggs[k].time + 1;
    if eggs[k].time > 300 then
    begin
      if eggs[k].lives > 0 then CreateSliz(round(eggs[k].x), round(eggs[k].y));
      eggs[k].lives := 0;
    end;
  end;

  for k := 0 to Length(aliens) - 1 do
  with aliens[k] do
  begin
    time := time + 1;

    if (time = 375) and (lives > 0) then
    begin
    pSndAlienDie.Play();
    lives := 0;
    end;

    if lives <= 0 then
    begin

    if round(aframe) < 2 then aframe := 2;
    aframe := aframe + 0.07;
    if round(aframe) >=7 then aframe := 7;

    end else
    begin
    if target <> -1 then
    angle := RadToDeg(ArcTan2((civis[target].y + 64) - (y + 16), (civis[target].x + 48) - (x + 16)));

    if random(50) = 25 then target := SlizFindTarget(k);

    aframe := aframe + 0.15;
    if round(aframe) > 1 then aframe := 0.0;

    if round(aframe) = 1 then
    begin
    x := x + 2.5*cos(degtorad(angle));
    y := y + 2.5*sin(degtorad(angle));
    end;

      for i := 0 to 12 do
      for j := 0 to 9 do
      if (cells[i,j] <> 1) and (cells[i,j] <> -1) then
      begin
      if IntersectRect(GetCellRectF(i,j), RectF(round(x), round(y) + 4, 32, 18)) then
      begin
        angle := random(360);
        target := -1;
        x := ox; y := oy;
      end;
      end;

    ox := x;
    oy := y;

    for i := 0 to Length(civis) - 1 do
      if (civis[i].state <> ALIEN) and (civis[i].state <> DIE) and IntersectRect(RectF(round(civis[i].x) + 33, round(civis[i].y) + 12, 24, 69), RectF(round(x), round(y) + 4, 32, 18)) then
      begin
       civis[i].state := ALIEN;
       if civis[i].chel_type = 4 then
       civis[i].lives := 150
       else
       civis[i].lives := 100;
       pSndAlienBrain.Play();
       if Length(aliens)>1 then
       aliens[k] := aliens[Length(aliens)-1];
       SetLength(aliens,Length(aliens)-1);
       Exit;
      end;
  end;//lives
  end;

  for k := 0 to Length(civis) - 1 do
  with civis[k] do
  begin

    case state of
    WALK: civ_speed := 1.2;
    PANIC: civ_speed := 2.4;
    DIE: civ_speed := 1.0;
    ALIEN: civ_speed := 1.8;
    end;

    if state = ALIEN then
    begin

    if random(40) = 20 then
      for i := 0 to Length(civis) - 1 do
      if (civis[i].state <> ALIEN) and (civis[i].state <> DIE) then
      begin
        dist := sqrt(sqr(x - civis[i].x) + sqr(y - civis[i].y));
        if min_dist > dist then
        begin
          min_dist := dist;
          angle := RadToDeg(ArcTan2((y + 64) - (civis[i].y + 64), (x + 48) - (civis[i].x + 48)) + PI);
        end;
      end;

      for i := 0 to Length(civis) - 1 do
      if (civis[i].state <> ALIEN) and (civis[i].state <> DIE) and
      IntersectRect(RectF(round(civis[i].x) + 30, round(civis[i].y) + 12, 27, 69), RectF(round(x) + 33, round(y) + 12, 24, 69)) then
      begin
      civis[i].lives := 0;
      civis[i].state := DIE;
      case civis[i].chel_type of
      2: pSndWomanDie.Play();
      4: pSndVdvDie.Play();
      else pSndManDie.Play();
      end;
      CreateBlood(round(civis[i].x) + 33, round(civis[i].y) + 20,Color4(255,0,0,255), 2);
      SetLength(money, Length(money)+1);
      money[Length(money) - 1].x := civis[i].x + 40;
      money[Length(money) - 1].y := civis[i].y + 50;
      money[Length(money) - 1].aframe := 0.0;
      end;

      time := time + 1;
      if time > 500 then
      begin
      lives := 0;
      state := DIE;
      CreateSliz(round(x + 64), round(y + 48));
      SetLength(eggs, Length(eggs)+1);
      eggs[Length(eggs) - 1].x := x + 32;
      eggs[Length(eggs) - 1].y := y + 48;
      eggs[Length(eggs) - 1].time := 0;
      eggs[Length(eggs) - 1].lives := 100;
      eggs[Length(eggs) - 1].ugol := false;
      end;
    end;

    aframe := aframe + 0.125*civ_speed;
    wait_shot := wait_shot + 1;

    if state = DIE then
    begin
      if round(aframe) >= 7 then aframe := 7;
    end else
    begin
    if round(aframe) > 3 then aframe := 0.0;

    if wait_shot > 5 then
    begin
    x := x + civ_speed*cos(degtorad(angle));
    y := y + civ_speed*sin(degtorad(angle));
    end;

    min_dist := 99999;
    if ((chel_type = 3) or (chel_type = 4)) and ((state = WALK) or (state = PANIC)) then //
    begin

      for i := 0 to Length(aliens) - 1 do
      if aliens[i].lives > 0 then
      begin
        dist := sqrt(sqr(x - aliens[i].x) + sqr(y - aliens[i].y));
        if min_dist > dist then
        begin
          min_dist := dist;
          if (min_dist < 250) and ((wait_shot > 30) or ((chel_type = 4) and (wait_shot > 15))) then CivilMakeShot(k, aliens[i].x + 16, aliens[i].y + 16);
        end;
      end;

      for i := 0 to Length(eggs) - 1 do
      if eggs[i].lives > 0 then
      begin
        dist := sqrt(sqr(x - eggs[i].x) + sqr(y - eggs[i].y));
        if min_dist > dist then
        begin
          min_dist := dist;
          if (min_dist < 250) and ((wait_shot > 30) or ((chel_type = 4) and (wait_shot > 15))) then CivilMakeShot(k, eggs[i].x + 16, eggs[i].y + 16);
        end;
      end;

      for i := 0 to Length(civis) - 1 do
      if civis[i].state = ALIEN then
      begin
        dist := sqrt(sqr(x - civis[i].x) + sqr(y - civis[i].y));
        if min_dist > dist then
        begin
          min_dist := dist;
          if (min_dist < 250) and ((wait_shot > 30) or ((chel_type = 4) and (wait_shot > 15))) then CivilMakeShot(k, civis[i].x + 48, civis[i].y + 48);
        end;
      end;

    end;

    if state = WALK then // 
    begin

      if (random(2000) = 1000) and (SoundsCounter < 2) then
      case chel_type of
      3: begin pSndCopNormal.Play(); SoundsCounter := SoundsCounter + 1; end;
      4: begin pSndVdvNormal.Play(); SoundsCounter := SoundsCounter + 1; end;
      end;

      for i := 0 to Length(aliens) - 1 do
      if aliens[i].lives > 0 then
      begin
        dist := sqrt(sqr(x - aliens[i].x) + sqr(y - aliens[i].y));
        if min_dist > dist then
        begin
          min_dist := dist;

          if min_dist < 125 then
          begin

            if (random(10) = 5) and (SoundsCounter < 7) then
            case chel_type of
            2: begin pSndWomanPanic.Play(); SoundsCounter := SoundsCounter + 1; end;
            3: begin pSndCopPanic.Play(); SoundsCounter := SoundsCounter + 1; end;
            4: begin pSndVdvPanic.Play(); SoundsCounter := SoundsCounter + 1; end;
            else begin pSndManPanic.Play(); SoundsCounter := SoundsCounter + 1; end;
            end;

          if chel_type <> 4 then
          begin
          angle := RadToDeg(ArcTan2((y + 64) - (aliens[i].y + 16), (x + 48) - (aliens[i].x + 16)));
          state := PANIC;
          end;
          end;

        end;
      end;

    for i := 0 to Length(civis) - 1 do
      if civis[i].state = ALIEN then
      begin
        dist := sqrt(sqr(x - civis[i].x) + sqr(y - civis[i].y));
        if min_dist > dist then
        begin
          min_dist := dist;

          if min_dist < 125 then
          begin

          if (random(10) = 5) and (SoundsCounter < 6) then
          case chel_type of
          2: begin pSndWomanPanic.Play(); SoundsCounter := SoundsCounter + 1; end;
          3: begin pSndCopPanic.Play(); SoundsCounter := SoundsCounter + 1; end;
          4: begin pSndVdvPanic.Play(); SoundsCounter := SoundsCounter + 1; end;
          else begin pSndManPanic.Play(); SoundsCounter := SoundsCounter + 1; end;
          end;

          if chel_type <> 4 then
          begin
          angle := RadToDeg(ArcTan2((y + 64) - (civis[i].y + 64), (x + 48) - (civis[i].x + 48)));
          state := PANIC;
          end;
          end;

        end;
      end;

    end;//end state walk

    if (x > 734) then angle := random(110) + 125;
    if (x < -30) then angle := random(110) - 55;
    if (y > 510) then angle := random(180) + 180;
    if ((angle >= 270) and (angle <= 305)) then angle := random(55) + 305;
    if ((angle >= 235) and (angle <= 270))then angle := random(55) + 180;
    if (y < 0) then angle := random(180);
    if ((angle >= 55) and (angle <= 90)) then angle := random(55);
    if ((angle >= 90) and (angle <= 125))then angle := random(55) + 125;

    for i := 0 to 12 do
      for j := 0 to 9 do
      if (cells[i,j] <> 1) and (cells[i,j] <> -1) then
      begin

      if IntersectRect(GetCellRectF(i,j), RectF(round(x) + 33, round(y) + 12, 24, 69)) then
      begin
        angle := GenChelAngle;
        x := ox; y := oy;
      end;

      end;

      if UfoAllowMove and (state = WALK) and IntersectRect(RectF(mouse.iX - 37, mouse.iY - 15, 75, 30), RectF(round(x) + 33, round(y) + 12, 24, 69)) then
      begin

      if chel_type <> 4 then
      begin
        angle := GenChelAngle;
        state := PANIC;
      end;

      if (random(10) = 5) and (SoundsCounter < 7) then
      case chel_type of
      3: begin pSndCopPanic.Play(); SoundsCounter := SoundsCounter + 1; end;
      4: if random(10) = 5 then begin pSndVdvPanic.Play(); SoundsCounter := SoundsCounter + 1; end;
      end;

      end;

      if (state = PANIC) and (random(100) = 50) then state := WALK;

    ox := x;
    oy := y;
    end;//die
  end;

  SortCivArray;

  k := 0;
  while k < Length(blood) do
  with blood[k] do
  begin
    aframe := aframe + 0.15;
    if round(aframe) > 3 then
    begin
      if Length(blood)>1 then
      blood[k] := blood[Length(blood)-1];
      SetLength(blood,Length(blood)-1);
      k := k - 1;
    end;
    k := k + 1;
  end;

  k := 0;
  while k < Length(bullets) do
  with bullets[k] do
  begin

    flag := false;

    x := x + 5.5*cos(degtorad(angle));
    y := y + 5.5*sin(degtorad(angle));

    if (x + 4 < 0) or (y +4 < 0) or (x > 800) or (y > 600) then
      flag := true;

    for i := 0 to Length(aliens) - 1 do
    if aliens[i].lives > 0 then
    if IntersectRect(RectF(round(aliens[i].x),round(aliens[i].y) + 4 ,32, 18), RectF(round(x), round(y), 4, 4)) then
    begin
      flag := true;
      aliens[i].lives := aliens[i].lives - 40;
      CreateBlood(x - 4, y - 4, Color4(0, 255, 0, 255));
      if aliens[i].lives <= 0 then
      pSndAlienDie.Play()
      else
      pSndAlienPain.Play();
    end;

    for i := 0 to Length(eggs) - 1 do
    if eggs[i].lives > 0 then
    if IntersectRect(RectF(round(eggs[i].x) + 3,round(eggs[i].y) + 1 ,26, 30), RectF(round(x), round(y), 4, 4)) then
    begin
      flag := true;
      eggs[i].lives := eggs[i].lives - 20;
      CreateBlood(x - 4, y - 4, Color4(0, 255, 0, 255));
    end;

    for i := 0 to Length(civis) - 1 do
    if (civis[i].lives > 0) and (i <> bullets[k].owner) then
    if IntersectRect(RectF(round(civis[i].x) + 33,round(civis[i].y) + 12 ,24, 69), RectF(round(x), round(y), 4, 4)) then
    begin
      flag := true;

      civis[i].lives := civis[i].lives - 25;

      CreateBlood(x - 4, y - 4, Color4(255, 0, 0, 255));

      if civis[i].lives <= 0 then
      begin

      if civis[i].state = ALIEN then
      begin
        CreateSliz(round(civis[i].x + 50), round(civis[i].y + 68));
        aliens[length(aliens)-1].lives := 0;
        pSndAlienDie.Play();
      end else
      case civis[i].chel_type of
      2: pSndWomanDie.Play();
      4: pSndVdvDie.Play();
      else pSndManDie.Play();
      end;

      civis[i].state := DIE;

      end else
      case civis[i].chel_type of
      2: pSndWomanPain.Play();
      4: pSndVdvPain.Play();
      else pSndManPain.Play();
      end;

    end;

    for i := 0 to 12 do
    for j := 0 to 9 do
    if (cells[i,j] <> 1) and (cells[i,j] <> -1) and (cells[i,j] <> 17) then
    if IntersectRect(GetCellRectF(i,j), RectF(round(x), round(y), 4, 4)) then
      flag := true;

    if flag then
    begin
      if Length(bullets)>1 then
      bullets[k] := bullets[Length(bullets)-1];
      SetLength(bullets,Length(bullets)-1);
      k := k - 1;
    end;

    k := k + 1;
    end;

end;

procedure DrawGame;
var
i, j, base_frame, draw_flag : Integer;
tex : ITexture;
triangle : array[0..2] of TVertex2;
begin

 for i := 0 to 12 do
  for j := 0 to 9 do
  if cells[i, j] <> -1 then
    pTexHlam.Draw2D(i*64, j*64, 64, 64, 0, cells[i, j]);

  for i := 0 to Length(bullets) - 1 do
    pTexBullet.Draw2D_Simple(round(bullets[i].x), round(bullets[i].y));

  for i := 0 to Length(aliens) - 1 do
  begin
    if aliens[i].ugol then pRender2D.SetColorMix(Color4(0,0,0, 255)) else pRender2D.SetColorMix(Color4());
    pRender2D.DrawSpriteA(pTexSliz, Point2(round(aliens[i].x), round(aliens[i].y)), Point2(32, 32), round(aliens[i].aframe), 0, EF_COLORMIX);
  end;

  for i := Length(civis) - 1 downto 0 do
  begin
    case civis[i].state of
    WALK: base_frame := 4;
    PANIC: base_frame := 8;
    DIE: base_frame := 12;
    ALIEN: base_frame := 0;
    end;

    if civis[i].wait_shot < 5 then
    begin
      base_frame := 20;
      civis[i].aframe := 0;
    end;

    draw_flag := EF_DEFAULT;

    if ((civis[i].angle >= -90) and (civis[i].angle < 90)) or ((civis[i].angle > 270) and (civis[i].angle <= 450)) then  draw_flag := EF_FLIPX;

    case civis[i].chel_type of
    0: tex := pTexMan1;
    1: tex := pTexMan2;
    2: tex := pTexWoman;
    3: tex := pTexCop;
    4: tex := pTexVdv;
    end;

    if civis[i].ugol then pRender2D.SetColorMix(Color4(0,0,0, 255)) else pRender2D.SetColorMix(Color4());

    pRender2D.DrawSpriteA(tex, Point2(civis[i].x, civis[i].y), Point2(96, 96), base_frame + round(civis[i].aframe), 0, draw_flag or EF_COLORMIX);

    if civis[i].state = ALIEN then
    begin
     if draw_flag = EF_DEFAULT then
     pTexSliz.Draw2D(round(civis[i].x + 32 - round(abs(sin(counter/20)*5))), round(civis[i].y), 25 + round(abs(sin(counter/20)*10)), 42)
     else
     pTexSliz.Draw2D(round(civis[i].x + 39 - round(abs(sin(counter/20)*5))), round(civis[i].y), 25 + round(abs(sin(counter/20)*10)), 42);
    end;
  end;

  for i := 0 to Length(eggs) - 1 do
  begin
    if eggs[i].lives > 0 then
    pTexEgg.Draw2D(round(eggs[i].x - sin(eggs[i].time/15)*5), round(eggs[i].y), round(32 + sin(eggs[i].time/15)*10), 32)
    else
    begin
    if eggs[i].ugol then pRender2D.SetColorMix(Color4(0,0,0, 255)) else pRender2D.SetColorMix(Color4());
    pRender2D.DrawSpriteA(pTexEgg, Point2(round(eggs[i].x), round(eggs[i].y)), Point2(32, 32), 1, 0, EF_COLORMIX);
    end;
  end;

  for i := 0 to Length(money) - 1 do
    pTexCoin.Draw2D(round(money[i].x), round(money[i].y), 16, 16, 0, round(money[i].aframe));

  for i := 0 to Length(blood) - 1 do
  with blood[i] do
  begin
    pRender2D.SetColorMix(color);
    pRender2D.DrawSpriteA(pTexBlood, Point2(x, y), Point2(16*scale, 16*scale), round(aframe), angle, EF_COLORMIX or EF_BLEND);
  end;

  if UfoX > -50 then
  begin
    pRender2D.SetBlendMode(EBF_ADD);

    triangle[0] := Vertex2(UfoX + 64, UfoY + 64, 0, 0, 0.85, 0.85, 0, 0.588);
    triangle[1] := Vertex2(mouse.iX - 50, mouse.iY, 0, 0, 0.85, 0.85, 0, 0.588);
    triangle[2] := Vertex2(mouse.iX + 50, mouse.iY, 0, 0, 0.85, 0.85, 0, 0.588);
    pRender2D.DrawPolygon(nil, @triangle[0], 3, PF_FILL);

    triangle[0] := Vertex2(UfoX + 64, UfoY + 64, 0, 0, 0.85, 0.85, 0, 0.588);
    triangle[1] := Vertex2(mouse.iX, mouse.iY - 25, 0, 0, 0.85, 0.85, 0, 0.588);
    triangle[2] := Vertex2(mouse.iX, mouse.iY + 25, 0, 0, 0.85, 0.85, 0, 0.588);
    pRender2D.DrawPolygon(nil, @triangle[0], 3, PF_FILL);

    pRender2D.DrawEllipse(Point2(mouse.iX, mouse.iY), Point2(50, 25), 32, Color4(217, 217, 0, 150), PF_FILL);
    pRender2D.SetBlendMode(EBF_NORMAL);
  end;

  if LaserCounter < 50 then
  begin
    pRender2D.DrawEllipse(Point2(UfoX + 64, UfoY + 260), Point2(55, 15), 32, Color4(255, 0, 0, 255), PF_FILL);
    pRender2D.DrawRect(RectF(round(UfoX + 64) - round(110/2), round(UfoY + 100), 110, 160), Color4(255, 0, 0, 125), PF_FILL);
    pRender2D.DrawRect(RectF(round(UfoX + 64) - round(LaserCounter*2.2/2), round(UfoY + 100), round(LaserCounter*2.2), 160), Color4(255, 0, 0, 255), PF_FILL);
    pRender2D.DrawRect(RectF(round(UfoX + 64) - round(LaserCounter*0.8/2), round(UfoY + 100), round(LaserCounter*0.8), 160), Color4(255, 255, 255, 155), PF_FILL);
  end;

  if round(counter/10.0) mod 2 = 0 then
  tex := pTextexUfo1
  else
  tex := pTextexUfo2;

  pRender2D.DrawMesh(pMdlUfo, tex, Point2(UfoX + 64, UfoY + 64), Point3(128, 128, 100.0), Point3(1.0, 0.0, 0.0), 35.0 + sin(counter/25)*10.0, false);

end;

////////////////////////////////////////////////////////////

procedure Init(pParametr : Pointer); stdcall;
var
pRender : IRender;
begin

  randomize;

	pEngineCore.GetSubsystem(ESS_RESOURCE_MANAGER,IEngineSubSystem(pResMan));
	pEngineCore.GetSubsystem(ESS_RENDER,IEngineSubSystem(pRender));
	pEngineCore.GetSubsystem(ESS_INPUT,IEngineSubSystem(pInput));

  pInput.Configure(ICF_HIDE_CURSOR);

	pRender.GetRender2D(pRender2D);
  pRender.SetClearColor(Color4(0,0,0,0));
  pRender2D.SetResolutionCorrection(800, 600);

  pResMan.Load(RESOURCE_PATH+'font.dft',IEngBaseObj(pFont));
  pResMan.Load(RESOURCE_PATH+'logo.png', IEngBaseObj(pTexLogo), TLF_FILTERING_NONE);
  pResMan.Load(RESOURCE_PATH+'trava.bmp', IEngBaseObj(pTexFon), TLF_FILTERING_NONE);
  pResMan.Load(RESOURCE_PATH+'Chel1.png', IEngBaseObj(pTexMan1), TLF_FILTERING_NONE);
  pResMan.Load(RESOURCE_PATH+'Chel2.png', IEngBaseObj(pTexMan2), TLF_FILTERING_NONE);
  pResMan.Load(RESOURCE_PATH+'cop.png', IEngBaseObj(pTexCop), TLF_FILTERING_NONE);
  pResMan.Load(RESOURCE_PATH+'coin.png', IEngBaseObj(pTexCoin), TLF_FILTERING_NONE);
  pResMan.Load(RESOURCE_PATH+'woman.png', IEngBaseObj(pTexWoman), TLF_FILTERING_NONE);
  pResMan.Load(RESOURCE_PATH+'cursor.png', IEngBaseObj(pTexCursor), TLF_FILTERING_NONE);
  pResMan.Load(RESOURCE_PATH+'hlam.png', IEngBaseObj(pTexHlam), TLF_FILTERING_NONE);
  pResMan.Load(RESOURCE_PATH+'sliz.png', IEngBaseObj(pTexSliz), TLF_FILTERING_NONE);
  pResMan.Load(RESOURCE_PATH+'UFO.png', IEngBaseObj(pTexUfo), TLF_FILTERING_NONE);
  pResMan.Load(RESOURCE_PATH+'egg.png', IEngBaseObj(pTexEgg), TLF_FILTERING_NONE);
  pResMan.Load(RESOURCE_PATH+'vdv.png', IEngBaseObj(pTexVdv), TLF_FILTERING_NONE);
  pResMan.Load(RESOURCE_PATH+'blood.png', IEngBaseObj(pTexBlood), TLF_FILTERING_NONE);
  pResMan.Load(RESOURCE_PATH+'bullet.png', IEngBaseObj(pTexBullet), TLF_FILTERING_NONE);
  pResMan.Load(RESOURCE_PATH+'ufo1.bmp', IEngBaseObj(pTextexUfo1), TLF_FILTERING_NONE or TLF_COORDS_REPEAT);
  pResMan.Load(RESOURCE_PATH+'ufo2.bmp', IEngBaseObj(pTextexUfo2), TLF_FILTERING_NONE or TLF_COORDS_REPEAT);
  pResMan.Load(RESOURCE_PATH+'ufo.dmd', IEngBaseObj(pMdlUfo));
  pResMan.Load(RESOURCE_PATH+'team.jpg', IEngBaseObj(pTexTeam));

  pResMan.Load(RESOURCE_PATH+'chpok.wav', IEngBaseObj(pSndChpok));
  pResMan.Load(RESOURCE_PATH+'laser.wav', IEngBaseObj(pSndLaser));
  pResMan.Load(RESOURCE_PATH+'netu.wav', IEngBaseObj(pSndNetu));
  pResMan.Load(RESOURCE_PATH+'teleport.wav', IEngBaseObj(pSndTeleport));
  pResMan.Load(RESOURCE_PATH+'alien_talk.wav', IEngBaseObj(pSndAlienTalk));
  pResMan.Load(RESOURCE_PATH+'shoot.wav', IEngBaseObj(pSndShot));
  pResMan.Load(RESOURCE_PATH+'alien_die.wav', IEngBaseObj(pSndAlienDie));
  pResMan.Load(RESOURCE_PATH+'alien_pain.wav', IEngBaseObj(pSndAlienPain));
  pResMan.Load(RESOURCE_PATH+'alien_brain.wav', IEngBaseObj(pSndAlienBrain));
  pResMan.Load(RESOURCE_PATH+'man_pain.wav', IEngBaseObj(pSndManPain));
  pResMan.Load(RESOURCE_PATH+'man_die.wav', IEngBaseObj(pSndManDie));
  pResMan.Load(RESOURCE_PATH+'woman_pain.wav', IEngBaseObj(pSndWomanPain));
  pResMan.Load(RESOURCE_PATH+'woman_die.wav', IEngBaseObj(pSndWomanDie));
  pResMan.Load(RESOURCE_PATH+'vdv_pain.wav', IEngBaseObj(pSndVdvPain));
  pResMan.Load(RESOURCE_PATH+'vdv_die.wav', IEngBaseObj(pSndVdvDie));
  pResMan.Load(RESOURCE_PATH+'man_panic.wav', IEngBaseObj(pSndManPanic));
  pResMan.Load(RESOURCE_PATH+'woman_panic.wav', IEngBaseObj(pSndWomanPanic));
  pResMan.Load(RESOURCE_PATH+'cop_normal.wav', IEngBaseObj(pSndCopNormal));
  pResMan.Load(RESOURCE_PATH+'cop_panic.wav', IEngBaseObj(pSndCopPanic));
  pResMan.Load(RESOURCE_PATH+'vdv_normal.wav', IEngBaseObj(pSndVdvNormal));
  pResMan.Load(RESOURCE_PATH+'vdv_panic.wav', IEngBaseObj(pSndVdvPanic));

  pResMan.Load(PAnsiChar(AnsiString(ExtractFilePath(ParamStr(0)) + '\music.mp3')), IEngBaseObj(pMusic));
  pMusic.Play();

  pTexMan1.SetFrameSize(32,32);
  pTexCoin.SetFrameSize(16,16);
  pTexBlood.SetFrameSize(16,16);
  pTexVdv.SetFrameSize(32,32);
  pTexEgg.SetFrameSize(32,32);
  pTexMan2.SetFrameSize(32,32);
  pTexWoman.SetFrameSize(32,32);
  pTexCop.SetFrameSize(32,32);
  pTexCursor.SetFrameSize(16,16);
  pTexHlam.SetFrameSize(32,32);
  pTexUfo.SetFrameSize(32,32);
  pTexSliz.SetFrameSize(32,32);

  GenGame();
end;

procedure Free(pParametr : Pointer); stdcall;
begin

  pResMan.FreeResource(IEngBaseObj(pTexLogo));
  pResMan.FreeResource(IEngBaseObj(pTexFon));
  pResMan.FreeResource(IEngBaseObj(pTexMan1));
  pResMan.FreeResource(IEngBaseObj(pTexMan2));
  pResMan.FreeResource(IEngBaseObj(pTexCop));
  pResMan.FreeResource(IEngBaseObj(pTexWoman));
  pResMan.FreeResource(IEngBaseObj(pTexSliz));
  pResMan.FreeResource(IEngBaseObj(pTexUfo));
  pResMan.FreeResource(IEngBaseObj(pTexBlood));
  pResMan.FreeResource(IEngBaseObj(pTexVdv));
  pResMan.FreeResource(IEngBaseObj(pTexEgg));
  pResMan.FreeResource(IEngBaseObj(pTexCoin));
  pResMan.FreeResource(IEngBaseObj(pTexBullet));
  pResMan.FreeResource(IEngBaseObj(pTextexUfo1));
  pResMan.FreeResource(IEngBaseObj(pTextexUfo2));
  pResMan.FreeResource(IEngBaseObj(pMdlUfo));
  pResMan.FreeResource(IEngBaseObj(pTexCursor));
  pResMan.FreeResource(IEngBaseObj(pTexHlam));
  pResMan.FreeResource(IEngBaseObj(pFont));
  pResMan.FreeResource(IEngBaseObj(pSndChpok));
  pResMan.FreeResource(IEngBaseObj(pSndLaser));
  pResMan.FreeResource(IEngBaseObj(pSndNetu));
  pResMan.FreeResource(IEngBaseObj(pSndTeleport));
  pResMan.FreeResource(IEngBaseObj(pSndAlienTalk));
  pResMan.FreeResource(IEngBaseObj(pSndShot));
  pResMan.FreeResource(IEngBaseObj(pSndAlienPain));
  pResMan.FreeResource(IEngBaseObj(pSndAlienDie));
  pResMan.FreeResource(IEngBaseObj(pSndAlienBrain));
  pResMan.FreeResource(IEngBaseObj(pSndManPain));
  pResMan.FreeResource(IEngBaseObj(pSndManDie));
  pResMan.FreeResource(IEngBaseObj(pSndWomanPain));
  pResMan.FreeResource(IEngBaseObj(pSndWomanDie));
  pResMan.FreeResource(IEngBaseObj(pSndVdvPain));
  pResMan.FreeResource(IEngBaseObj(pSndVdvDie));
  pResMan.FreeResource(IEngBaseObj(pSndManPanic));
  pResMan.FreeResource(IEngBaseObj(pSndWomanPanic));
  pResMan.FreeResource(IEngBaseObj(pSndCopNormal));
  pResMan.FreeResource(IEngBaseObj(pSndCopPanic));
  pResMan.FreeResource(IEngBaseObj(pSndVdvNormal));
  pResMan.FreeResource(IEngBaseObj(pSndVdvPanic));
  pResMan.FreeResource(IEngBaseObj(pMusic));
  pResMan.FreeResource(IEngBaseObj(pTexTeam));

  pTexTeam := nil;
  pMusic := nil;
  pSndManPanic := nil;
  pSndWomanPanic := nil;
  pSndCopNormal := nil;
  pSndCopPanic := nil;
  pSndVdvNormal := nil;
  pSndVdvPanic := nil;
  pSndManPain := nil;
  pSndManDie := nil;
  pSndWomanPain := nil;
  pSndWomanDie := nil;
  pSndVdvPain := nil;
  pSndVdvDie := nil;
  pSndAlienPain := nil;
  pSndAlienDie := nil;
  pSndAlienBrain := nil;
  pSndShot := nil;
  pSndAlienTalk := nil;
  pSndTeleport := nil;
  pSndNetu := nil;
  pSndLaser := nil;
  pSndChpok:=nil;
  pTexLogo := nil;
  pTexFon := nil;
  pTexMan1 := nil;
  pTexCop := nil;
  pTexWoman := nil;
  pTexMan2 := nil;
  pTexSliz := nil;
  pTexUfo := nil;
  pTexCoin := nil;
  pTexVdv := nil;
  pTexBullet := nil;
  pTexEgg := nil;
  pTexBlood := nil;
  pTextexUfo1 := nil;
  pTextexUfo2 := nil;
  pTexCursor := nil;
  pTexHlam := nil;
  pFont := nil;
  pMdlUfo := nil;

  pEngineCore := nil;
  pResMan     := nil;
  pRender2D   := nil;
  pInput      := nil;
end;

procedure Process(pParametr : Pointer); stdcall;
var
pressed, flag : boolean;
i, j : Integer;
begin

if not GameEnd then
begin
  pInput.GetMouseStates(mouse);

  if not isGame then
  begin

  pInput.GetKey(KEY_RETURN, pressed);

  if pressed then
  begin
    CurrentLevel := START_LEVEL;
    GenGame(CurrentLevel);
    isGame := true;
    GameEnd := false;
    Score := 0;
  end;

  pInput.GetKey(KEY_ESCAPE, pressed);
  if pressed and (counter>25) then pEngineCore.QuitEngine;

  end else
  if counter > 255 then
  begin
   if not mouse.bLeftButton and ClickFlagL then
   begin//LeftClick
      if CurHint < 4 then CurHint := CurHint + 1;

      if CurHint = 1 then
      begin
      UfoAllowMove := true;
      pSndAlienTalk.Play();
      end;

      if (CurHint > 1) and (UfoX>0) then
      begin
      flag := false;
      for i := 0 to 12 do
      for j := 0 to 9 do
      if(cells[i,j] <> -1) and (cells[i,j] <> 1) then
      if IntersectRect(GetCellRectF(i,j), RectF(round(UfoX + 64) - 16, round(UfoY+128) -12, 32, 18)) then
      begin
        pSndNetu.Play();
        flag := true;
      end;
      if not flag and (SlizCounter >= 1) then
      begin
        CreateSliz(round(UfoX + 64), round(UfoY+128));
        CreateBlood(round(UfoX + 64) - 32, round(UfoY+128) -32, Color4(255, 255, 255, 125), 4.0);
        SlizCounter := SlizCounter - 1;
        pSndTeleport.Play();
      end
      else pSndNetu.Play();

      end;
   end;
   ClickFlagL := mouse.bLeftButton;

   if not mouse.bRightButton and ClickFlagR then
   begin//RightClick
      if CurHint = 4 then CurHint := CurHint + 1;
      if LaserArmed and (UfoX>0)then
      begin
        pSndLaser.Play();
        LaserCounter := 0;
        LaserArmed := false;

        for i := 0 to Length(civis) - 1 do
        if (civis[i].state <> DIE) and
        IntersectRect(RectF(round(UfoX + 64) - round(110/2), round(UfoY + 100), 110, 160),
        RectF(round(civis[i].x) + 33, round(civis[i].y) + 12, 24, 69)) then
        begin
          civis[i].lives := 0;
          civis[i].ugol := true;
          if civis[i].state = ALIEN then
          begin
            CreateSliz(round(civis[i].x + 50), round(civis[i].y + 68));
            aliens[length(aliens)-1].lives := 0;
            aliens[length(aliens)-1].ugol := true;
          end;
          civis[i].state := DIE;
        end;
        for i := 0 to Length(aliens) - 1 do
        if (aliens[i].lives >0) and
        IntersectRect(RectF(round(UfoX + 64) - round(110/2), round(UfoY + 100), 110, 160),RectF(round(aliens[i].x), round(aliens[i].y), 32, 19)) then
        begin
          aliens[i].lives := 0;
          aliens[i].ugol := true;
        end;
        for i := 0 to Length(eggs) - 1 do
        if (eggs[i].lives >0) and
        IntersectRect(RectF(round(UfoX + 64) - round(110/2), round(UfoY + 100), 110, 160),RectF(round(eggs[i].x), round(eggs[i].y), 25, 30)) then
        begin
          eggs[i].lives := 0;
          eggs[i].ugol := true;
        end;

      end else pSndNetu.Play();

   end;
   ClickFlagR := mouse.bRightButton;

    pInput.GetKey(KEY_ESCAPE, pressed);
    if pressed then
    begin
    GenGame();
    isGame := false;
    end;

    //Win
    flag := true;

    for i:=0 to Length(civis)-1 do
    if(civis[i].state <> ALIEN) and (civis[i].state <> DIE) then flag := false;

    if flag and (Win = 0) then Win := 1;

    if Win > 0 then
    begin
      Win := Win + 1;
      if Win > 150 then
      begin
        CurrentLevel := CurrentLevel + 1;
        GenGame(CurrentLevel);
        if score > BestScore then BestScore := score;
      end;
    end;

    //Loose
   if win = 0 then
   begin
    flag := true;

    for i:=0 to Length(civis)-1 do
    if(civis[i].state = ALIEN) then flag := false;

    for i:=0 to Length(eggs)-1 do
    if(eggs[i].lives > 0) then flag := false;

    for i:=0 to Length(aliens)-1 do
    if(aliens[i].lives > 0) then flag := false;

    if SlizCounter > 0 then flag := false;

    if flag and (Loose = 0) then Loose := 1;

    if Loose > 0 then
    begin
    Loose := Loose + 1;

      pInput.GetKey(KEY_RETURN, pressed);

      if pressed then
      begin
        GenGame(CurrentLevel);
        isGame := true;
        Score := 0;
      end;

    end;

   end;

  end;

  ProcessGame;
end else
begin
  pInput.GetKey(KEY_ESCAPE, pressed);
  if pressed then
  begin
  GenGame();
  isGame := false;
  GameEnd := false;
  end;
end;

	counter := counter + 1;

end;

procedure Render(pParametr : Pointer); stdcall;
var w, h : Cardinal;
i, j: Integer;
ts : PAnsiChar;
angle : Integer;
begin

	pRender2D.Begin2D();

if not GameEnd then
begin
  for i := 0 to 3 do
    for j := 0 to 2 do
      pTexFon.Draw2D(i*256, j*256, 256, 256);

  DrawGame;

  if not isGame then
  begin

	pTexLogo.Draw2D(0, 50, 800, 200);
  pTexUfo.Draw2D( -8, 35, 64, 64, -round(35 + sin(counter/25)*10), round(counter/10.0) mod 2);

  pFont.GetDimensions('  !', w, h);
	pFont.Draw2D(trunc((800 - w)/2), 285, '  !', Color4());

  ts := PAnsiChar(AnsiString(' : ' + IntToStr(BestScore)));
  pFont.GetDimensions(ts, w, h);
	pFont.Draw2D(trunc((800 - w)/2), 320, ts, Color4());

  pFont.GetDimensions(' "Enter"   !!!', w, h);
  pRender2D.DrawRect(Rectf(trunc((800 - w)/2), 420, w, h), Color4(), PF_FILL);
  pRender2D.DrawRect(Rectf(trunc((800 - w)/2), 420, w, h), Color4(0,0,0,255));
	pFont.Draw2D(trunc((800 - w)/2),420,' "Enter"   !!!', Color4(255, 0, 0, 255));

  pFont.GetDimensions(' : DRON, }{@ & artyks.', w, h);
	pFont.Draw2D(trunc((800 - w)/2),500,' : DRON, }{@ & artyks.', Color4());

  pFont.GetDimensions('http://dron.deeprosoft.com/', w, h);
	pFont.Draw2D(trunc((800 - w)/2), 530, 'http://dron.deeprosoft.com/', Color4());

  pFont.GetDimensions('(c) 2010   .', w, h);
	pFont.Draw2D(trunc((800 - w)/2), 560, '(c) 2010   .', Color4());

  end else
  begin

  if counter < 255 then
  begin
    pFont.SetScale(counter/45);
    ts := PAnsiChar(AnsiString(' ' + IntToStr(CurrentLevel)));
    angle := round(counter*5.5);
    if angle > 360 then angle := 0;
    pFont.GetDimensions(ts, w, h);
	  pFont.Draw2D(trunc((800 - w)/2),trunc((600 - h)/2),ts, Color4(255, 255, 255, 255 - counter), angle);
    pFont.SetScale(1.0);
  end else
  begin
   if(CurHint = 0) and (UfoX < -50) then
    pTexCursor.Draw2D(mouse.iX, mouse.iY, 32, 32, 0, round(counter/5.0) mod 8);
   ts := PAnsiChar(AnsiString(': ' + IntToStr(score)));
   pFont.GetDimensions(ts, w, h);
   pFont.Draw2D(800 - w, 5, ts, Color4());

   pFont.Draw2D(5, 5, PAnsiChar(AnsiString(' : ' + IntToStr(SlizCounter))), Color4());
   pTexSliz.Draw2D(0,-3,25,25,-35);

   if LaserArmed and (counter mod 20 > 10) then
   begin
    pFont.GetDimensions(' ', w, h);
    pFont.Draw2D((800 - w) div 2, 5, ' ', Color4(255, 0, 0, 125));
   end;

  end;

  if(CurrentLevel = 1) and (counter > 255) and (CurHint < 5) then
  begin

    case CurHint of
    0: ts := '        !';
    1: ts := '  -   .';
    2: ts := '   ,    !';
    3: ts := '     .';
    4: ts := '  -  .';
    end;

    pFont.GetDimensions(ts, w, h);

    pRender2D.DrawRect(Rectf(trunc((800 - w)/2), 100, w, h), Color4(), PF_FILL);
    pRender2D.DrawRect(Rectf(trunc((800 - w)/2), 100, w, h), Color4(0,0,0,255));

	  pFont.Draw2D(trunc((800 - w)/2), 100, ts, Color4(255, 0, 0, 255));

    pTexUfo.Draw2D(trunc((800 - w)/2) - 64, 50, 96, 96, -round(35 + sin(counter/25)*10), round(counter/10.0) mod 2);

  end;

  if Win > 0 then
  begin
    ts := '!  ,   !';
    pFont.GetDimensions(ts, w, h);
    pRender2D.DrawRect(Rectf(trunc((800 - w)/2), 300, w, h), Color4(), PF_FILL);
    pRender2D.DrawRect(Rectf(trunc((800 - w)/2), 300, w, h), Color4(0,0,0,255));
	  pFont.Draw2D(trunc((800 - w)/2), 300, ts, Color4(255, 0, 0, 255));
    pTexUfo.Draw2D(trunc((800 - w)/2) - 64, 250, 96, 96, -round(35 + sin(counter/25)*10), round(counter/10.0) mod 2);
  end else
  if Loose > 0 then
  begin
    ts := '  ,   .  !';
    pFont.GetDimensions(ts, w, h);
    pRender2D.DrawRect(Rectf(trunc((800 - w)/2), 300, w, h), Color4(), PF_FILL);
    pRender2D.DrawRect(Rectf(trunc((800 - w)/2), 300, w, h), Color4(0,0,0,255));
	  pFont.Draw2D(trunc((800 - w)/2), 300, ts, Color4(255, 0, 0, 255));
    pTexUfo.Draw2D(trunc((800 - w)/2) - 64, 250, 96, 96, -round(35 + sin(counter/25)*10), round(counter/10.0) mod 2);

    ts := '"Enter" -  . "ESC" -  .';
    pFont.GetDimensions(ts, w, h);
    pFont.Draw2D(trunc((800 - w)/2),420,ts, Color4());
  end;

  end;
end else
begin
 pTexTeam.Draw2D_Simple(0,0);
end;

	pRender2D.End2D();

end;

begin

  if(GetEngine(DLL_PATH, pEngineCore)) then
  begin

    if(SUCCEEDED(pEngineCore.InitializeEngine(0, APP_CAPTION, EngWindow(800, 600, true, true)))) then
      begin

        pEngineCore.AddProcedure(EPT_INIT,     @Init);
        pEngineCore.AddProcedure(EPT_FREE,     @Free);
        pEngineCore.AddProcedure(EPT_PROCESS,  @Process);
        pEngineCore.AddProcedure(EPT_RENDER,   @Render);

        pEngineCore.StartEngine();

      end;

		FreeEngine();

  end
    else
      MessageBox(0,'Couldn''t load '+DLL_PATH+'!', APP_CAPTION, MB_OK or MB_ICONERROR or MB_SETFOREGROUND);
end.
