unit dl_text;

interface

uses
  classes, sysutils, powerd3d, agfunit;

type
  TDLFont = class
  private
    Shifts : array [0..255] of Byte;
    HShift : byte;
    WShift : byte;
    FontImage : TAGFImage;
    FPowerGraph : TPowerGraph;
  public
    Constructor Create(_PowerGraph : TPowerGraph);
    Destructor Destroy; override;
            //    
    procedure LoadShifts( SFileName : String );
    procedure LoadImageFromStream( St : TStream );
    procedure LoadImageFromFile( FileName : String );
            //   ,  , 
    Procedure TextList( strin : array of String; x, y : word; color : Cardinal ); overload;
            //  StringList',   
    Procedure TextList( strin : TstringList; x, y : word; color : Cardinal ); overload;
            //       , W -    
    Procedure TextListAuto( strin : array of String; x, y : word; W : word; color : Cardinal ); overload;
    Procedure TextListAuto( strin : TStringList; x, y : word; W : word; color : Cardinal ); overload;
            //    Y . .
    Procedure TextCentered( strin : String; x, y : word; color : Cardinal ); overload;
    Procedure TextCentered( strin : String; x, y, scale : word; color : Cardinal ); overload;
    Procedure TextCenteredCursor( strin : String; x, y, scale : word; color : Cardinal; visible : boolean );
            //   ,    
    Procedure TextListCentered( strin : array of String; x, y : word; color : Cardinal ); overload;
    Procedure TextListCentered( strin : TStringList; x, y : word; scale : byte; color : Cardinal ); overload;
            //  
    Procedure Text( strin : string; x, y : word; color : Cardinal ); overload;
    Function TextWidth( strin : string ) : integer;    
            //   
    Procedure Text( strin : integer; x, y : word; color : Cardinal ); overload;
            //     
    Procedure Text( strin : double; x, y : word; color : Cardinal ); overload;
            //  ,     
    Procedure TextBack( strin : string; x, y : word; color : Cardinal ); overload;
            //   ,     
    Procedure TextBack( strin : integer; x, y : word; color : Cardinal ); overload;
            //     ,     
    Procedure TextBack( strin : double; x, y : word; color : Cardinal ); overload;
            //     
  end;

implementation

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
Constructor TDLFont.Create(_PowerGraph : TPowerGraph);
begin
  inherited Create;

  FPowerGraph:= _PowerGraph;
end;

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
Destructor TDLFont.Destroy;
begin
  if FontImage<>nil then FontImage.Finalize;
  
  inherited Destroy;
end;

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
procedure TDLFont.LoadShifts( SFileName : String );
var
  f : file;
begin
  if not FPowerGraph.Initialized then exit;

  AssignFile(f, SFileName);
  Reset(f, 1);
  BlockRead(f, HShift, 1);
  BlockRead(f, WShift, 1);
  BlockRead(f, Shifts[0], 256);
  CloseFile(f);
end;

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
procedure TDLFont.LoadImageFromStream( St : TStream );
begin
  if not FPowerGraph.Initialized then exit;

  FontImage:= TAGFImage.create();
  FontImage.LoadFromStream(FPowerGraph.D3DDevice8, St, WShift, HShift, 256, 256, FMT_A8R8G8B8);
end;

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
procedure TDLFont.LoadImageFromFile( FileName : String );
begin
  if not FPowerGraph.Initialized then exit;

  FontImage:= TAGFImage.create();
  FontImage.LoadFromFile(FPowerGraph.D3DDevice8, FileName, WShift, HShift, 256, 256, FMT_A8R8G8B8);
end;

//-----------------------------------------------------------------------------
//Procedure to draw array of Strings
//-----------------------------------------------------------------------------
Procedure TDLFont.TextList( strin : array of String; x, y : word; color : Cardinal );
var
  u, tekX, TekY : word;
Begin
  tekY:= 0;
  repeat
    tekX:= x;
    for u:= 1 to length( strin[TekY] ) do
    Try
      FPowerGraph.RenderEffectCol( FontImage, tekX, y + TekY*20, color, ord(strin[TekY][u]), effectSrcAlpha );
      inc( tekX, Shifts[ord(strin[TekY][u])] );
    Except
      //
    End;
    inc(TekY,1);
  until TekY>high(Strin);
End;

//-----------------------------------------------------------------------------
//Procedure to draw StringList
//-----------------------------------------------------------------------------
Procedure TDLFont.TextList( strin : TStringList; x, y : word; color : Cardinal );
var
  u, tekX, TekY : word;
Begin
  tekY:= 0;
  repeat
    tekX:= x;
    for u:= 1 to length( strin.strings[TekY] ) do
    Try
      FPowerGraph.RenderEffectCol( FontImage, tekX, y + TekY*20, color, ord(strin.strings[TekY][u]), effectSrcAlpha or effectDiffuseAlpha );
      inc( tekX, Shifts[ord(strin.strings[TekY][u])] );
    Except
      //
    End;
    inc(TekY,1);
  until TekY>(Strin.Count-1);
End;

//-----------------------------------------------------------------------------
//Procedure to draw formatted Stringlist
//-----------------------------------------------------------------------------
Procedure TDLFont.TextListAuto( strin : array of String; x, y : word; W : word; color : Cardinal );
var
  u, tekX, TekY : word;
  xx : word;
  Prob : word;
  probN : word;
Begin
  tekY:= 0;
  repeat
    xx:= 0;
    probN:= 0;
    for u:= 1 to length( strin[TekY] ) do
     if strin[TekY][u]<>' ' then
     inc(xx,Shifts[ord(strin[TekY][u])])
     else inc(ProbN);

    if probN>0 then
    prob:= (w-xx) div probN;

    if prob > 20 then prob:= 8;

    tekX:= x;
    for u:= 1 to length( strin[TekY] ) do
    if strin[TekY][u]<>' ' then
    Try
      FPowerGraph.RenderEffectCol( FontImage, tekX, y + TekY*20, color, ord(strin[TekY][u]), effectSrcAlpha or effectDiffuseAlpha );
      inc( tekX, Shifts[ord(strin[TekY][u])] );
    Except
      //
    End
    else inc( tekX, prob );
    inc(TekY,1);
  until TekY>high(Strin);
End;

//-----------------------------------------------------------------------------
//Procedure to draw formatted Stringlist
//-----------------------------------------------------------------------------
Procedure TDLFont.TextListAuto( strin : TStringList; x, y : word; W : word; color : Cardinal );
var
  u, tekX, TekY : word;
  xx : word;
  Prob : word;
  probN : word;
Begin
  tekY:= 0;
  repeat
    xx:=0;
    probN:=0;
    for u:= 1 to length( strin[TekY] ) do
     if strin.Strings[TekY][u]<>' ' then
     inc(xx,Shifts[ord(strin.Strings[TekY][u])])
     else inc(ProbN);

    if probN>0 then
    prob:=(w-xx) div probN;

    if prob>20 then prob:=8;

    tekX:= x;
    for u:= 1 to length( strin.Strings[TekY] ) do
    if strin.Strings[TekY][u]<>' ' then
    Try
      FPowerGraph.RenderEffectCol( FontImage, tekX, y + TekY*20, color, ord(strin.Strings[TekY][u]), effectSrcAlpha or effectDiffuseAlpha );
      inc( tekX, Shifts[ord(strin.Strings[TekY][u])] );
    Except
      //
    End
    else inc( tekX, prob );
    inc(TekY,1);
  until TekY>Strin.Count-1;
End;

//-----------------------------------------------------------------------------
//Procedure to draw of centered String
//-----------------------------------------------------------------------------
Procedure TDLFont.TextCentered( strin : String; x, y : word; color : Cardinal );
var
  u, tekX : word;
  lineLenght : word;
Begin
  lineLenght:=0;
  for u:= 1 to length( strin ) do
  inc(lineLenght, Shifts[ord(strin[u])]);
  lineLenght:=lineLenght div 2;

  tekX:= x;
  for u:= 1 to length( strin ) do
  Try
    FPowerGraph.RenderEffectCol( FontImage, tekX - lineLenght, y, color, ord(strin[u]), effectSrcAlpha or effectDiffuseAlpha );
    inc( tekX, Shifts[ord(strin[u])] );
  Except
    //
  End;
End;

//-----------------------------------------------------------------------------
//Procedure to draw of centered String
//-----------------------------------------------------------------------------
Procedure TDLFont.TextCenteredCursor( strin : String; x, y, scale : word; color : Cardinal; visible : Boolean);
var
  u, tekX : word;
  lineLenght : word;
Begin
  if visible then strin:= strin + 'I' else strin:= strin + ' ';
  lineLenght:=0;
  for u:= 1 to length( strin ) do
  inc(lineLenght, round((Shifts[ord(strin[u])] / 100) * scale));
  lineLenght:=lineLenght div 2;

  tekX:= x;
  for u:= 1 to length( strin ) do
  Try
    FPowerGraph.RenderEffectCol( FontImage, tekX - lineLenght, y, round((256/100)*scale), color, ord(strin[u]), effectSrcAlpha or effectDiffuseAlpha );
    inc( tekX, round((Shifts[ord(strin[u])] / 100) * scale) );
  Except
    //
  End;
End;

//-----------------------------------------------------------------------------
//Procedure to draw of centered String
//-----------------------------------------------------------------------------
Procedure TDLFont.TextCentered( strin : String; x, y, scale : word; color : Cardinal );
var
  u, tekX : word;
  lineLenght : word;
Begin
  lineLenght:=0;
  for u:= 1 to length( strin ) do
  inc(lineLenght, round((Shifts[ord(strin[u])] / 100) * scale));
  lineLenght:=lineLenght div 2;

  tekX:= x;
  for u:= 1 to length( strin ) do
  Try
    FPowerGraph.RenderEffectCol( FontImage, tekX - lineLenght, y, round((256/100)*scale), color, ord(strin[u]), effectSrcAlpha or effectDiffuseAlpha );
    inc( tekX, round((Shifts[ord(strin[u])] / 100) * scale) );
  Except
    //
  End;
End;

//-----------------------------------------------------------------------------
//Procedure to draw of centered StringList
//-----------------------------------------------------------------------------
Procedure TDLFont.TextListCentered( strin : array of String; x, y : word; color : Cardinal );
var
  u, tekX, TekY : word;
  lineLenght : word;
Begin
  tekY:= 0;
  repeat
    lineLenght:=0;
    for u:= 1 to length( strin[TekY] ) do
    inc(lineLenght, Shifts[ord(strin[TekY][u])]);
    lineLenght:=lineLenght div 2;

    tekX:= x;
    for u:= 1 to length( strin[TekY] ) do
    Try
      FPowerGraph.RenderEffectCol( FontImage, tekX - lineLenght, y + TekY*20, color, ord(strin[TekY][u]), effectSrcAlpha or effectDiffuseAlpha );
      inc( tekX, Shifts[ord(strin[TekY][u])] );
    Except
      //
    End;
    inc(TekY,1);
  until TekY>high(Strin);
End;

//-----------------------------------------------------------------------------
//Procedure to draw of centered StringList
//-----------------------------------------------------------------------------
Procedure TDLFont.TextListCentered( strin : TStringList; x, y : word; scale : byte; color : Cardinal );
var
  u, tekX, TekY : word;
  lineLenght : word;
Begin
  tekY:= 0;
  repeat
    lineLenght:=0;
    for u:= 1 to length( strin[TekY] ) do
    inc(lineLenght, round((Shifts[ord(strin[TekY][u])] / 100) * scale) );
    lineLenght:=lineLenght div 2;

    tekX:= x - lineLenght;
    for u:= 1 to length( strin[TekY] ) do
    Try
      FPowerGraph.RenderEffectCol( FontImage, tekX, y + TekY*15, round((256/100)*scale), color, ord(strin[TekY][u]), effectSrcAlpha or effectDiffuseAlpha );
      inc( tekX, round((Shifts[ord(strin[TekY][u])] / 100) * scale) );
    Except
      //
    End;
    inc(TekY,1);
  until TekY>Strin.Count-1;
End;

//-----------------------------------------------------------------------------
//Procedure that count width of text
//-----------------------------------------------------------------------------
Function TDLFont.TextWidth( strin : string ) : integer;
var
  u, tekX : word;
Begin
  if Length(strin)<1 then exit;
  tekX:= 0;
  for u:= 1 to length( strin )-1 do
  inc( tekX, Shifts[ord(strin[u])] );
  Result:= tekX;
End;

//-----------------------------------------------------------------------------
//Procedure that Draw string at any position of the screen.
//-----------------------------------------------------------------------------
Procedure TDLFont.Text( strin : string; x, y : word; color : Cardinal ) overload;
var
  u, tekX : word;
Begin
  if Length(strin)<1 then exit;
  tekX:= x;
  for u:= 1 to length( strin ) do
  Try
    FPowerGraph.RenderEffectCol( FontImage, tekX, y, color, ord(strin[u]), effectSrcAlpha or effectDiffuseAlpha );
    inc( tekX, Shifts[ord(strin[u])] );
  Except
    //
  End;
End;

//-----------------------------------------------------------------------------
// Same as previos
//-----------------------------------------------------------------------------
Procedure TDLFont.Text( strin : integer; x, y : word; color : Cardinal ) overload;
var
  u, tekX : word;
  str : string;
Begin
  str:=inttostr(strin);
  tekX:= x;
  for u:= 1 to length( str ) do
  Try
    FPowerGraph.RenderEffectCol( FontImage, tekX, y, color, ord(str[u]), effectSrcAlpha or effectDiffuseAlpha );
    inc( tekX, Shifts[ord(str[u])] );
  Except
    //
  End;
End;

//-----------------------------------------------------------------------------
// Same as previos
//-----------------------------------------------------------------------------
Procedure TDLFont.Text( strin : double; x, y : word; color : Cardinal ) overload;
var
  u, tekX : word;
  str : string;
Begin
  str:=FloatToStr(strin);
  tekX:= x;
  for u:= 1 to length( str ) do
  Try
    FPowerGraph.RenderEffectCol( FontImage, tekX, y, color, ord(str[u]), effectSrcAlpha or effectDiffuseAlpha);
    inc( tekX, Shifts[ord(str[u])] );
  Except
    //
  End;
End;

//-----------------------------------------------------------------------------
//Procedure to draw any string in [x,y] of screen from back to front
//-----------------------------------------------------------------------------
Procedure TDLFont.TextBack( strin : string; x, y : word; color : Cardinal );
var
  u, tekX : word;
Begin
  TekX:= x;
  for u:= length ( strin ) downto 1 do
  dec( tekX, Shifts[ord( strin[u] )] );
  Text(strin, tekx, y, color);
End;

//-----------------------------------------------------------------------------
// Same as previous
//-----------------------------------------------------------------------------
Procedure TDLFont.TextBack( strin : integer; x, y : word; color : Cardinal );
var
  u, tekX : word;
  str : string;
Begin
  str:=inttostr(strin);
  TekX:= x;
  for u:= length ( str ) downto 1 do
  dec( tekX, Shifts[ord( str[u] )] );
  Text(str, tekx, y, color);
End;

//-----------------------------------------------------------------------------
// Same as previous
//-----------------------------------------------------------------------------
Procedure TDLFont.TextBack( strin : double; x, y : word; color : Cardinal );
var
  u, tekX : word;
  str : string;
Begin
  str:=FloatToStr(strin);
  TekX:= x;
  for u:= length ( str ) downto 1 do
  dec( tekX, Shifts[ord( str[u] )] );
  Text(str, tekx, y, color);
End;

end.

