unit u_game;

interface

uses OMEGA, omegaGUI, Windows, u_const, SysUtils, u_map, PathFind, Dialogs;

const
  sX=1024;
  sY=768;

procedure GameLoad;
procedure GameInit;
procedure GameDraw;
procedure GameQuit;
procedure GameProcess;
procedure GameSlovProcess;
procedure SelectUnit(n:Integer);

type
  TMyPoint = record
    X, Y: single;
  end;

var
  // GUI
  txtCursor: PTexture;
  edMyName:TMyEdit;
  // GAME TEXTURES/SOUNDS
  txMainMenuBG, txLogo, txNoise, txGround, txRend, txShadow,
  txMap, txNumbers, txNumbersTable, txMiniRend, txPlatform,
  txSignals, txCCPlatform, txMainPlatform, txRadioTree,
  txSelectedTile, txGoTile, txSignal, txTrack, txQPlatform,
  txBildImages, txBuild, txGrass, txWeapons, txEnemy, txBullet,
  txExplosion: PTexture;
  sndNoise, sndAddDevice, sndDeleteDevice, sndIIArming,
  sndIIHiding, vBildingDone, vBilding, sndExplosion,
  vBasePlaced, sndQSet, sndBeep, sndWeapon: PSound;
  // GAME VARS
  GameState:Integer = gs_mainmenu;                                           
  GameDifficult:Integer = gd_easy;
  SelectedUnit:Integer = -1;
  GameLevel:Integer = 1;
  GameEnemies:Integer = 0;
  TotalKills,Kills,PlayStart:Integer;
  GameEnemiesKilled:Integer;
  IgnoreId,IIdo,CursorId:Integer;
  IgnoreEnemy:boolean;
  GamePlan:Integer = gp_none;
  GamePrePlan:Integer;
  GameTime:Integer = 300;
  GameDemo:Boolean=False;
  DemoId,AntenneId,RobotId:Integer;
  DebugStr:string;
  //AntenneId:Integer = 0;
  EyeX,EyeY,LastEyeX,LastEyeY:Integer;
  EyeXVol,EyeYVol:Single;
  NeedRender:boolean;
  Ecreation:Integer;
  CursorState:Integer=0;
  PlaceId:Integer;
  // MAIN MENU VARS
  MyName:string;
  Players:array of TLoadPlayer;
  Iam:string = ',';
  NoiseStart,NoiseRand,NoiseLen,NoiseId,NoiseStep:Integer;
  // SYSTEM
  Home:string;
  

implementation

procedure AddRadioItem(Signal,Level,Id,Parent:integer;add:Boolean=true);
var n,m:Integer;
    b:Boolean;
begin
  if Units[Id].UnitClass=uc_enemy then Exit;
  if add then begin
    Inc(RadioTreeCount);
    RadioTree[RadioTreeCount].Parent:=Parent;
    RadioTree[RadioTreeCount].Level:=Level;
    RadioTree[RadioTreeCount].UnitId:=Id;
  end;
  Units[Id].Signal:=Signal;
  Units[Id].Detected:=true;
  for n:=1 to UnitsCount do begin
    b:=True;
    for m:=1 to RadioTreeCount do begin
      if RadioTree[m].UnitId=n then b:=False;
    end;
    if b and (n<>Parent) and
       (M_Distance(Units[n].X,Units[n].Y,Units[Id].X,Units[id].Y)<ClassSets[Units[Id].UnitClass].Shadow) and
       (Units[n].Enabled) and (Units[n].UnitClass<>uc_enemy) then begin
         if Units[n].Connecting<connection_time then begin
           Inc(Units[n].Connecting);
         end else begin
           Inc(RadioTreeCount);
           Inc(Units[Id].DevicesCount);
           Units[Id].Devices[Units[Id].DevicesCount]:=n;
           RadioTree[RadioTreeCount].Parent:=Id;
           RadioTree[RadioTreeCount].Level:=Level+1;
           RadioTree[RadioTreeCount].UnitId:=n;
           Units[n].Signal:=Round(4*(1-(M_Distance(Units[n].X,Units[n].Y,Units[Id].X,Units[Id].Y)/ClassSets[Units[Id].UnitClass].Shadow)));
           Units[n].Detected:=true;
         end;
       end;
  end;
  for n:=1 to RadioTreeCount do begin
    if RadioTree[n].Parent=Id then
      AddRadioItem(Round(4*(1-(M_Distance(Units[RadioTree[n].UnitId].X,Units[RadioTree[n].UnitId].Y,Units[Id].X,Units[Id].Y)/ClassSets[Units[Id].UnitClass].Shadow))),
        Level+1,RadioTree[n].UnitId,Id,false);
  end;
end;

procedure DrawNumbers(X,Y,i,count:Integer);
var s:string;
    n:Integer;
begin
  s:=U_IntToStr(i);
  if Length(s)<count then begin
    for n:=1 to count-Length(s) do
      s:='0'+s;
  end;
  ASprite_Draw(txNumbersTable,X-16,Y-10,16,46,1);
  ASprite_Draw(txNumbersTable,X+Length(s)*16,Y-10,16,46,3);
  for n:=1 to Length(s) do begin
    ASprite_Draw(txNumbersTable,X+(n-1)*16,Y-10,16,46,2);
    ASprite_Draw(txNumbers,
      x+(n-1)*16,y,16,26,StrToIntDef(s[n],0)+1);
  end;
end;

procedure DrawShadow(X,Y:Single;Radius:Integer; coefx,coefy:single);
begin
  SSprite_Draw(txShadow,
    (X-Radius)*coefx, (Y-Radius)*coefy,
    Round(Radius*2*coefx),round(Radius*2*coefy));
end;

function GetShadow(n:Integer):integer;
begin
  result:=0;
  case Units[n].UnitClass of
    uc_programmator,uc_other:if Units[n].Detected and Units[n].Enabled then result:=ClassSets[Units[n].UnitClass].Shadow
      else result:=0;
    uc_antenne:begin
      case Units[n].State of
        as_opened:result:=ClassSets[Units[n].UnitClass].Shadow;
        as_plasing:result:=Round((ClassSets[Units[n].UnitClass].Shadow-2)*(Units[n].Tag/18))+2;
        as_hiding:result:=Round((ClassSets[Units[n].UnitClass].Shadow-2)*(1-Units[n].Tag/18))+2;
        else result:=2;
      end;
    end;
  end;
end;

procedure RenderShadow;
var n:Integer;
begin
  Tex_BeginRenderIn(txRend,False,True,false);
    Pr2D_Rect(0,0,ten_x,ten_y,ten_color,255,PR_FILL);
    for n:=1 to UnitsCount do begin
      if Units[n].Enabled and (Units[n].Health>0) then
      DrawShadow(Units[n].X*ground_size+EyeX,Units[n].Y*ground_size+EyeY,
        GetShadow(n)*ground_size,
        ten_dx,ten_dy);
    end;
  Tex_EndRenderIn;
  Tex_BeginRenderIn(txMiniRend,False,True,false);
    Pr2D_Rect(0,0,236,236,ten_color,255,PR_FILL);
    for n:=1 to UnitsCount do begin
      if Units[n].Enabled and (Units[n].Health>0) then
      DrawShadow(Units[n].X*ground_size,Units[n].Y*ground_size,
        GetShadow(n)*ground_size,
        map_px/map_coef,map_py/map_coef);
    end;
  Tex_EndRenderIn;
end;

procedure GameLoad;
begin
  Pr2D_Rect((sX div 2) - 64,sY div 2,128,4,$333333,PR_FILL);
end;

procedure GameQuit;
begin
  OE_Quit;
end;

procedure Addtolog(s:string);
var f:Text;
begin
  AssignFile(f,'log.txt');
  append(f);
  Writeln(f,s);
  CloseFile(f);
end;

procedure WriteArr;
var n:Integer;
    s:string;
begin
  s:='';
  for n:=1 to GrassIdsCount do
    s:=s+'['+inttostr(GrassIds[n])+'] ';
  Addtolog(s);
end;

procedure AddTrack(X,Y,Angle:Integer;unitId:integer);
begin
  if Units[unitId].UnitClass=uc_enemy then exit;
  Inc(TracksCount);
  Tracks[TracksCount].X:=X;
  Tracks[TracksCount].Y:=Y;
  Tracks[TracksCount].Angle:=Angle;
  Tracks[TracksCount].Time:=track_time;
  
end;

procedure AddExplosion(X,Y:integer);
begin
  Inc(ExplosionsCount);
  Explosions[ExplosionsCount].X:=X;
  Explosions[ExplosionsCount].Y:=Y;
  Explosions[ExplosionsCount].Time:=explo_time;
  Snd_Play(sndExplosion,0,VOLUME(guiVolume));
end;

procedure AddBullet(X,Y:Single;KillId,Angle,Time:Integer);
begin
  Inc(BulletsCount);
  Bullets[BulletsCount].X:=X;
  Bullets[BulletsCount].Y:=Y;
  Bullets[BulletsCount].Angle:=Angle;
  Bullets[BulletsCount].KillId:=KillId;
  Bullets[BulletsCount].Time:=Round(Time/bullet_speed);
end;

procedure SearchPlayers;
var id:Integer;
    SR:TSearchRec;
begin
  Players:=nil;
  if FindFirst(Home+'Data/Players/*',faDirectory,SR)=0 then begin
    repeat
      if (SR.Name<>'.') and (SR.Name<>'..') then begin
        id:=Length(Players);
        SetLength(Players,id+1);
        Players[id].Name:=SR.Name;
        Players[id].Progress:=StrToIntDef(IniGet('Data/Players/'+SR.Name+'/Player.ini','Player','progress','0'),0);
        Players[id].Difficult:=StrToIntDef(IniGet('Data/Players/'+SR.Name+'/Player.ini','Player','difficult','0'),0);
        Players[id].Kills:=StrToIntDef(IniGet('Data/Players/'+SR.Name+'/Player.ini','Player','kills','0'),0);
      end;
    until FindNext(SR)<>0;
  end;
end;

procedure CreateEnemy;
var i:Integer;
begin
  Inc(UnitsCount);
  i:=Random(EnemyPointsCount)+1;
  Units[UnitsCount].X:=EnemyPoints[i].X;
  Units[UnitsCount].Y:=EnemyPoints[i].Y;
  Units[UnitsCount].UnitClass:=uc_enemy;
  Units[UnitsCount].Health:=ClassSets[uc_enemy].Health;
  Units[UnitsCount].Detected:=False;
  Units[UnitsCount].State:=us_none;
  Units[UnitsCount].Enabled:=false;
  Units[UnitsCount].Destroyed:=False;
  Units[UnitsCount].Piercing:=1;
  Units[UnitsCount].Tag:=0;
  Units[UnitsCount].Tag2:=0;
  Units[UnitsCount].PTag:=0;
  Units[UnitsCount].Connecting:=0;
end;

procedure GameOver;
begin
  GameState:=gs_fail;
  NoiseId:=1;
  TracksCount:=0;
  Media_Play('Data/Sound/Fail.ogg',False);
  Media_SetVolume(VOLUME(guiVolume));
  IniSet('Data/Players/'+MyName+'/Player.ini','Player','kills',IntToStr(Kills));
  LastRadioTreeCount:=0;
  BulletsCount:=0;
  ExplosionsCount:=0;
end;

procedure FinishGame;
begin
  GameState:=gs_ok;
  NoiseId:=1;
  TracksCount:=0;
  Media_Play('Data/Sound/Ok.ogg',False);
  Media_SetVolume(VOLUME(guiVolume));
  IniSet('Data/Players/'+MyName+'/Player.ini','Player','progress',IntToStr(Round(100*(GameLevel/LEVELS_COUNT))));
  Inc(GameLevel);
  LastRadioTreeCount:=0;
  BulletsCount:=0;
  ExplosionsCount:=0;
  IniSet('Data/Players/'+MyName+'/Player.ini','Player','level',IntToStr(GameLevel));
  TotalKills:=TotalKills+Kills;
  IniSet('Data/Players/'+MyName+'/Player.ini','Player','kills',IntToStr(TotalKills));
end;

procedure GameInit;
var n:Integer;
begin
  //guiVolume:=0;
  guiInit('Data/GUI');
  txGround:= Tex_LoadFromFile('Data/Textures/Ground.tga',0,0);
  txLevelBG:=Tex_LoadFromFile('Data/Textures/BG.jpg',0,0);
  Tex_SetFramesSize(txGround,ground_size,ground_size);
  txtCursor:= Tex_LoadFromFile('Data/Textures/Cursor.tga',0,0);
  Tex_SetFramesSize(txtCursor, 64, 64);
  txLogo:= Tex_LoadFromFile('Data/Textures/Logo.jpg',0,0);
  txEnemy:= Tex_LoadFromFile('Data/Textures/Enemy.tga',-1,TEX_FILTER_BILINEAR);
  txBullet:= Tex_LoadFromFile('Data/Textures/Bullet.tga',-1,TEX_FILTER_BILINEAR);
  txExplosion:= Tex_LoadFromFile('Data/Textures/Explosion.tga',0,0);
  Tex_SetFramesSize(txExplosion, 48, 48);
  txNoise:= Tex_LoadFromFile('Data/Textures/Noise.jpg',0,0);
  txRadioTree:= Tex_LoadFromFile('Data/Textures/RadioTree.tga',0,0);
  txSelectedTile:= Tex_LoadFromFile('Data/Textures/SelectedTile.tga',0,0);
  txGoTile:= Tex_LoadFromFile('Data/Textures/GoTile.tga',0,0);
  txSignal:= Tex_LoadFromFile('Data/Textures/Signal.tga',0,0);
  txTrack:= Tex_LoadFromFile('Data/Textures/Track.tga',0,0);
  txBildImages:= Tex_LoadFromFile('Data/Textures/BildImages.tga',0,0);
  txQPlatform:= Tex_LoadFromFile('Data/Textures/QPlatform.tga',0,0);
  txBuild:= Tex_LoadFromFile('Data/Textures/Build.tga',0,0);
  txGrass:= Tex_LoadFromFile('Data/Textures/Grass.tga',0,0);
  txWeapons:= Tex_LoadFromFile('Data/Textures/Weapons.tga',0,TEX_DEFAULT_2D or TEX_FILTER_BILINEAR);
  Tex_SetFramesSize(txQPlatform, 64, 64);
  Tex_SetFramesSize(txBildImages, 64, 64);
  Tex_SetFramesSize(txRadioTree, 16, 16);
  Tex_SetFramesSize(txGrass, 64, 64);
  Tex_SetFramesSize(txWeapons, 32, 32);

  Tex_SetFramesSize(txSignal, 16, 16);
  txShadow:= Tex_LoadFromFile('Data/Textures/Shadow.tga',0,TEX_FILTER_BILINEAR);
  txNumbers:= Tex_LoadFromFile('Data/Textures/Numbers.jpg',-1,0);
  txPlatform:= Tex_LoadFromFile('Data/Textures/Platform.tga',-1,TEX_FILTER_BILINEAR);
  txCCPlatform:= Tex_LoadFromFile('Data/Textures/CCPlatform.tga',-1,TEX_FILTER_BILINEAR);
  txMainPlatform:= Tex_LoadFromFile('Data/Textures/MainPlatform.tga',-1,TEX_FILTER_BILINEAR);
  Tex_SetFramesSize(txMainPlatform, 64, 64);
  txSignals:= Tex_LoadFromFile('Data/Textures/Signals.tga',-1,0);
  Tex_SetFramesSize(txSignals, 7, 7);
  txNumbersTable:= Tex_LoadFromFile('Data/Textures/NumbersTable.tga',-1,0);
  Tex_SetFramesSize(txNumbersTable, 16, 46);
  Tex_SetFramesSize(txNumbers, 16, 26);
  sndNoise:= Snd_LoadFromFile('Data/Sound/Noise.wav',2);
  sndAddDevice:= Snd_LoadFromFile('Data/Sound/AddDevice.wav',2);
  sndDeleteDevice:= Snd_LoadFromFile('Data/Sound/DeleteDevice.wav',2);
  sndIIArming:= Snd_LoadFromFile('Data/Sound/IIArming.wav',2);
  sndIIHiding:= Snd_LoadFromFile('Data/Sound/IIHiding.wav',2);
  sndExplosion:= Snd_LoadFromFile('Data/Sound/Explosion.wav',2);
  Tex_SetFramesSize(txNoise, 128, 128);
  txMainMenuBG:= Tex_LoadFromFile('Data/Textures/MainMenuBG.jpg',0,0);
  Tex_SetFramesSize(txMainMenuBG, 128, 128);

  vBildingDone:= Snd_LoadFromFile('Data/Voice/BildingDone.wav',2);
  vBilding:= Snd_LoadFromFile('Data/Voice/Bilding.wav',2);
  vBasePlaced:= Snd_LoadFromFile('Data/Voice/BasePlaced.wav',2);
  sndQSet:= Snd_LoadFromFile('Data/Sound/QSet.wav',2);
  sndBeep:= Snd_LoadFromFile('Data/Sound/Beep.wav',2);
  sndWeapon:= Snd_LoadFromFile('Data/Sound/Weapon.wav',2);

  for n:=0 to uc_count-1 do begin
    if ClassSets[n].NeedSnd then
    ClassSets[n].sndSelect:= Snd_LoadFromFile(PAnsiChar('Data/Sound/SelectDevice'+inttostr(n)+'.wav'),2);
  end;

  ten_x:=ten_size;
  ten_y:=round(ten_size*(sY/sX));
  ten_dx:=ten_x/sX;
  ten_dy:=ten_y/sY;
  txRend:=Tex_CreateZero(ten_x,ten_y,TEX_DEFAULT_2D );
  txMiniRend:=Tex_CreateZero((forms_width-20) div map_coef,(forms_width-20) div map_coef,TEX_FILTER_BILINEAR );

  home:=extractfilepath(ParamStr(0));
  Sleep(1000);
  NoiseId:=1;
  guiInitEdit(@edMyName,sX div 2 - 76,sY div 2 - 24,144);

  Media_Play('Data/Sound/MainMenu.mp3',True);
  Media_SetVolume(VOLUME(guiVolume));
end;

procedure LoadGame(s:string);
begin
  MyName:=s;
  GameDifficult:=StrToIntDef(IniGet('Data/Players/'+MyName+'/Player.ini','Player','difficult','0'),0);
  GameLevel:=StrToIntDef(IniGet('Data/Players/'+MyName+'/Player.ini','Player','level','1'),1);
  TotalKills:=StrToIntDef(IniGet('Data/Players/'+MyName+'/Player.ini','Player','kills','0'),0);
  Kills:=0;
  EyeX:=StrToIntDef(IniGet('Data/Players/'+MyName+'/Player.ini','Player','EyeX','0'),0);
  EyeY:=StrToIntDef(IniGet('Data/Players/'+MyName+'/Player.ini','Player','EyeY','0'),0);
  LoadMap(GameLevel);
  Media_Stop;
  RenderShadow;
  PlayStart:=1;
  GameState:=gs_game;
end;

procedure SelectUnit(n:Integer);
begin
  SelectedUnit:=n;
  CursorState:=0;
  Snd_Play(ClassSets[Units[n].UnitClass].sndSelect,0,VOLUME(guiVolume));
  if (Units[n].X*ground_size+EyeX<0) or (Units[n].Y*ground_size+EyeY<0) or
    (Units[n].X*ground_size+EyeX>sX) or (Units[n].Y*ground_size+EyeY>sY) then begin
      EyeX:=-round(Units[n].X*ground_size)+sX div 2;
      EyeY:=-round(Units[n].Y*ground_size)+sY div 2;
    end;
end;

function DrawTree(Id:Integer; j:Integer;Level:Integer):Integer;
var n,m:Integer;
begin
  m:=Units[Id].Signal;
  if MouseInBorder(8,j,190,15) then begin
    Pr2D_Rect(8,j,206,16,$FFFFFF,64,PR_FILL);
    guiDrawMemoBorder(206,48,176,48, 200);
    ASprite_Draw(txRadioTree, 212, 56,
      16, 16, Units[Id].UnitClass+1);
    if m>0 then begin
      ASprite_Draw(txSignal,
        228, 56,
        16,16,m);
    end;
    guiDrawText(ClassSets[Units[Id].UnitClass].Name,
      246,56);
    guiDrawText(' :', 212, 80);
    Pr2D_RECT(216, 98, 172, 4, $777777, 255, PR_FILL);
    Pr2D_RECT(216, 98, (Units[Id].Health/ClassSets[Units[Id].UnitClass].Health)*172, 4, $FFFFFF, 255, PR_FILL);
    guiDrawText(': '+inttostr(Units[Id].Piercing)+'.0', 212, 106);
    if Mouse_Click(1) then SelectUnit(Id);
  end;
  ASprite_Draw(txRadioTree,16+Level*16,j,
    16,16, Units[Id].UnitClass+1);
  if m>0 then begin
    ASprite_Draw(txSignal,
      32+Level*16,j,
      16,16,m);
  end;
  guiDrawText(ClassSets[Units[Id].UnitClass].Name,48+Level*16,j);
  j:=j+16;
  for n:=1 to units[Id].DevicesCount do begin
    j:=DrawTree(units[Id].Devices[n],j,Level+1);
  end;
  result:=j;
end;

function GetR(x,y:Integer;fromx,fromy:single):TPoint;
var n,m:Integer;
    j,k:Single;
begin
  j:=MapX*2;
  result.X:=X;
  result.Y:=Y;
  for m:=-1 to 1 do
    for n:=-1 to 1 do
      if (n<>0) and (m<>0) and (MapCoast(x+n,y+m,0)<>-1) then begin
        k:=M_Distance(x+n,y+m,fromx,fromy);
        if k<j then begin
          j:=k;
          result.X:=x+n;
          result.Y:=y+m;
        end;
      end;
end;

function UnitAtXY(x,y:Integer):integer;
var n:Integer;
begin
  result:=-1;
  for n:=1 to UnitsCount do begin

    if (Units[n].Health>0) and (Round(Units[n].X)=X) and (Round(Units[n].Y)=Y) then begin
      Result:=n;
      Exit;
    end;
  end;
end;

function DrawForm(Text,Button:string):Boolean;
begin
  result:=False;
  guiDrawForm(' ',0,sY div 2,520,sY div 4);
  guiDrawText(Text,10,sY div 2+34);
  if Button<>'' then
    result:=guiDrawButton(Button,520-170, sY-sY div 4 - 40, 150)=button_click;
end;

procedure GameDraw;
var n,m,j,a,i: Integer;
    eX,eY:single;
    aX,aY:Integer;
    f:TPoint;
    TempPlayer:TLoadPlayer;
    s:string;
begin
  if GameState<>gs_game then begin
    ASprite_Draw(txMainMenuBG, sX div 2-128, 128, 256, 256, NoiseId);
    SSprite_Draw(txLogo, sX div 2-256, 64, 512, 64);
    if NoiseLen>0 then begin
      if NoiseLen>noise_time*0.9 then n:=Round((1-(NoiseLen-noise_time*0.9)/(noise_time*0.1))*255) else
      if NoiseLen<noise_time*0.1 then n:=Round((NoiseLen/(noise_time*0.1))*255) else
        n:=255;
      ASprite_Draw(txNoise,sX div 2-128, 128, 256, 256,NoiseRand, n);
    end;
  end;
  if not (GameState in [gs_game,gs_ok,gs_fail,gs_esc]) then
    DrawNumbers(sX div 2-128+4, 128+4, NoiseStart, 3);
  guiFocus;

  case GameState of
    gs_esc:begin
      guiDrawForm('?',sX div 2 - 98,sY div 2 - 80,196,162);
      eX:=sX div 2 - 98; eY:=sY div 2 - 86;
      guiDrawText(' : '+#13+inttostr(Kills)+
        #13#13+':'#13+inttostr(TotalKills),
        eX+8, eY+90);
      if guiDrawButton('',eX+8,eY+32,180)=button_click then begin
        GameState:=gs_game;
      end;
      if guiDrawButton('',eX+8,eY+58,180)=button_click then begin
        GameState:=gs_mainmenu;
        Media_Play('Data/Sound/MainMenu.mp3',True);
        Media_SetVolume(VOLUME(guiVolume));
      end;
    end;
    gs_fail:begin
      guiDrawForm(' !',sX div 2 - 98,sY div 2 - 80,196,162);
      eX:=sX div 2 - 98; eY:=sY div 2 - 86;
      guiDrawText(' : '+#13+inttostr(Kills)+
        #13#13+':'#13+inttostr(TotalKills),
        eX+8, eY+90);
      if guiDrawButton('!',eX+8,eY+32,180)=button_click then begin
        LoadGame(MyName);
      end;
      if guiDrawButton('',eX+8,eY+58,180)=button_click then begin
        GameState:=gs_mainmenu;
        Media_Play('Data/Sound/MainMenu.mp3',True);
        Media_SetVolume(VOLUME(guiVolume));
      end;
    end;
    gs_ok:begin
      guiDrawForm(' !',sX div 2 - 98,sY div 2 - 80,196,162);
      eX:=sX div 2 - 98; eY:=sY div 2 - 86;
      guiDrawText(' : '+#13+inttostr(Kills)+
        #13#13+':'#13+inttostr(TotalKills),
        eX+8, eY+90);

      if guiDrawButton('',eX+8,eY+32,180)=button_click then begin
        if GameLevel=LEVELS_COUNT then begin
          Media_Play('Data/Sound/MainMenu.mp3',True);
          Media_SetVolume(VOLUME(guiVolume));
          GameState:=gs_author;
        end else begin
          LoadGame(MyName);
        end;
      end;
      if guiDrawButton('',eX+8,eY+58,180)=button_click then begin
        GameState:=gs_mainmenu;
        Media_Play('Data/Sound/MainMenu.mp3',True);
        Media_SetVolume(VOLUME(guiVolume));
      end;
    end;
    gs_game:begin

      DrawMap;
      for n:=1 to TracksCount do begin
      if Tracks[n].Time>track_time div 2 then j:=255 else
          j:=Round(255*(Tracks[n].Time/(track_time div 2)));
        SSprite_Draw(txTrack,
              Tracks[n].X+EyeX-4,
              Tracks[n].Y+EyeY-32,
              8,64,j,Tracks[n].Angle);
      end;
      for n:=1 to BulletsCount do begin
        SSprite_Draw(txBullet,
              Bullets[n].X+EyeX-16,
              Bullets[n].Y+EyeY-16,
              32,32,255,Bullets[n].Angle);
      end;
      for n:=1 to ExplosionsCount do begin
        ASprite_Draw(txExplosion,
              Explosions[n].X+EyeX-64,
              Explosions[n].Y+EyeY-64,
              128,128,Round(18*(1-Explosions[n].Time/explo_time)));
      end;
      for n:=1 to UnitsCount do begin
        if Units[n].Health<=0 then Continue;
        if Units[n].Detected and (Units[n].Health>0) then begin
          j:=Units[n].Signal;
          if j>0 then begin
            ASprite_Draw(txSignal,
              Units[n].X*ground_size+EyeX-8,
              Units[n].Y*ground_size+EyeY-40,
              16,16,j);
          end;
          //guiDrawText(IntToStr(Units[n].DevicesCount),Units[n].X*ground_size+EyeX-8,Units[n].Y*ground_size+EyeY-40);
        end;
        if MouseInBorder(Units[n].X*ground_size+EyeX-32,
          Units[n].Y*ground_size+EyeY-32,64,64) then begin
          if Mouse_Click(1) then begin
            SelectUnit(n);
          end;
        end;
        case Units[n].UnitClass of
          uc_build:begin
            SSprite_Draw(txBuild,
              Units[n].X*ground_size+EyeX-32,
              Units[n].Y*ground_size+EyeY-32,
              64,64,255);
            Pr2D_Rect(Units[n].X*ground_size+EyeX-10,
              Units[n].Y*ground_size+EyeY-10,
              Round((Units[n].BuildTime/ao_bases[Units[n].BildId].Bt)*20),1,$00FF00,255,PR_FILL);
          end;
          uc_programmator:begin
            SSprite_Draw(txCCPlatform,
              Units[n].X*ground_size+EyeX-32,
              Units[n].Y*ground_size+EyeY-32,
              64,64,255,Units[n].Angle);
          end;
          uc_enemy:begin
            if Units[n].Destroyed=False then
            SSprite_Draw(txEnemy,
              Units[n].X*ground_size+EyeX-32,
              Units[n].Y*ground_size+EyeY-32,
              64,64,255,Units[n].Angle);
          end;
          uc_grass:begin
            if Units[n].Health>0 then
            ASprite_Draw(txGrass,
              Units[n].X*ground_size+EyeX-32,
              Units[n].Y*ground_size+EyeY-32,
              64,64,Units[n].Tag+1);
          end;
          uc_qualificator:begin
            j:=1;
            case Units[n].State of
              as_closed:j:=1;
              as_opened:j:=1;
              as_plasing:j:=1+Units[n].Tag;
              as_hiding:j:=15-Units[n].Tag;
            end;
            ASprite_Draw(txQPlatform,
              Units[n].X*ground_size+EyeX-32,
              Units[n].Y*ground_size+EyeY-32,
              64,64,j);
            if Units[n].State=as_opened then begin
              ASprite_Draw(txQPlatform,
                Units[n].X*ground_size+EyeX-32,
                Units[n].Y*ground_size+EyeY-32,
                64,64,16,255,Units[n].Tag2);
            end;

          end;
          uc_antenne:begin
            j:=1;
            case Units[n].State of
              as_closed:j:=1;
              as_opened:j:=1;
              as_plasing:j:=1+Units[n].Tag;
              as_hiding:j:=19-Units[n].Tag;
            end;
            ASprite_Draw(txMainPlatform,
              Units[n].X*ground_size+EyeX-32,
              Units[n].Y*ground_size+EyeY-32,
              64,64,j);

            if Units[n].State=as_opened then begin
              ASprite_Draw(txMainPlatform,
                Units[n].X*ground_size+EyeX-32,
                Units[n].Y*ground_size+EyeY-32,
                64,64,20,255,Units[n].Tag2);
            end;

          end;
          else begin
            SSprite_Draw(txPlatform,
              Units[n].X*ground_size+EyeX-32,
              Units[n].Y*ground_size+EyeY-32,
              64,64,255,Units[n].Angle);
            if Units[n].BeepCounter<beep_time div 4 then begin
              if Units[n].Enabled then j:=2 else j:=1;
              ASprite_Draw(txSignals,
                Round(Units[n].X*ground_size-M_Cos(Units[n].Angle)*11+EyeX-4),
                Round(Units[n].Y*ground_size-M_Sin(Units[n].Angle)*11+EyeY-4),
              7,7,j);
            end;
            case Units[n].Qualification of
              rs_firer:begin
                ASprite_Draw(txWeapons,
                  Units[n].X*ground_size+EyeX-16,
                  Units[n].Y*ground_size+EyeY-16,32,32,
                  Round(1+2*(Units[n].Tag2/firer_time)),255,Units[n].TurrelAngle);
              end;
            end;
          end;
        end;
      end;
      FX_SetBlendMode(FX_BLEND_MASK);
        SSprite_Draw(txRend,0,0,sX,sY);
      FX_SetBlendMode(FX_BLEND_NORMAL );
      if GameDemo then begin
        case DemoId of
          0:begin
            DrawForm(' !     !'+#13+
                     ' ,   .    '+#13+
                     '""   .','');
            if Units[AntenneId].State=as_opened then Inc(DemoId);
          end;
          1:begin
            DrawForm('!  ,    .   '+#13+
                     '      .  ,'+#13+
                     '     .','');
            if Units[SelectedUnit].UnitClass=uc_programmator then Inc(DemoId);
          end;
          2:begin
            Pr2D_Rect(ground_size*9+EyeX-ground_size div 2,ground_size*9+EyeY-ground_size div 2,ground_size,ground_size,$00FF00,64,PR_FILL);
            DrawForm('!   -  ,  ,  ,'+#13+
                     '   ,     '+#13+
                     '  .        '+#13+
                     ' ','');
            IgnoreId:=-1;
            IgnoreEnemy:=False;
            if MapCoast(9,9,0)=-1 then Inc(DemoId);
          end;
          3:begin
            Pr2D_Rect(Units[RobotId].X*ground_size+EyeX-ground_size div 2,Units[RobotId].Y*ground_size+EyeY-ground_size div 2,ground_size,ground_size,$00FF00,64,PR_FILL);
            DrawForm('!         '+#13+
                     '.  ,      .'+#13+
                     '         ','');
            if Units[RobotId].Enabled then Inc(DemoId);
          end;
          4:begin
            DrawForm('   !  ,   '+#13+
                     '  .    '+#13+
                     '  ,   .  ,   '+#13+
                     '  .      .'+#13+
                     '           '+#13+
                     ' " "   "".','');
            if (CursorState=cs_placebase) and (PlaceId=1) then Inc(DemoId);
          end;
          5:begin
            DrawForm('       ,'+#13+
                     '    .','');
            if (Units[UnitsCount].UnitClass=uc_build) then Inc(DemoId);
          end;
          6:begin
            DrawForm('...','');
            if (Units[UnitsCount].UnitClass=uc_qualificator) then Inc(DemoId);
          end;
          7:begin
              DrawForm(',    !   '+#13+
                     '.   ,   .','');
            if Units[UnitsCount].State=qs_opened then Inc(DemoId);
          end;
          8:begin
            Pr2D_Rect(Units[RobotId].X*ground_size+EyeX-ground_size div 2,Units[RobotId].Y*ground_size+EyeY-ground_size div 2,ground_size,ground_size,$00FF00,64,PR_FILL);
            DrawForm('    .'+#13+
                     '  ,   ""' + #13+
                     '    (. . )  ','');
            if Units[RobotId].State=us_goto then Inc(DemoId);
          end;
          9:begin
            if DrawForm(',      .'+#13+
                        '  -       '+#13+
                        '.     ,  .',
            '') then FinishGame;
            
          end;
        end;
      end;
      guiDrawForm(' ',10,sY-forms_width-30,forms_width,forms_width+20);
      SSprite_Draw(txMap, 20,sY-forms_width,forms_width-20,forms_width-20);
      for n:=1 to UnitsCount do begin
        if Units[n].Health<=0 then Continue;
        if Units[n].Detected then begin
          if SelectedUnit=n then j:=$0000FF else j:=$FFFFFF;
          Pr2D_Circle(
          (Units[n].X-1)*((forms_width-20)/MapX)+20,
          (Units[n].Y-1)*((forms_width-20)/MapY)+sY-forms_width,
          (forms_width-20)/MapX/2,j,255,8,PR_FILL or PR_SMOOTH);
        end;
      end;
      FX_SetBlendMode(FX_BLEND_MASK);
        SSprite_Draw(txMiniRend,20,sY-forms_width,forms_width-20,forms_width-20);
      FX_SetBlendMode(FX_BLEND_NORMAL );
      Pr2D_Rect(20,sY-forms_width,forms_width-20,forms_width-20,$555555);
      guiDrawMemoBorder(20-EyeX*map_px,sY-forms_width-EyeY*map_py,
        (sX-EyeX)*map_px+EyeX*map_px-32,(sY-EyeY)*map_py+EyeY*map_py-32,128);
      DrawNumbers(sX-74,10,GameTime,4);
      guiDrawText('',sX-124,10);
      guiDrawText(' ',sX-164,20);
      if (GameTime>0) and (GameDemo=false) then
        if guiDrawButton(' ', sX - 250, 10, 80)=button_click then GameTime:=0;

      DrawNumbers(sX-74,40,GameEnemiesKilled,4);
      guiDrawText('',sX-129,40);
      guiDrawText('-',sX-129,50);

      DrawNumbers(sX-58,70,GrassIdsCount,3);
      guiDrawText('',sX-134,70);
      guiDrawText('',sX-113,80);
      DrawNumbers(10,10,RadioTreeCount,3);
      guiDrawText('',64,10);
      guiDrawText('  ',64,20);
      if RadioTreeCount>0 then begin
        guiDrawMemoBorder(8,48,176,RadioTreeCount*16-16, 200);
        j:=56;
        for n:=1 to UnitsCount do
          if Units[n].UnitClass=uc_antenne then
            j:=DrawTree(n,j,0);
      end;
      if SelectedUnit<>-1 then begin
        SSprite_Draw(txSelectedTile,
          Units[SelectedUnit].X*ground_size+EyeX-64,
              Units[SelectedUnit].Y*ground_size+EyeY-64,
              128,128,255);
        guiDrawForm(ClassSets[Units[SelectedUnit].UnitClass].Name, forms_width+20, sY-forms_width-30,sX-40-forms_width*2,forms_width+20);
        if (Units[SelectedUnit].UnitClass <> uc_antenne) and
          (Units[SelectedUnit].Detected=False) then begin
          guiDrawText('      . ,' + #13+
                      '     - ' + #13+
                      '.   ,  .' + #13+
                      '   .',forms_width+30,sY-forms_width-6);
        end else begin
          case Units[SelectedUnit].UnitClass of
            uc_build:begin
              guiDrawText(' : '+ClassSets[ao_bases[Units[SelectedUnit].BildId].I].Name,
                forms_width+30,sY-forms_width-6);
              guiDrawProgressBar(forms_width+30,sY-forms_width+14,160,
                Units[SelectedUnit].BuildTime/ao_bases[Units[SelectedUnit].BildId].Bt);
            end;
            uc_programmator:begin
              guiDrawText(' ' + #13+
                          ', ' + #13+
                          '  ' + #13+
                          ' ' + #13+
                          ':',forms_width+30,sY-forms_width-6);
              case Units[SelectedUnit].State of
                us_goto:begin
                  SSprite_Draw(txGoTile,
                    Units[SelectedUnit].toX*ground_size+EyeX-64,
                    Units[SelectedUnit].toY*ground_size+EyeY-64,
                    128,128,255);
                end;
              end;
              guiDrawTabs(forms_width+184,sY-forms_width-20,
                sX-220-forms_width*2, forms_width-24,' ','||',IIdo);
              case CursorState of
                cs_none:begin
                  aX:=(Mouse_X-EyeX+ground_size div 2) div ground_size;
                  aY:=(Mouse_Y-EyeY+ground_size div 2) div ground_size;
                  j:=UnitAtXY(aX,aY);
                  if j<>-1 then begin
                    case Units[j].UnitClass of
                      uc_build:begin
                        CursorId:=2;
                      end;
                      uc_other:begin
                        if Units[j].Enabled=False then begin
                          CursorId:=3;
                          if Mouse_Click(0) then begin
                            Units[SelectedUnit].State:=us_goto;
                            Units[SelectedUnit].NextState:=us_open;
                            Units[SelectedUnit].Tag:=0;
                            Units[SelectedUnit].Tag2:=0;
                            f:=GetR(aX,aY,Units[SelectedUnit].X,Units[SelectedUnit].Y);
                            Units[SelectedUnit].toX:=f.X;
                            Units[SelectedUnit].toY:=f.Y;
                            Units[SelectedUnit].Speed:=0;
                            IgnoreId:=SelectedUnit;
                            Units[SelectedUnit].BildId:=j;
                            IgnoreEnemy:=False;
                            FindPath(Units[SelectedUnit].Path,
                              Round(Units[SelectedUnit].X),Round(Units[SelectedUnit].Y),
                              Units[SelectedUnit].toX,Units[SelectedUnit].toY,MapCoast);
                            if Units[SelectedUnit].Path.Size=0 then begin
                              Units[SelectedUnit].State:=us_none;
                            end;
                            Units[SelectedUnit].PathId:=1;
                          end;
                        end else begin
                          CursorId:=4;
                          if Mouse_Click(0) then begin
                            Units[SelectedUnit].State:=us_goto;
                            Units[SelectedUnit].NextState:=us_qualify;
                            Units[SelectedUnit].Tag:=0;
                            Units[SelectedUnit].Tag2:=0;
                            f:=GetR(aX,aY,Units[SelectedUnit].X,Units[SelectedUnit].Y);
                            Units[SelectedUnit].toX:=f.X;
                            Units[SelectedUnit].toY:=f.Y;
                            Units[SelectedUnit].Speed:=0;
                            IgnoreId:=SelectedUnit;
                            Units[SelectedUnit].BildId:=j;
                            IgnoreEnemy:=False;
                            FindPath(Units[SelectedUnit].Path,
                              Round(Units[SelectedUnit].X),Round(Units[SelectedUnit].Y),
                              f.X,f.Y,MapCoast);
                            if Units[SelectedUnit].Path.Size=0 then begin
                              Units[SelectedUnit].State:=us_none;
                            end;
                            Units[SelectedUnit].PathId:=1;
                          end;
                        end;

                      end;
                    end;
                    if Mouse_Click(0) then begin
                      case Units[j].UnitClass of
                        uc_build:begin
                          Units[SelectedUnit].State:=us_goto;
                          Units[SelectedUnit].NextState:=us_build;
                          Units[SelectedUnit].Tag:=0;
                          Units[SelectedUnit].Tag2:=0;
                          f:=GetR(aX,aY,Units[SelectedUnit].X,Units[SelectedUnit].Y);
                          Units[SelectedUnit].toX:=f.X;
                          Units[SelectedUnit].toY:=f.Y;
                          Units[SelectedUnit].Speed:=0;
                          IgnoreId:=SelectedUnit;
                          Units[SelectedUnit].BildId:=j;
                          IgnoreEnemy:=False;
                          FindPath(Units[SelectedUnit].Path,
                            Round(Units[SelectedUnit].X),Round(Units[SelectedUnit].Y),
                            f.X,f.Y,MapCoast);
                          if Units[SelectedUnit].Path.Size=0 then begin
                            Units[SelectedUnit].State:=us_none;
                          end;
                          Units[SelectedUnit].PathId:=1;
                        end;
                      end;
                    end;
                  end;
                  
                  case IIdo of
                    0:begin
                      for n:=1 to ao_basescount do begin
                        ASprite_Draw(txBildImages,
                          forms_width+194+(n-1)*68,sY-forms_width+10,64,64,
                          ao_bases[n].B);
                        if MouseInBorder(forms_width+194+(n-1)*68,sY-forms_width+10,64,64) then begin
                          Pr2D_Rect(forms_width+194+(n-1)*68,sY-forms_width+10,64,64,$FFFFFF);
                          Pr2D_Rect(forms_width+194+(n-1)*68,sY-forms_width+10,64,64,$00FF00,64,PR_FILL);
                          guiDrawText(ClassSets[ao_bases[n].I].Name,
                            forms_width+194+(n-1)*68,sY-forms_width);
                          if Mouse_Click(1) then begin
                            CursorState:=cs_placebase;
                            PlaceId:=n;
                            Break;
                          end;
                        end;
                      end;
                    end;
                    1:begin
                      for n:=1 to q_count do begin
                        ASprite_Draw(txBildImages,
                          forms_width+194+(n-1)*68,sY-forms_width+10,64,64,
                          q_sets[n].B);
                        if Units[SelectedUnit].Q=n-1 then begin
                          Pr2D_Rect(forms_width+194+(n-1)*68,sY-forms_width+10,64,64,$FFFFFF);
                          Pr2D_Rect(forms_width+194+(n-1)*68,sY-forms_width+10,64,64,$00FF00,32,PR_FILL);
                        end;
                        if MouseInBorder(forms_width+194+(n-1)*68,sY-forms_width+10,64,64) then begin

                          guiDrawText(q_sets[n].N,
                            forms_width+194+(n-1)*68,sY-forms_width);
                          if Mouse_Click(1) then begin
                            Units[SelectedUnit].Q:=n-1;

                          end;
                        end;

                      end;
                    end;
                  end;
                end;
                cs_placebase:begin
                  if Mouse_Y<sY-forms_width then begin
                    Pr2D_Rect(((Mouse_X-EyeX+ground_size div 2) div ground_size)*ground_size-(ground_size div 2)+EyeX,
                     ((Mouse_Y-EyeY+ground_size div 2) div ground_size)*ground_size-(ground_size div 2)+EyeY,
                     ground_size,ground_size,$0000FF,64, PR_FILL);
                    CursorId:=2;
                    if Mouse_Click(M_RBUTTON) then begin
                      CursorState:=cs_none;
                      
                      aX:=(Mouse_X-EyeX+ground_size div 2) div ground_size;
                      aY:=(Mouse_Y-EyeY+ground_size div 2) div ground_size;
                      IgnoreEnemy:=False;
                      if MapCoast(aX,aY,0)<>-1 then begin

                        Inc(UnitsCount);
                        Units[UnitsCount].X:=aX;
                        Units[UnitsCount].Y:=aY;
                        Units[UnitsCount].UnitClass:=uc_build;
                        Units[UnitsCount].BildId:=PlaceId;
                        Units[UnitsCount].Enabled:=False;
                        Units[UnitsCount].State:=0;
                        Units[UnitsCount].BuildTime:=0;
                        Units[UnitsCount].Health:=ClassSets[ao_bases[PlaceId].I].Health;
                        Snd_Play(vBasePlaced,0,VOLUME(guiVolume));

                        Units[SelectedUnit].State:=us_goto;
                        Units[SelectedUnit].NextState:=us_build;
                        Units[SelectedUnit].Tag:=0;
                        Units[SelectedUnit].Tag2:=0;
                        f:=GetR(aX,aY,Units[SelectedUnit].X,Units[SelectedUnit].Y);
                        Units[SelectedUnit].toX:=f.X;
                        Units[SelectedUnit].toY:=f.Y;
                        Units[SelectedUnit].Speed:=0;
                        IgnoreId:=SelectedUnit;
                        Units[SelectedUnit].BildId:=UnitsCount;
                        IgnoreEnemy:=False;
                        FindPath(Units[SelectedUnit].Path,
                          Round(Units[SelectedUnit].X),Round(Units[SelectedUnit].Y),
                          Units[SelectedUnit].toX,Units[SelectedUnit].toY,MapCoast);
                        if Units[SelectedUnit].Path.Size=0 then begin
                          Units[SelectedUnit].State:=us_none;
                        end;
                        Units[SelectedUnit].PathId:=1;
                      end else begin

                      end;
                    end;
                  end;
                end;
              end;
              if Mouse_Click(M_RBUTTON) then begin
                Units[SelectedUnit].State:=us_goto;
                Units[SelectedUnit].NextState:=us_none;
                Units[SelectedUnit].Tag:=0;
                Units[SelectedUnit].Tag2:=0;
                Units[SelectedUnit].toX:=(Mouse_X-EyeX+ground_size div 2) div ground_size;
                Units[SelectedUnit].toY:=(Mouse_Y-EyeY+ground_size div 2) div ground_size;
                Units[SelectedUnit].Speed:=0;
                IgnoreId:=SelectedUnit;
                IgnoreEnemy:=False;
                FindPath(Units[SelectedUnit].Path,
                  Round(Units[SelectedUnit].X),Round(Units[SelectedUnit].Y),
                  Units[SelectedUnit].toX,Units[SelectedUnit].toY,MapCoast);
                if Units[SelectedUnit].Path.Size=0 then begin
                  Units[SelectedUnit].State:=us_none;
                end;
                Units[SelectedUnit].PathId:=1;
              end;
            end;
            uc_qualificator:begin
              case Units[SelectedUnit].State of
                as_closed:
                  if guiDrawButton('',forms_width+30,sY-forms_width+2,140)=button_click then begin
                    Units[SelectedUnit].State:=as_plasing;
                    Snd_Play(sndIIArming,0,VOLUME(guiVolume));
                    Units[SelectedUnit].Tag:=0;
                    Units[SelectedUnit].Tag2:=0;
                  end;
                as_opened:begin
                  if guiDrawButton('',forms_width+30,sY-forms_width+2,140)=button_click then begin
                    Units[SelectedUnit].State:=as_hiding;
                    Snd_Play(sndIIHiding,0,VOLUME(guiVolume));
                    Units[SelectedUnit].Tag:=0;
                    Units[SelectedUnit].Tag2:=0;
                  end;
                end;
              end;
            end;
            uc_antenne:begin
              case Units[SelectedUnit].State of
                as_closed:
                  if guiDrawButton('',forms_width+30,sY-forms_width+2,140)=button_click then begin
                    Units[SelectedUnit].State:=as_plasing;
                    Snd_Play(sndIIArming,0,VOLUME(guiVolume));
                    Units[SelectedUnit].Tag:=0;
                    Units[SelectedUnit].Tag2:=0;
                  end;
                as_opened:begin
                  if guiDrawButton('',forms_width+30,sY-forms_width+2,140)=button_click then begin
                    Units[SelectedUnit].State:=as_hiding;
                    Snd_Play(sndIIHiding,0,VOLUME(guiVolume));
                    Units[SelectedUnit].Tag:=0;
                    Units[SelectedUnit].Tag2:=0;
                  end;

                end;
              end;
            end;
          end;
          guiDrawForm('', sX-forms_width-10, sY-forms_width-30,forms_width,forms_width+20);
          guiDrawText(':', sX-forms_width, sY-40);
          Pr2D_RECT(sX-forms_width, sY-22, forms_width-20, 4, $777777, 255, PR_FILL);
          Pr2D_RECT(sX-forms_width, sY-22, (Units[SelectedUnit].Health/ClassSets[Units[SelectedUnit].UnitClass].Health)*(forms_width-20), 4, $FFFFFF, 255, PR_FILL);
          guiDrawText(': '+inttostr(Units[SelectedUnit].Piercing)+'.0',
            sX-forms_width, sY-54);
        end;

      end;
//      if GamePlan>=0 then begin
//
//        guiDrawText(GamePlans[GamePlan],sX-forms_width,sY-66);
//        if guiDrawButton(' ',sX-forms_width,sY-42,forms_width-20)=button_click then begin
//          GamePlan:=-1;
//
//        end;
//      end else begin
//        s:='';
//        for n:=0 to gp_count-1 do
//          s:=s+GamePlans[n]+'|';
//        guiDrawTabs(sX-forms_width-10,sY-forms_width-10,forms_width,forms_width-20,' ',s,GamePrePlan);
//        guiDrawText(GamePlansMore[GamePrePlan], sX-forms_width-4, sY-forms_width+12);
//        if guiDrawButton(' ',sX-forms_width,sY-66,forms_width-20)=button_click then begin
//          GamePlan:=GamePrePlan;
//
//        end;
//      end;

    end;
    gs_mainmenu:begin
      guiDrawForm(' ',sX div 2 - 98,sY div 2 - 80,196,162);
      eX:=sX div 2 - 98; eY:=sY div 2 - 86;
      if guiDrawButton(' ',eX+8,eY+32,180)=button_click then begin
        GameState:=gs_newgame_difficult;
      end;
      if guiDrawButton('  ',eX+8,eY+58,180)=button_click then begin
        SearchPlayers;
        if Length(Players)=0 then
          GameState:=gs_newgame_difficult
        else
          GameState:=gs_loadgame;
      end;
      if guiDrawButton('',eX+8,eY+84,180)=button_click then begin
        SearchPlayers;
        for n:=0 to Length(Players)-1 do
         for m:=0 to Length(Players)-2 do
           if Players[m].Kills<Players[m+1].Kills then begin
             TempPlayer:=Players[m];
             Players[m]:=Players[m+1];
             Players[m+1]:=TempPlayer;
           end;
        GameState:=gs_records;
      end;
      if guiDrawButton('',eX+8,eY+110,180)=button_click then begin
        GameState:=gs_author;
      end;
      if guiDrawButton('',eX+8,eY+136,180)=button_click then begin
        GameQuit;
      end;
    end;
    gs_author:begin
      guiDrawForm('',sX div 2 - 98,sY div 2 - 70,196,140);
      eX:=sX div 2 - 98; eY:=sY div 2 - 74;
      guiDrawText(':           Dj_smart'+#13+
                  ' :   igdc.ru'+#13+
                  ':               1.0'+#13+
                  ''+#13+
                  ':       [icq]9804336',eX+8,eY+32);
      if guiDrawButton('',eX+8,eY+110,180)=button_click then begin
        GameState:=gs_mainmenu;
      end;
    end;
    gs_newgame_difficult:begin
      guiDrawForm(' ',sX div 2 - 98,sY div 2 - 70,196,140);
      eX:=sX div 2 - 98; eY:=sY div 2 - 74;
      if guiDrawButton(gd_names[gd_easy],eX+8,eY+32,180)=button_click then begin
        GameDifficult:=gd_easy;
        GameState:=gs_newgame_name;
      end;
      if guiDrawButton(gd_names[gd_normal],eX+8,eY+58,180)=button_click then begin
        GameDifficult:=gd_normal;
        GameState:=gs_newgame_name;
      end;
      if guiDrawButton(gd_names[gd_hard],eX+8,eY+84,180)=button_click then begin
        GameDifficult:=gd_hard;
        GameState:=gs_newgame_name;
      end;
      if guiDrawButton(', !',eX+8,eY+110,180)=button_click then begin
        GameState:=gs_mainmenu;
      end;
    end;
    gs_newgame_name:begin
      guiDrawForm('  ',sX div 2 - 98,sY div 2 - 70,196,140);
      eX:=sX div 2 - 98; eY:=sY div 2 - 74;
      guiDrawText(Iam,eX+8,eY+32);
      guiDrawEdit(@edMyName);
      if edMyName.Text<>'' then
        if guiDrawButton('',eX+8,eY+84,180)=button_click then begin
          if DirectoryExists('Data/Players/'+edMyName.Text) then begin
            Iam:='   !';
          end else begin
            MkDir('Data/Players/'+edMyName.Text);
            IniSet('Data/Players/'+edMyName.Text+'/Player.ini','Player','difficult',IntToStr(GameDifficult));
            LoadGame(edMyName.Text);
          end;
        end;
      if guiDrawButton(' ',eX+8,eY+110,180)=button_click then begin
        GameState:=gs_mainmenu;
      end;
    end;
    gs_records:begin
      guiDrawForm(' ',sX div 2 - 98,sY div 2 - 32 - Length(Players)*32,196,64+Length(Players)*64);
      eX:=sX div 2 - 98; eY:=sY div 2 - 32 - (Length(Players)-1)*32 - 24;
      for n:=0 to Length(Players)-1 do begin
        guiDrawText(Players[n].Name,eX+8,eY+n*64+20);
        Pr2D_Rect(eX+8,eY+n*64+38,179,2,$333333,255,PR_FILL);
        Pr2D_Rect(eX+8,eY+n*64+38,(Players[n].Progress/100)*179,2,$FFFFFF,255,PR_FILL);
        guiDrawText(': '+inttostr(Players[n].Progress)+'%'+#13+
          ': '+gd_names[Players[n].Difficult]+#13+
          ': '+inttostr(Players[n].Kills),
          eX+8,eY+n*64+43);                          
        
      end;
      if guiDrawButton('',eX+8,eY+Length(Players)*64+20,180)=button_click then begin
        GameState:=gs_mainmenu;
      end;
    end;
    gs_loadgame:begin
      guiDrawForm(' ',sX div 2 - 98,sY div 2 - 32 - Length(Players)*32,196,64+Length(Players)*64);
      eX:=sX div 2 - 98; eY:=sY div 2 - 32 - (Length(Players)-1)*32 - 24;
      for n:=0 to Length(Players)-1 do begin
        Pr2D_Rect(eX+8,eY+n*64+48,179,2,$333333,255,PR_FILL);
        Pr2D_Rect(eX+8,eY+n*64+48,(Players[n].Progress/100)*179,2,$FFFFFF,255,PR_FILL);
        guiDrawText(': '+inttostr(Players[n].Progress)+'%',
          eX+8,eY+n*64+53);
        guiDrawText(': '+gd_names[Players[n].Difficult],
          eX+8,eY+n*64+66);
        if guiDrawButton(Players[n].Name,eX+8,eY+n*64+20,180)=button_click then begin
          LoadGame(Players[n].Name);
        end;
      end;
      if guiDrawButton(' ',eX+8,eY+Length(Players)*64+20,180)=button_click then begin
        GameState:=gs_mainmenu;
      end;
    end;
  end;
  guiDraw;
  ASprite_Draw(txtCursor, Mouse_X-32, Mouse_Y-32, 64, 64, CursorId);
  CursorId:=1;
end;

procedure GameSlovProcess;
var n,m:Integer;
begin
  if GameTime>0 then begin
    Dec(GameTime);
    if GameTime<3 then Snd_Play(sndBeep,0,VOLUME(guiVolume));
  end;
  if PlayStart>0 then begin
    Dec(PlayStart);
    if PlayStart=0 then begin
      
      Media_Play('Data/Sound/Game.ogg',True);
      Media_SetVolume(VOLUME(Round(guiVolume*0.9)));
      //Media_SetPosition(0);
    end;
  end;
  if GameState<>gs_game then Exit;
  for n:=1 to UnitsCount do
    if (Units[n].UnitClass=uc_other) and (Units[n].Qualification=rs_firer) and (Units[n].Enabled) then begin
      if Units[n].FireToId=0 then begin
        for m:=1 to UnitsCount do
         if (Units[m].Destroyed=false) and (Units[m].Health>0) and (Units[m].UnitClass=uc_enemy) and (M_Distance(Units[n].X,Units[n].Y,Units[m].X,Units[m].Y)<=see_dist) then begin
           Units[n].FireToId:=m;
           Break;
         end;
      end;
    end;
end;

procedure GameProcess;
var n,m,j,z,k,kI:Integer;
    TempA,Temp2A,h:Single;
    TempX,TempY:Integer;
    b:Boolean;
    f:TPoint;
begin
  if GameState in [gs_mainmenu,gs_newgame_difficult,gs_newgame_name,gs_loadgame] then begin
    if (NoiseLen=0) then begin
      if (NoiseStart=0) then begin
        NoiseLen:=noise_time;
        Snd_Play(sndNoise,0,VOLUME(guiVolume));
      end else Dec(NoiseStart);
    end else begin
      Dec(NoiseLen);
      if NoiseLen = noise_time div 2 then begin
        Inc(NoiseId);
        if NoiseId>4 then NoiseId:=1;
      end;
      Inc(NoiseStep);
      if NoiseStep>10 then begin
        NoiseStep:=0;
        NoiseRand:=Random(4)+1;
      end;
      if NoiseLen=0 then begin
        NoiseStart:=Random(100)+noise_start;
      end;
    end;
  end;
  case GameState of
    gs_game:begin
      if (GameTime=0) and (GameEnemies>0) then begin
        Dec(Ecreation);
        if Ecreation<=0 then begin
          j:=Round(ecreation_time/(GameDifficult+1));
          Ecreation:=j+Random(j);
          CreateEnemy;
          Dec(GameEnemies);
        end;
      end;

      if (Mouse_X<4) and (EyeX<0) then begin
        if EyeXVol<ground_size then EyeXVol:=EyeXVol+1;
      end else if (Mouse_X>sX-4) and (EyeX>-MapX*ground_size+sX) then begin
        if EyeXVol>-ground_size then EyeXVol:=EyeXVol-1;
      end else EyeXVol:=EyeXVol/1.5;
      if (Mouse_Y<4) and (EyeY<0) then begin
        if EyeYVol<ground_size then EyeYVol:=EyeYVol+1;
      end else if (Mouse_Y>sY-4) and (EyeY>-MapY*ground_size+sY) then begin
        if EyeYVol>-ground_size then EyeYVol:=EyeYVol-1;
      end else EyeYVol:=EyeYVol/1.5;
      EyeY:=EyeY+round(EyeYVol);
      EyeX:=EyeX+round(EyeXVol);
      if EyeX>-32 then EyeX:=-32;
      if EyeY>-32 then EyeY:=-32;
      if EyeX<-(MapX-1)*ground_size+sX then EyeX:=-(MapX-1)*ground_size+sX;
      if EyeY<-(MapY-1)*ground_size+sY then EyeY:=-(MapY-1)*ground_size+sY;
      if (LastEyeX<>EyeX) or (LastEyeY<>EyeY) then begin
        LastEyeX:=EyeX;
        LastEyeY:=EyeY;
        NeedRender:=True;
      end;
      RadioTreeCount:=0;
      for n:=1 to UnitsCount do begin
        if Units[n].Health<=0 then Continue;
        Units[n].Detected:=False;
        Units[n].DevicesCount:=0;
      end;
      //for n:=1 to MessagesCount do
      //  if (Messages[n].Time=0) and (Messages[n].AdressId<>Messages[n].ToId) then
      //    AddMessage(Messages[n].MsgType,Messages[n].ToId,Messages[n].AdressId,Messages[n].Param1,Messages[n].Param2);
      n:=1;
      while n<=TracksCount do begin
        Dec(Tracks[n].Time);
        if Tracks[n].Time=0 then begin
          Tracks[n]:=Tracks[TracksCount];
          Dec(TracksCount);
          Dec(n);
        end;
        Inc(n);
      end;
      n:=1;
      while n<=BulletsCount do begin
        Dec(Bullets[n].Time);
        Bullets[n].X:=Bullets[n].X-M_Cos(Bullets[n].Angle)*bullet_speed;
        Bullets[n].Y:=Bullets[n].Y-M_Sin(Bullets[n].Angle)*bullet_speed;
        if Bullets[n].Time=0 then begin
          //Addtolog('bullet');
          Dec(Units[Bullets[n].KillId].Health);
          if Units[Bullets[n].KillId].Health<=0 then begin
            if Units[Bullets[n].KillId].Destroyed=False then begin
              //Addtolog(' d');
              AddExplosion(Round(Bullets[n].X),Round(Bullets[n].Y));
              Units[Bullets[n].KillId].Destroyed:=true;
              dec(GameEnemiesKilled);                       
              //Addtolog(' x');
              Inc(Kills);
              if GameEnemiesKilled=0 then begin
                FinishGame;
                Break;
              end;
             // Addtolog(' z');
            end;
          end;

          Bullets[n]:=Bullets[BulletsCount];
          Dec(BulletsCount);
          Dec(n);
          //Addtolog('end');
        end;
        Inc(n);
      end;
      n:=1;
      while n<=ExplosionsCount do begin
        Dec(Explosions[n].Time);
        if Explosions[n].Time<=0 then begin
          Explosions[n]:=Explosions[ExplosionsCount];
          Dec(ExplosionsCount);
          Dec(n);
        end;
        Inc(n);
      end;
      for n:=1 to UnitsCount do begin
        if Units[n].Health<=0 then Continue;
        case Units[n].UnitClass of
          uc_enemy:begin
            if Units[n].State<>us_goto then begin
              if (Units[n].PTag <= 0) or (Units[n].PTag >= GrassIdsCount+1)  then begin
                if GrassIdsCount>0 then begin
                  j:=Random(GrassIdsCount)+1;
                  Units[n].State:=us_goto;
                  Units[n].NextState:=us_none;
                  Units[n].PTag:=j;
                  Units[n].Tag:=0;
                  Units[n].Tag2:=0;
                  IgnoreEnemy:=True;
                  Units[n].toX:=Round(Units[GrassIds[j]].X);
                  Units[n].toY:=Round(Units[GrassIds[j]].Y);
                  //Addtolog('J:'+IntToStr(J)+' COUNT:'+inttostr(GrassIdsCount)+' XY:'+IntToStr(f.X)+'-'+IntToStr(f.Y)+' UNITID:'+inttostr(GrassIds[j])+
                  //  ' UNITPOS:'+FloatToStr(Units[GrassIds[j]].X)+'-'+FloatToStr(Units[GrassIds[j]].Y)+' H:'+
                  //   IntToStr(Units[GrassIds[j]].Health));
                  Units[n].Speed:=0;
                  IgnoreId:=n;
                  
                  FindPath(Units[n].Path,
                    Round(Units[n].X),Round(Units[n].Y),
                    Units[n].toX,Units[n].toY,MapCoast);
                  if Units[n].Path.Size=0 then begin
                    Units[n].State:=us_none;

                  end;
                  Units[n].PathId:=1;
                end;
              end else begin
                if Units[GrassIds[Units[n].PTag]].Health>0 then begin
                  Dec(Units[GrassIds[Units[n].PTag]].Health);
                  if Units[GrassIds[Units[n].PTag]].Health<=0 then begin
                    //Addtolog('before(delete '+inttostr(Units[n].PTag)+'):');
                    //WriteArr;
                    for j:=1 to UnitsCount do
                      if (Units[j].UnitClass=uc_enemy) and (j<>n) and (Units[j].PTag=Units[n].PTag) then
                        Units[j].PTag:=0;
                    for j:=1 to UnitsCount do
                      if (Units[j].UnitClass=uc_enemy) and (j<>n) and (Units[j].PTag=GrassIdsCount) then
                        Units[j].PTag:=Units[n].PTag;
                    GrassIds[Units[n].PTag]:=GrassIds[GrassIdsCount];
                    Dec(GrassIdsCount);
                    if GrassIdsCount=0 then begin
                      GameOver;
                      Exit;
                    end;

                    //Addtolog('after:');
                    Units[n].PTag:=0;
                    //WriteArr;
                  end;
                end else begin
                  Units[n].PTag:=0;
                end;
              end;
            end;
          end;
        end;
      end;
      for n:=1 to UnitsCount do begin
        if Units[n].Health<=0 then Continue;
        case Units[n].UnitClass of
          uc_other:begin
            case Units[n].Qualification of
              rs_firer:begin
                if Units[n].State=us_none then begin
                  Dec(Units[n].QTag);
                end;
                if Units[n].FireToId=0 then begin
                  Inc(Units[n].TurrelAngle);
                  if Units[n].TurrelAngle>=360 then Units[n].TurrelAngle:=0;
                end else begin
                  j:=Units[n].FireToId;
                  h:=M_Distance(Units[n].X,Units[n].Y,Units[j].X,Units[j].Y);
                  if h>see_dist then begin
                    Units[n].FireToId:=0;
                    Units[n].Tag2:=0;
                  end else begin
                    TempA:=Units[n].TurrelAngle;
                    Temp2A:=M_FindAngle(Round(Units[n].X*ground_size),
                      Round(Units[n].Y*ground_size),
                      Round(Units[j].X*ground_size),
                      Round(Units[j].Y*ground_size));
                    if TempA<0 then TempA:=TempA+360;
                    if Temp2A<0 then Temp2A:=Temp2A+360;
                    if TempA>360 then TempA:=TempA-360;
                    if Temp2A>360 then Temp2A:=Temp2A-360;
                    if abs(Temp2A-TempA)<180 then begin
                      if Temp2A>TempA then Units[n].TurrelAngle:=Units[n].TurrelAngle+2 else
                        Units[n].TurrelAngle:=Units[n].TurrelAngle-2
                    end else begin
                      if Temp2A<TempA then Units[n].TurrelAngle:=Units[n].TurrelAngle+2 else
                        Units[n].TurrelAngle:=Units[n].TurrelAngle-2;
                    end;
                    if (abs(Temp2A-TempA)<3) or (abs(Temp2A-TempA)>357) then begin
                      Inc(Units[n].Tag2);
                      if Units[n].Tag2>=firer_time then begin
                        Units[n].Tag2:=0;
                        Snd_Play(sndWeapon);
                        AddBullet(Units[n].X*ground_size,Units[n].Y*ground_size,Units[n].FireToId,Units[n].TurrelAngle,Round(h*ground_size));
                      end;
                    end;
                  end;
                  if Units[j].Destroyed then begin
                    Units[n].FireToId:=0;
                  end;
                end;
                if Units[n].QTag<=0 then begin
                  Units[n].State:=us_goto;
                  Units[n].NextState:=us_none;
                  Units[n].Tag:=0;
                  Units[n].Tag2:=0;
                  j:=Random(RescuePointsCount)+1;
                  Units[n].toX:=RescuePoints[j].X;
                  Units[n].toY:=RescuePoints[j].Y;
                  Units[n].Speed:=0;
                  IgnoreId:=n;
                  IgnoreEnemy:=False;
                  FindPath(Units[n].Path,
                    Round(Units[n].X),Round(Units[n].Y),
                    Units[n].toX,Units[n].toY,MapCoast);
                  if Units[n].Path.Size=0 then begin
                    Units[n].State:=us_none;
                  end else Units[n].QTag:=qfirer_time div 2 + random(qfirer_time);
                  Units[n].PathId:=1;
                end;
              end;
            end;
          end;
        end;
        case Units[n].UnitClass of
          uc_grass:begin

          end;
          uc_programmator,uc_other,uc_enemy:begin
            case Units[n].State of
              us_build,us_open,us_qualify,us_qualifyset:begin
                TempA:=Units[n].Angle;
                Temp2A:=M_FindAngle(Round(Units[n].X*ground_size),
                  Round(Units[n].Y*ground_size),
                  Round(Units[Units[n].BildId].X*ground_size),
                  Round(Units[Units[n].BildId].Y*ground_size));
                if TempA<0 then TempA:=TempA+360;
                if Temp2A<0 then Temp2A:=Temp2A+360;
                if TempA>360 then TempA:=TempA-360;
                if Temp2A>360 then Temp2A:=Temp2A-360;
                if (abs(Temp2A-TempA)<3) or (abs(Temp2A-TempA)>357) then begin
                  case Units[n].State of
                    us_build:begin
                      Inc(Units[Units[n].BildId].BuildTime);
                      if Units[Units[n].BildId].BuildTime>=ao_bases[Units[Units[n].BildId].BildId].Bt then begin
                        Units[Units[n].BildId].UnitClass:=ao_bases[Units[Units[n].BildId].BildId].I;
                        Units[n].State:=us_none;
                        Units[Units[n].BildId].Enabled:=true;
                        Snd_Play(vBildingDone,0,VOLUME(guiVolume));
                      end;
                    end;
                    us_open:begin
                      Units[Units[n].BildId].Enabled:=true;
                      Units[n].State:=us_none;
                    end;
                    us_qualifyset:begin
                      Units[n].Qualification:=Units[n].Q;
                      Units[n].QTag:=0;
                      Units[n].State:=us_none;
                      Snd_Play(sndQSet);
                    end;
                    us_qualify:begin
                      j:=Units[n].BildId;
                      k:=MapX*2;
                      b:=False;
                      for m:=1 to UnitsCount do begin
                        if (Units[m].UnitClass=uc_qualificator) and (Units[m].Enabled) then begin
                          z:=Round(M_Distance(Units[m].X,Units[m].Y,
                            Units[j].X,Units[j].Y));
                          if z<k then begin
                            b:=True;
                            k:=z;
                            kI:=m;
                          end;
                        end;
                      end;

                      if b then begin
                        Units[j].State:=us_goto;
                        Units[j].NextState:=us_qualifyset;
                        Units[j].Tag:=kI;
                        Units[j].Tag2:=0;
                        Units[j].Q:=Units[n].Q+1;
                        f:=GetR(Round(Units[kI].X),Round(Units[kI].Y),Units[j].X,Units[j].Y);
                        Units[j].toX:=f.X;
                        Units[j].toY:=f.Y;
                        Units[j].Speed:=0;
                        IgnoreId:=j;
                        Units[j].BildId:=kI;
                        IgnoreEnemy:=False;
                        FindPath(Units[j].Path,
                          Round(Units[j].X),Round(Units[j].Y),
                          Units[j].toX,Units[j].toY,MapCoast);
                        if Units[j].Path.Size=0 then begin
                          Units[j].State:=us_none;
                        end;
                        Units[j].PathId:=1;
                        Units[n].State:=us_none;
                      end;
                    end;
                  end;
                end else begin
                  if abs(Temp2A-TempA)<180 then begin
                    if Temp2A>TempA then Units[n].Angle:=Units[n].Angle+2 else
                      Units[n].Angle:=Units[n].Angle-2
                  end else begin
                    if Temp2A<TempA then Units[n].Angle:=Units[n].Angle+2 else
                      Units[n].Angle:=Units[n].Angle-2;
                  end;
                  inc(Units[n].Tag);
                  if Units[n].Tag>=4 then begin
                    AddTrack(Round(Units[n].X*ground_size),Round(Units[n].Y*ground_size),
                      Units[n].Angle,n);
                    Units[n].Tag:=0;
                  end;
                end;
              end;
              us_goto:begin
                if (Units[n].PathId>Units[n].Path.Size-1) or (Units[n].PathId<1) then begin
                  Units[n].State:=us_none;
                  Continue;
                end;
                TempA:=Units[n].Angle;
                TempX:=Units[n].Path.Path[Units[n].PathId].X;
                TempY:=Units[n].Path.Path[Units[n].PathId].Y;
                IgnoreId:=n;

                if MapCoast(TempX,TempY,0)=-1 then begin
                  IgnoreEnemy:=Units[n].UnitClass=uc_enemy;
                  FindPath(Units[n].Path,Round(Units[n].X),Round(Units[n].Y),
                    Units[n].toX,Units[n].toY,MapCoast);
                  if Units[n].Path.Path<>nil then begin
                    Units[n].PathId:=1;
                    TempX:=Units[n].Path.Path[1].X;
                    TempY:=Units[n].Path.Path[1].Y;
                  end else begin
                    Units[n].State:=Units[n].NextState;
                    Continue;
                  end;
                end;
                if M_Distance(Units[n].X,Units[n].Y,TempX,TempY)<0.1 then begin
                  if Units[n].PathId=Units[n].Path.Size-1 then begin
                    Units[n].State:=Units[n].NextState;
                    case Units[n].State of
                      us_build:begin
                        Snd_Play(vBilding,0,VOLUME(guiVolume));
                      end;
                      us_qualifyset:begin
                      
                      end;
                    end;
                  end else begin
                    inc(Units[n].PathId);
                  end;
                end else begin
                  Units[n].X:=Units[n].X-M_Cos(Units[n].Angle)*Units[n].Speed;
                  Units[n].Y:=Units[n].Y-M_Sin(Units[n].Angle)*Units[n].Speed;
                  Temp2A:=M_FindAngle(Round(Units[n].X*ground_size),
                    Round(Units[n].Y*ground_size),
                    Round(TempX*ground_size),
                    Round(TempY*ground_size));
                  if TempA<0 then TempA:=TempA+360;
                  if Temp2A<0 then Temp2A:=Temp2A+360;
                  if TempA>360 then TempA:=TempA-360;
                  if Temp2A>360 then Temp2A:=Temp2A-360;
                  if Units[n].Speed<0.015 then
                    Units[n].Speed:=Units[n].Speed+0.001;

                  if (abs(Temp2A-TempA)<3) or (abs(Temp2A-TempA)>357) then begin
                    Units[n].Speed:=Units[n].Speed/1.001;
                    inc(Units[n].Tag);
                    if Units[n].Tag>=8 then begin
                      AddTrack(Round(Units[n].X*ground_size),Round(Units[n].Y*ground_size),
                        Units[n].Angle,n);
                      Units[n].Tag:=0;
                    end;
                  end else begin
                    if abs(Temp2A-TempA)<180 then begin
                      if Temp2A>TempA then Units[n].Angle:=Units[n].Angle+2 else
                        Units[n].Angle:=Units[n].Angle-2
                    end else begin
                      if Temp2A<TempA then Units[n].Angle:=Units[n].Angle+2 else
                        Units[n].Angle:=Units[n].Angle-2;
                    end;
                    inc(Units[n].Tag);
                    if Units[n].Tag>=4 then begin
                      AddTrack(Round(Units[n].X*ground_size),Round(Units[n].Y*ground_size),
                        Units[n].Angle,n);
                      Units[n].Tag:=0;
                    end;
                  end;
                end;
              end;
            end;
          end;
          uc_qualificator:begin
            case Units[n].State of
              as_plasing:begin
                Inc(Units[n].Tag2);
                if Units[n].Tag2>ao_animspeed then begin
                  NeedRender:=True;
                  Units[n].Tag2:=0;
                  Inc(Units[n].Tag);
                  if Units[n].Tag>14 then begin
                    Units[n].Tag:=0;
                    Units[n].Tag2:=0;
                    Units[n].State:=as_opened;
                    NeedRender:=True;
                    Units[n].IsSendHello:=False;
                  end;
                end;
              end;
              as_hiding:begin
                Inc(Units[n].Tag2);
                if Units[n].Tag2>ao_animspeed then begin
                  NeedRender:=True;
                  Units[n].Tag2:=0;
                  Inc(Units[n].Tag);
                  if Units[n].Tag>14 then begin
                    Units[n].Tag:=0;
                    Units[n].Tag2:=0;
                    Units[n].State:=as_closed;
                  end;
                end;
              end;
              as_opened:begin
                Inc(Units[n].Tag2);
                if Units[n].Tag2>360 then begin
                  Units[n].Tag2:=0;
                end;
              end;
            end;
          end;
          uc_antenne:begin
            case Units[n].State of
              as_plasing:begin
                Inc(Units[n].Tag2);
                if Units[n].Tag2>ao_animspeed then begin
                  NeedRender:=True;
                  Units[n].Tag2:=0;
                  Inc(Units[n].Tag);
                  if Units[n].Tag>18 then begin
                    Units[n].Tag:=0;
                    Units[n].Tag2:=0;
                    Units[n].State:=as_opened;
                    NeedRender:=True;
                    Units[n].IsSendHello:=False;
                  end;
                end;
              end;
              as_hiding:begin
                Inc(Units[n].Tag2);
                if Units[n].Tag2>ao_animspeed then begin
                  NeedRender:=True;
                  Units[n].Tag2:=0;
                  Inc(Units[n].Tag);
                  if Units[n].Tag>18 then begin
                    Units[n].Tag:=0;
                    Units[n].Tag2:=0;
                    Units[n].State:=as_closed;
                  end;
                end;
              end;
              as_opened:begin
                AddRadioItem(4,0,n,-1);
                Inc(Units[n].Tag2);
                if Units[n].Tag2>360 then begin
                  Units[n].Tag2:=0;
                end;
              end;
            end;
          end;
        end;
        Units[n].BeepCounter:=Units[n].BeepCounter+1;
        //Units[n].Connecting:=0;
        if Units[n].BeepCounter>beep_time then begin
          Units[n].BeepCounter:=0;
        end;
      end;
      if RadioTreeCount>LastRadioTreeCount then begin
        Snd_Play(sndAddDevice,0,VOLUME(guiVolume));
      end;
      if RadioTreeCount<LastRadioTreeCount then begin
        Snd_Play(sndDeleteDevice,0,VOLUME(guiVolume));
      end;
      LastRadioTreeCount:=RadioTreeCount;
      NeedRender:=True;
      if NeedRender then begin
        RenderShadow;
        NeedRender:=False;
      end;
      if Key_Down(K_ESCAPE) then GameState:=gs_esc;
    end;
    gs_mainmenu:begin
      if Key_Down(K_ESCAPE) then GameQuit;
    end;
  end;
  guiRefreshMouse;
  guiProcess;
end;

end.

