unit u_map;

interface

  uses OMEGA, u_const, Classes, Graphics, PathFind, SysUtils;

  type
    TMapCell=record
      Ground:Byte;
      doGround:Boolean;
    end;

  var
    txLevelBG:PTexture;
    MapX,MapY:Integer;
    Map:array of array of TMapCell;

  procedure LoadMap(Level:integer);
  procedure DrawMap;
  function GetSubPoint(x,y:integer):integer;
  function MapCoast(X,Y,Direction : Integer):integer;

implementation

uses u_game;

  function MapCoast(X,Y,Direction : Integer):integer;
  var n:Integer;
  begin
    result:=1;
    for n:=1 to UnitsCount do begin
      if IgnoreEnemy and ((Units[n].UnitClass=uc_enemy) or (Units[n].UnitClass=uc_grass)) then Continue;
      if (n<>IgnoreId) and (Units[n].Health>0) then
      if (Round(Units[n].X)=X) and (Round(Units[n].Y)=Y) then begin
        Result:=-1;
        Exit;
      end;
    end;
    case Direction of
      0,2,4,6:Result:=6;
      else
      Result:=7;
    end;
  end;

  procedure DrawMap;
  var n,m:Integer;
  begin
    STile_Draw(txLevelBG,EyeX mod bg_size,EyeY mod bg_size,bg_size,bg_size,
      sX div bg_size+2, sY div bg_size+2);
    for m:=0 to MapY-1 do
      for n:=0 to MapX-1 do begin
        if Map[n,m].Ground>0 then begin
          ASprite_Draw(txGround,
            n*ground_size+EyeX+ground_size div 2,
            m*ground_size+EyeY+ground_size div 2,ground_size,ground_size, Map[n,m].Ground);
        end;
      end;
  end;

  function GetSubPoint(x,y:integer):integer;
  begin
    if Map[x,y].doGround then begin
      result:=Random(4)+17;
    end else begin
      result:=0;
      if (y>0) and (Map[x,y-1].doGround) then result:=result+1;
      if (x<MapX-1) and (Map[x+1,y].doGround) then result:=result+8;
      if (y<MapY-1) and (Map[x,y+1].doGround) then result:=result+4;
      if (x>0) and (Map[x-1,y].doGround) then result:=result+2;
    end;
  end;

  procedure LoadMap(Level:integer);
  var bit:TBitmap;
     n,m:integer;
  begin
    if Assigned(txLevelBG) then begin
      Tex_Free(txMap);
    end else begin
    end;
    RescuePointsCount:=0;
    EnemyPointsCount:=0;
    GrassIdsCount:=0;
    GameDemo:=IniGet('Data/Levels/'+U_inttostr(Level)+'/Config.ini','Main','demo','false')='true';
    GameTime:=StrToIntDef(IniGet('Data/Levels/'+U_inttostr(Level)+'/Config.ini','Main','time','300'),300);
    GameEnemies:=StrToIntDef(IniGet('Data/Levels/'+U_inttostr(Level)+'/Config.ini','Main','enemies','0'),0)*(GameDifficult+1);
    GameEnemiesKilled:=GameEnemies;
    DemoId:=0;
    bit:=TBitmap.Create;
    bit.LoadFromFile('Data/Levels/'+U_inttostr(Level)+'/Data.bmp');
    MapX:=bit.Width;
    MapY:=Bit.Height;
    InitMap(MapX,MapY);
    SetLength(Map,MapX,MapY);
    for m:=1 to MapY do
      for n:=1 to MapX do
        Map[n-1,m-1].doGround:=bit.Canvas.Pixels[n-1,m-1]<>$FFFFFF;
    for m:=1 to MapY-1 do
      for n:=1 to MapX-1 do
        Map[n-1,m-1].Ground:=GetSubPoint(n,m);
    txMap:=Tex_CreateZero(MapX,MapY,TEX_OMEGA_TO_2X2 or TEX_FILTER_NEAREST or TEX_CLAMP);
    Tex_BeginRenderIn(txMap,false,True,False);
      Pr2D_Rect(0,0,MapX,MapY,$9DBBCD,255,PR_FILL);
      for m:=0 to MapY-1 do
        for n:=0 to MapX-1 do
          if Map[n,m].doGround then
            Pr2D_Pixel(n,m,$3A4D5B);
    Tex_EndRenderIn;
    UnitsCount:=0;
    for m:=1 to MapY do
      for n:=1 to MapX do
        if (bit.Canvas.Pixels[n,m]<>$FFFFFF) and (bit.Canvas.Pixels[n,m]<>$000000) then begin
          Inc(UnitsCount);
          Units[UnitsCount].X:=n;
          Units[UnitsCount].Y:=m;
          Units[UnitsCount].BeepCounter:=Random(beep_time);
          Units[UnitsCount].Angle:=Random(360);
          Units[UnitsCount].Piercing:=1;
          Units[UnitsCount].Tag:=0;
          Units[UnitsCount].Tag2:=0;
          Units[UnitsCount].Qualification:=0;
          Units[UnitsCount].Q:=0;
          Units[UnitsCount].Connecting:=0;
          Units[UnitsCount].State:=0;
          case bit.Canvas.Pixels[n,m] of
            $6E00FF:begin
                      Inc(RescuePointsCount);
                      RescuePoints[RescuePointsCount].X:=n;
                      RescuePoints[RescuePointsCount].Y:=m;
                    end;
            $006AFF:begin
                      Inc(EnemyPointsCount);
                      EnemyPoints[EnemyPointsCount].X:=n;
                      EnemyPoints[EnemyPointsCount].Y:=m;
                    end;
          end;
          case bit.Canvas.Pixels[n,m] of
            $0000FF:begin
                      Units[UnitsCount].UnitClass:=uc_antenne;
                      Units[UnitsCount].Enabled:=true;
                      Units[UnitsCount].Health:=ClassSets[uc_antenne].Health;
                      Units[UnitsCount].IsSendHello:=False;
                      SelectedUnit:=UnitsCount;
                      AntenneId:=UnitsCount;
                    end;
            $21FF00:begin
                      Units[UnitsCount].UnitClass:=uc_programmator;
                      Units[UnitsCount].Enabled:=True;
                      Units[UnitsCount].Health:=ClassSets[uc_programmator].Health;
                    end;
            $00D8FF:begin
                      Units[UnitsCount].UnitClass:=uc_other;
                      Units[UnitsCount].Enabled:=False;

                      Units[UnitsCount].Health:=ClassSets[uc_other].Health;
                      RobotId:=UnitsCount;
                    end;
            $FF9400:begin
                      Units[UnitsCount].UnitClass:=uc_grass;
                      Units[UnitsCount].Enabled:=False;
                      Units[UnitsCount].Tag:=Random(3);
                      Units[UnitsCount].Health:=ClassSets[uc_grass].Health;
                      Inc(GrassIdsCount);
                      GrassIds[GrassIdsCount]:=UnitsCount;
                    end;
            else Dec(UnitsCount);
          end;
        end;
    bit.Free;
    map_px:=(forms_width-20)/((MapX-1)*ground_size);
    map_py:=(forms_width-20)/((MapY-1)*ground_size);
  end;

end.
