unit GameTypes;

interface

uses
  Lapage,
  GameConstr;

type
  TGameArea = class;
  TGameObject = class;

  TGame = class
  private
    FArea: TGameArea;
    FTilesSpt: TLpgList;
    FArrowsSpt: TLpgList;
  public
    constructor Create;
    destructor Destroy; override;

    property Area: TGameArea read FArea;
    procedure Draw(Buff: TLpgRowsBuffer);
  end;

  TGameCell = class
  private
    FArea: TGameArea;
    FOrientation : Byte;
    FTileIndex :  Integer;
  public
    constructor Create(AArea: TGameArea);
    destructor Destroy; override;
  end;

  TGameArea = class
  private
    FGame: TGame;
    FWidth: Word;
    FHeight: Word;
    FCells: array of array of TGameCell;
    FOffsetX: Integer;
    FOffsetY: Integer;
  public
    constructor Create(AGame: TGame);
    destructor Destroy; override;

    procedure SetSize(ANewWidth, ANewHeight: Word);
    property Width: Word read FWidth;
    property Height: Word read FHeight;
    property OffsetX: Integer read FOffsetX;
    property OffsetY: Integer read FOffsetY;

    procedure Draw(ABuff: TLpgRowsBuffer);
  end;

  TGameObject = class

  end;

implementation

{ TGame }

constructor TGame.Create;
begin
  inherited;
  FArea := TGameArea.Create(self);
  FTilesSpt := LpgCreateCropList('data\tiles.png', $FF000000, Tile_Width,
    Tile_Height, LAPAGE_OBJECT_ROWBUFFER);
  FArrowsSpt := LpgCreateCropList('data\Arrows.png', $FF000000, Arrow_Width,
    Arrow_Height, LAPAGE_OBJECT_ROWBUFFER);
end;

destructor TGame.Destroy;
begin
  if Assigned(FArea) then
    FArea.Free;
  FTilesSpt.Release;
  inherited;
end;

procedure TGame.Draw(Buff: TLpgRowsBuffer);
begin
  Area.Draw(Buff);
end;

{ TGameArea }

constructor TGameArea.Create(AGame: TGame);
begin
  inherited Create;
  FGame := AGame;
end;

destructor TGameArea.Destroy;
begin
  SetSize(0, 0);
  inherited;
end;

procedure TGameArea.Draw(ABuff: TLpgRowsBuffer);
var
  i, j: Integer;
begin
  for i := 0 to Width - 1 do
    for j := 0 to Height - 1 do
    begin
      FGame.FTilesSpt.Draw(ABuff, i * Tile_Width + OffsetX,
        j * Tile_Height + OffsetY, FCells[i,j].FTileIndex);
      if FCells[i,j].FTileIndex < 2 then
      FGame.FArrowsSpt.Draw(ABuff, i *Tile_Width +OffsetX + 1,
        j* Tile_Height + OffsetY+ 1,
        FCells[i,j].FOrientation);
    end;
end;

procedure TGameArea.SetSize(ANewWidth, ANewHeight: Word);
var
  i, j: Integer;
begin
  for i := 0 to Width - 1 do
    for j := 0 to Height - 1 do
    begin
      FCells[i, j].Free;
    end;

  FWidth := ANewWidth;
  FHeight := ANewHeight;

  if FWidth >= (Screen_Width div Tile_Width) then
    FWidth := (Screen_Width div Screen_Width) - 1;

  if FHeight >= (Screen_Height div Tile_Height) then
    FHeight := (Screen_Height div Tile_Height) - 1;

  SetLength(FCells, Width, Height);

  FOffsetX := (Screen_Width - Width * Tile_Width) div 2;
  FOffsetY := (Screen_Height - Height * Tile_Height) div 2;

  for i := 0 to Width - 1 do
    for j := 0 to Height - 1 do
    begin
      FCells[i, j] := TGameCell.Create(self);
    end;
end;

{ TGameCell }

constructor TGameCell.Create(AArea: TGameArea);
begin
  inherited Create;
  FArea := AArea;
  FOrientation := random(4);
  FTileIndex := 3 - Trunc(sqrt(random(4*4)));
end;

destructor TGameCell.Destroy;
begin

  inherited;
end;

end.
