unit u_Particles;

interface

uses
  Classes, zglHeader;

const
  PARTICLE_KIND_EMITTER = 1;
  PARTICLE_KIND_PARTICLE = 2;

type
  TColorEx = record
    dR, dG, dB, dA : Single;
    R, G, B, A : Single;
  end;

  PParticle = ^TParticle;
  TParticle = record
    X, Y : Double;
    Dx, Dy : Integer;
    CenterX, CenterY : Double;
    Color : Cardinal;
    ColorEx : TColorEx;
    Color1 : Cardinal;
    Color2 : Cardinal;
    ColorChange : Double;
    XAcc, YAcc : Double;
    XAcc2, YAcc2 : Double;
    XVel, YVel : Double;
    XVel2, YVel2 : Double;
    Size : Integer;
    Size2 : Integer;
    LifeTime : Double;
    Texture : Integer;
    Kind : Byte;
    Sine : Boolean;
    SineAng, SineRad, SineRadInc : Double;
    Cosi : Boolean;
    CosiAng, CosiRad, CosiRadInc : Double;
    NewEmmiter : Double;
    EmitteParticle : PParticle;
    Magnet : Boolean;
    pps : Integer;
    toBorn : Single;
  end;

  PEmmiter = ^TEmmiter;
  TEmmiter = record
    Loop : boolean;
    EmitterLink : PParticle;
    IsLived : Boolean;
  end;

  TParticleEngine = class
  private
    FParticles : TList;
    FEmitters : TList;
    FEmmiterslist : array of TEmmiter;
    (*FPowerGraph : TPowerGraph;*)
    FSprites : zglPTexture(*TAGFImage*);
    function GetParticlesCount: Integer;
  protected

  public
    constructor Create((*PowerGraph : TPowerGraph; *)sprites : zglPTexture(*TAGFImage*));
    destructor Destroy;
    procedure Process(Delta : Double);
    procedure Render;
    procedure AddParticle(index : integer);
    procedure AddEmitter(params : TParticle);
    procedure DeleteDeadParticles;
    function LoadFromFile(FileName : String) : integer;
    function PlayEmitter(index : Integer; X, Y : Double; Looped : boolean = false) : Integer;
    procedure MoveEmitter(Index : integer; X, Y : Double);
    procedure KillEmitter(var Index : integer);
    function IsEmitterLooped(Emitter : PParticle) : Boolean;
    function GetFreeEmittersListItem : integer;
    procedure KillAllEmitters;
    procedure KillAllParticles;
    procedure KillAll;
  published
    property ParticlesCount : Integer read GetParticlesCount;
  end;

implementation

uses Math;

{ TParticleEngine }

procedure TParticleEngine.AddEmitter(params : TParticle);
var
  Emitter : PParticle;
begin
  New(Emitter);

  Emitter^ := params;
  Emitter^.CenterX := params.X;
  Emitter^.CenterY := params.Y;
  Emitter^.toBorn := 0;

  FParticles.Add(Emitter);
end;

procedure TParticleEngine.AddParticle(index: integer);
var
  Particle : PParticle;
  Emmiter : TParticle;
  TempX, TempY : Double;
  tR, tG, tB, tA : Single;
begin
  New(Particle);

  FillChar(Particle^, SizeOf(TParticle), 0);

  Emmiter := TParticle(FParticles.Items[index]^);

  Particle^ := Emmiter.EmitteParticle^;

  With Particle^ do
  begin
    TempX := Random(Emmiter.Dx)/100;
    TempY := Random(Emmiter.Dy)/100;
    X := Emmiter.X + TempX - Emmiter.Dx/200;
    Y := Emmiter.Y + TempY - Emmiter.Dy/200;
    CenterX := X;
    CenterY := Y;

    if Emmiter.Magnet then
    begin
      XVel := (Emmiter.X - X) / LifeTime / 60;
      YVel := (Emmiter.Y - Y) / LifeTime / 60;
      XAcc := 0;
      YAcc := 0;
    end else
    begin
      TempX := (random(round(XVel2 - XVel)*1000))/1000;
      XVel := XVel + TempX;
      TempY := (random(round(YVel2 - YVel)*1000))/1000;
      YVel := YVel + TempY;

      TempX := random(round((XAcc2 - XAcc)*1000))/1000;
      XAcc := XAcc + TempX;
      TempY := random(round((YAcc2 - YAcc)*1000))/1000;
      YAcc := YAcc + TempY;
    end;

    Size := Size + random(Size2 - Size);

    Color := Color1;

    with ColorEx do
    begin
      R := Color1 and $FF;
      G := (Color1 and $FF00) shr 8;
      B := (Color1 and $FF0000) shr 16;
      A := (Color1 and $FF000000) shr 24;

      tR := Color2 and $FF;
      tG := (Color2 and $FF00) shr 8;
      tB := (Color2 and $FF0000) shr 16;
      tA := (Color2 and $FF000000) shr 24;

      dR:= (tR - R) / ColorChange;
      dG:= (tG - G) / ColorChange;
      dB:= (tB - B) / ColorChange;
      dA:= (tA - A) / ColorChange;
    end;

    Kind := PARTICLE_KIND_PARTICLE;
  end;

  FParticles.Add(Particle);
end;

constructor TParticleEngine.Create((*PowerGraph : TPowerGraph;*) sprites : zglPTexture(*TAGFImage*));
var
  i : integer;
begin
  (*FPowerGraph := PowerGraph;*)
  FSprites := sprites;

  FParticles := TList.Create();
  FEmitters := TList.Create();

  SetLength(FEmmiterslist, 256);
  for i := 0 to 255 do
  FEmmiterslist[i].IsLived := false;
end;

procedure TParticleEngine.DeleteDeadParticles;
var
  i, j : Integer;
begin
  i := 0;
  repeat
    if TParticle(FParticles.Items[i]^).LifeTime <= 0 then
    begin
      if TParticle(FParticles.Items[i]^).Kind = PARTICLE_KIND_EMITTER then
      begin
        Dispose(TParticle(FParticles.Items[i]^).EmitteParticle);

        j := -1;
        repeat
          inc(j);
        until (FEmmiterslist[j].EmitterLink = PParticle(FParticles.Items[i])) or (j >= length(FEmmiterslist));

        if j < length(FEmmiterslist) then
        begin
          FEmmiterslist[j].IsLived := False;
          FEmmiterslist[j].EmitterLink.LifeTime := -1;
        end;
      end;

      Dispose(PParticle(FParticles.Items[i]));
      FParticles.Delete(i);
    end else
      inc(i);
  until i >= FParticles.Count
end;

destructor TParticleEngine.Destroy;
begin
  FEmitters.Free;
  FParticles.Free;

  SetLength(FEmmiterslist, 0);

  inherited;
end;

function TParticleEngine.GetFreeEmittersListItem: integer;
var
  i : integer;
  bingo : boolean;
begin
  i := 0;
  bingo := false;

  repeat
    if FEmmiterslist[i].IsLived then
      inc(i)
    else
      bingo := true;
  until bingo or (i >= length(FEmmiterslist));

  Result := i;

  if bingo then
  begin
    Exit;
  end else
  begin
    SetLength(FEmmiterslist, length(FEmmiterslist) + 1);
  end;
end;

function TParticleEngine.GetParticlesCount: Integer;
begin
  Result := FParticles.Count;
end;

function TParticleEngine.IsEmitterLooped(Emitter: PParticle): Boolean;
var
  i : integer;
begin
  i := -1;
  repeat
    inc(i);
  until FEmmiterslist[i].EmitterLink = Emitter;

  result := FEmmiterslist[i].Loop;
end;

procedure TParticleEngine.KillAll;
var
  i : Integer;
begin
  for i := 0 to FParticles.Count - 1 do
  with TParticle(FParticles.Items[i]^) do
  LifeTime := -1;
end;

procedure TParticleEngine.KillAllEmitters;
var
  i : Integer;
begin
  for i := 0 to FParticles.Count - 1 do
  with TParticle(FParticles.Items[i]^) do
  if Kind = PARTICLE_KIND_EMITTER then
  LifeTime := -1;
end;

procedure TParticleEngine.KillAllParticles;
var
  i : Integer;
begin
  for i := 0 to FParticles.Count - 1 do
  with TParticle(FParticles.Items[i]^) do
  if Kind = PARTICLE_KIND_PARTICLE then
  LifeTime := -1;
end;

procedure TParticleEngine.KillEmitter(var Index: integer);
begin
  if (index > length(FEmmiterslist)) or (index < 0) or (not FEmmiterslist[Index].IsLived) then exit;

  FEmmiterslist[Index].IsLived := False;
  FEmmiterslist[Index].EmitterLink.LifeTime := -1;
  Index := -1; 
end;

function TParticleEngine.LoadFromFile(FileName: String): integer;
var
  f : file;
  Emitter, EmittedParticle : PParticle;
begin
  AssignFile(f, FileName);
  Reset(f, 1);

  New(Emitter);
  New(EmittedParticle);

  BlockRead(f, Emitter^, Sizeof(TParticle));
  Emitter^.EmitteParticle := EmittedParticle;
  BlockRead(f, EmittedParticle^, Sizeof(TParticle));

  CloseFile(f);

  result := FEmitters.Add(Emitter);
end;

procedure TParticleEngine.MoveEmitter(Index : integer; X, Y: Double);
begin
  if (index > length(FEmmiterslist)) or (index < 0) or (not FEmmiterslist[Index].IsLived) then exit;

  FEmmiterslist[Index].EmitterLink.CenterX := X;
  FEmmiterslist[Index].EmitterLink.CenterY := Y;
  FEmmiterslist[Index].EmitterLink.X := X;
  FEmmiterslist[Index].EmitterLink.Y := Y;
end;

function TParticleEngine.PlayEmitter(index: Integer; X,
  Y: Double; Looped : boolean = false): Integer;
var
  Emitter : PParticle;
  Particle : PParticle;
begin
  New(Emitter);
  New(Particle);

  Emitter^ := TParticle(FEmitters.Items[index]^);
  Particle^ := TParticle(FEmitters.Items[index]^).EmitteParticle^;


  //Particle^.MaxLife := TParticle(FEmitters.Items[index]^).EmitteParticle^.LifeTime;


  Emitter^.EmitteParticle := Particle;
  Emitter^.X := X;
  Emitter^.Y := Y;
  Emitter^.CenterX := X;
  Emitter^.CenterY := Y;
  Emitter^.toBorn := 0;

  Result := GetFreeEmittersListItem;
  with FEmmiterslist[Result] do
  begin
    Loop := Looped;
    EmitterLink := Emitter;
    IsLived := true;
  end;

  FParticles.Add(Emitter);
end;

procedure TParticleEngine.Process(Delta: Double);
var
  i, j : integer;
begin
  if FParticles.Count = 0 then
  exit;

  DeleteDeadParticles;

  for i := 0 to FParticles.Count - 1 do
  with TParticle(FParticles.Items[i]^) do
  begin
    X := CenterX;
    Y := CenterY;

    LifeTime := LifeTime - Delta;
    X := X + XVel;
    Y := Y + YVel;
    XVel := XVel + XAcc;
    YVel := YVel + YAcc;

    CenterX := X;
    CenterY := Y;

    // Sine
    if Sine then
    begin
      X := X + SineRad * Sin(SineAng);
      SineAng := SineAng + pi/1.8*delta;
      SineRad := SineRad + SineRadInc;
    end;

    // Cosi
    if Cosi then
    begin
      Y := Y + CosiRad * Cos(CosiAng);
      CosiAng := CosiAng + pi/1.8*delta;
      CosiRad := CosiRad + CosiRadInc;
    end;

    //color
    if ColorChange > 0 then
    with ColorEx do
    begin
      R := R + dR * Delta;
      G := G + dG * Delta;
      B := B + dB * Delta;
      A := A + dA * Delta;;

      if R > 255 then R := 255;
      if G > 255 then G := 255;
      if B > 255 then B := 255;
      if A > 255 then A := 255;
      if R < 0 then R := 0;
      if G < 0 then G := 0;
      if B < 0 then B := 0;
      if A < 0 then A := 0;

      color := (trunc(R) + trunc(G) shl 8 + trunc(B) shl 16 + trunc(A) shl 24);

      ColorChange := ColorChange - Delta;
    end;

    // Emitter
    if kind = PARTICLE_KIND_EMITTER then
    begin
      if IsEmitterLooped(PParticle(FParticles.Items[i])) then
      LifeTime := 1;

      toBorn := toBorn + pps * Delta;

      if trunc(toBorn) >= 1 then
      begin
        for j := 0 to trunc(toBorn) do
        AddParticle(i);

        toBorn := toBorn - trunc(toBorn);
      end;

      NewEmmiter := NewEmmiter - Delta;
      if NewEmmiter <= 0 then
      begin
        NewEmmiter:= LifeTime + 1;
//        AddEmitter(TParticle(FParticles.Items[i]^));
      end;
    end;
  end;

end;

procedure TParticleEngine.Render;
var
  i : Integer;
  r, g, b : Byte;
begin
  if FParticles.Count = 0 then
  exit;

  //fx_SetBlendMode( FX_BLEND_ADD );
  for i := 0 to FParticles.Count - 1 do
  with TParticle(FParticles.Items[i]^) do
  begin
    if Kind = PARTICLE_KIND_PARTICLE then
    (*FPowerGraph.RotateEffect(FSprites, X, Y, 0, Size, Color, Texture, effectAdd or effectSrcAlpha or effectDiffuseAlpha);*)
    begin
    r :=   Color             shr 16;
    g := ( Color and $FF00 ) shr 8;
    b :=   Color and $FF;
    //fx2d_SetColor( ( b shl 16 ) + ( g shl 8 ) + r  );
    asprite2d_Draw( FSprites, X, Y, Size/10, Size/10, 0, Texture+1, Round(COlorEx.A) );
    end;
  end;
  //fx_SetBlendMode( FX_BLEND_NORMAL );

end;


end.
