#include "PNG.h"

#include <System/Log/Log.h>

#include <_external/pnglite/pnglite.h>

namespace Graphics
{
	Types::Property<const char *> PNG::Subsystem("Graphics/PNG", true);
};

Graphics::Texture * Graphics::PNG::Load(const char * Filename)
{
	png_t png;
	memset(&png, 0, sizeof(png));

	png_init(0, 0);
	int ErrorCode = png_open_file(&png, Filename);
	if (ErrorCode != PNG_NO_ERROR)
	{
		System::Log::Error(Subsystem, "Cannot load \"%s\" (error %d)", Filename, ErrorCode);
		return nullptr;
	}

	int Format = -1;
	switch(png.color_type)
	{
	case PNG_GREYSCALE:
		Format = GL_LUMINANCE;
		break;
	case PNG_TRUECOLOR:
		Format = GL_RGB;
		break;
	case PNG_GREYSCALE_ALPHA:
		Format = GL_LUMINANCE_ALPHA;
		break;
	case PNG_TRUECOLOR_ALPHA:
		Format = GL_RGBA;
		break;
	}

	if (Format >= 0)
	{
		Graphics::Texture * Tex = new Graphics::Texture();
		void * Data = malloc(png.width * png.height * png.bpp);

		png_get_data(&png, (unsigned char *)Data);

		Tex->Create(png.width, png.height, Format, Format, Data, GL_UNSIGNED_BYTE);

		free(Data);

		png_close_file(&png);

		System::Log::Message(Subsystem, "Texture \"%s\" was loaded", Filename);

		return Tex;
	} else {
		png_close_file(&png);

		System::Log::Error(Subsystem, "Cannot load \"%s\" (wrong color type)", Filename);

		return nullptr;
	}
}