#include "Texture.h"

namespace Graphics
{
	Types::Property<const char *> Texture::Subsystem("Graphics/Texture", true);
};

Graphics::Texture::Texture()
{
	TextureId = 0;
	TextureIdAssigned = false;

	Width = 0;
	Height = 0;
	Format = -1;
	StoreFormat = -1;
}

Graphics::Texture::~Texture()
{
	Delete();
}

int Graphics::Texture::GetWidth()
{
	return Width;
}

int Graphics::Texture::GetHeight()
{
	return Height;
}

int Graphics::Texture::GetFormat()
{
	return Format;
}

int Graphics::Texture::GetStoreFormat()
{
	return StoreFormat;
}

void Graphics::Texture::Create(int Width, int Height, int StoreFormat, int Format, void * Data, int DataType)
{
	Delete();

	this->Width = Width;
	this->Height = Height;
	this->Format = Format;
	this->StoreFormat = StoreFormat;

	glGenTextures(1, &TextureId);
	TextureIdAssigned = true;

	GLuint oldTexId;
	glGetIntegerv(GL_TEXTURE_BINDING_2D, (GLint *)&oldTexId);

	glBindTexture(GL_TEXTURE_2D, TextureId);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	gluBuild2DMipmaps(GL_TEXTURE_2D, StoreFormat, Width, Height, Format, DataType, Data);

	glBindTexture(GL_TEXTURE_2D, oldTexId);
}

void Graphics::Texture::Delete()
{
	if (TextureIdAssigned)
	{
		glDeleteTextures(1, &TextureId);
		TextureIdAssigned = false;
	}

	Width = 0;
	Height = 0;
	Format = -1;
	StoreFormat = -1;
}

void Graphics::Texture::Bind()
{
	if (!TextureIdAssigned) return;

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, TextureId);
}

void Graphics::Texture::Bind(int Index)
{
	if (!TextureIdAssigned) return;
	if (Index < 0 || Index > 31) return;

	glActiveTextureARB(GL_TEXTURE0_ARB + Index);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, TextureId);
}

void Graphics::Texture::Unbind()
{
	glDisable(GL_TEXTURE_2D);
}

void Graphics::Texture::Unbind(int Index)
{
	if (Index < 0 || Index > 31) return;

	glActiveTextureARB(GL_TEXTURE0_ARB + Index);
	glDisable(GL_TEXTURE_2D);
}

void Graphics::Texture::ChangeSlot(int Index)
{
	if (Index < 0 || Index > 31) return;

	glActiveTextureARB(GL_TEXTURE0_ARB + Index);
}

void Graphics::Texture::SetEnvMode(int EnvMode)
{
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, EnvMode);
}

void Graphics::Texture::SetCoord(float U, float V)
{
	glTexCoord2f(U, V);
}

void Graphics::Texture::SetCoord(int Index, float U, float V)
{
	if (Index < 0 || Index > 31) return;

	glMultiTexCoord2fARB(GL_TEXTURE0_ARB + Index, U, V);
}