#include "Info.h"

namespace System
{
	Types::Property<const char *> Info::Subsystem("System/Info", true);

	OSVERSIONINFOEX Info::OSVersion = {0};
	SYSTEM_INFO Info::SysInfo = {0};

	Info::PGPI Info::pGPI = nullptr;
	Info::LPFN_ISWOW64PROCESS Info::fnIsWow64Process = nullptr;

	char Info::VersionStr[1024] = {0};
	char Info::FeaturesStr[512] = {0};

	bool Info::Initialized = false;
};

bool System::Info::Initialize()
{
	if (Initialized) return true;

	memset(&OSVersion, 0, sizeof(OSVersion));
	OSVersion.dwOSVersionInfoSize = sizeof(OSVersion);

	memset(&SysInfo, 0, sizeof(SysInfo));

	PGNSI pGNSI = (PGNSI)GetProcAddress(GetModuleHandle(TEXT("kernel32.dll")), "GetNativeSystemInfo");
	if(pGNSI)
		pGNSI(&SysInfo);
	else
		GetSystemInfo(&SysInfo);

	if (!GetVersionEx((OSVERSIONINFO *)&OSVersion))
	{
		System::Log::Error(Subsystem, "Cannot retrieve OS version information");
		return false;
	}

	pGPI = (PGPI)GetProcAddress(GetModuleHandle(TEXT("kernel32.dll")), "GetProductInfo");
	fnIsWow64Process = (LPFN_ISWOW64PROCESS)GetProcAddress(GetModuleHandle(TEXT("kernel32")), "IsWow64Process");

	Initialized = true;
	return true;
}

const char * System::Info::GetVersionStr()
{
	if (!Initialized && !Initialize())
		return "";

	if (OSVersion.dwPlatformId == Platform::Win32NT && OSVersion.dwMajorVersion > 4)
	{
		sprintf_s(VersionStr, "%s", "Microsoft Windows ");
		if (OSVersion.dwMajorVersion == 6)
		{
			if (OSVersion.dwMinorVersion == 0)
			{
				if (OSVersion.wProductType == NTVersion::Workstation)
					sprintf_s(VersionStr, "%s%s", VersionStr, "Vista");
				else
					sprintf_s(VersionStr, "%s%s", VersionStr, "Server 2008");
			}

			if (OSVersion.dwMinorVersion == 1)
			{
				if (OSVersion.wProductType == NTVersion::Workstation)
					sprintf_s(VersionStr, "%s%s", VersionStr, "7");
				else
					sprintf_s(VersionStr, "%s%s", VersionStr, "Server 2008 R2");
			}

			if (pGPI)
			{
				DWORD dwType;

				pGPI(OSVersion.dwMajorVersion, OSVersion.dwMinorVersion, 0, 0, &dwType);

				switch(dwType)
				{
				case ProductType::Ultimate:
					sprintf_s(VersionStr, "%s%s", VersionStr, " Ultimate Edition");
					break;
				case ProductType::Professional:
					sprintf_s(VersionStr, "%s%s", VersionStr, " Professional");
					break;
				case ProductType::HomePremium:
					sprintf_s(VersionStr, "%s%s", VersionStr, " Home Premium Edition");
					break;
				case ProductType::HomeBasic:
					sprintf_s(VersionStr, "%s%s", VersionStr, " Home Basic Edition");
					break;
				case ProductType::Enterprise:
					sprintf_s(VersionStr, "%s%s", VersionStr, " Enterprise Edition");
					break;
				case ProductType::Business:
					sprintf_s(VersionStr, "%s%s", VersionStr, " Business Edition");
					break;
				case ProductType::Starter:
					sprintf_s(VersionStr, "%s%s", VersionStr, " Starter Edition");
					break;
				case ProductType::ClusterServer:
					sprintf_s(VersionStr, "%s%s", VersionStr, " HPC Server Edition");
					break;
				case ProductType::DataCenterServer:
					sprintf_s(VersionStr, "%s%s", VersionStr, " Datacenter Edition");
					break;
				case ProductType::DataCenterServerCore:
					sprintf_s(VersionStr, "%s%s", VersionStr, " Datacenter Edition (core installation)");
					break;
				case ProductType::EnterpriseServer:
					sprintf_s(VersionStr, "%s%s", VersionStr, " Enterprise Edition");
					break;
				case ProductType::EnterpriseServerCore:
					sprintf_s(VersionStr, "%s%s", VersionStr, " Enterprise Edition (core installation)");
					break;
				case ProductType::EnterpriseServerIA64:
					sprintf_s(VersionStr, "%s%s", VersionStr, " Enterprise Edition for Itanium-based Systems");
					break;
				case ProductType::SmallBusinessServer:
					sprintf_s(VersionStr, "%s%s", VersionStr, " Small Business Server");
					break;
				case ProductType::SmallBusinessServerPremium:
					sprintf_s(VersionStr, "%s%s", VersionStr, " Small Business Server Premium Edition");
					break;
				case ProductType::StandartServer:
					sprintf_s(VersionStr, "%s%s", VersionStr, " Standard Edition");
					break;
				case ProductType::StandartServerCore:
					sprintf_s(VersionStr, "%s%s", VersionStr, " Standard Edition (core installation)");
					break;
				case ProductType::WebServer:
					sprintf_s(VersionStr, "%s%s", VersionStr, " Web Server Edition");
					break;
				default:
					sprintf_s(VersionStr, "%s%s%d%s", VersionStr, " (version code ", dwType, ")");
					break;
				}
			}
		}

		else if (OSVersion.dwMajorVersion == 5 && OSVersion.dwMinorVersion == 2)
		{
			if(GetSystemMetrics(SM_SERVERR2))
				sprintf_s(VersionStr, "%s%s", VersionStr, "Server 2003 R2");
			else if (OSVersion.wSuiteMask & Suite::StorageServer)
				sprintf_s(VersionStr, "%s%s", VersionStr, "Storage Server 2003");
			else if (OSVersion.wSuiteMask & Suite::WHServer)
				sprintf_s(VersionStr, "%s%s", VersionStr, "Home Server");
			else if(OSVersion.wProductType == NTVersion::Workstation && SysInfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_AMD64)
				sprintf_s(VersionStr, "%s%s", VersionStr, "XP Professional x64 Edition");
			else
				sprintf_s(VersionStr, "%s%s", VersionStr, "Server 2003");

			if (OSVersion.wProductType != NTVersion::Workstation)
			{
				if (SysInfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_IA64)
				{
					if(OSVersion.wSuiteMask & Suite::DataCenter)
						sprintf_s(VersionStr, "%s%s", VersionStr, " Datacenter Edition for Itanium-based Systems");
					else if(OSVersion.wSuiteMask & Suite::Enterprise)
						sprintf_s(VersionStr, "%s%s", VersionStr, " Enterprise Edition for Itanium-based Systems");
				}
				else if (SysInfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_AMD64)
				{
					if(OSVersion.wSuiteMask & Suite::DataCenter)
					   sprintf_s(VersionStr, "%s%s", VersionStr, " Datacenter x64 Edition");
					else if(OSVersion.wSuiteMask & Suite::Enterprise)
					   sprintf_s(VersionStr, "%s%s", VersionStr, " Enterprise x64 Edition");
					else sprintf_s(VersionStr, "%s%s", VersionStr, " Standard x64 Edition");
				}
				else
				{
					if (OSVersion.wSuiteMask & Suite::ComputeServer)
					   sprintf_s(VersionStr, "%s%s", VersionStr, " Compute Cluster Edition");
					else if(OSVersion.wSuiteMask & Suite::DataCenter)
					   sprintf_s(VersionStr, "%s%s", VersionStr, " Datacenter Edition");
					else if(OSVersion.wSuiteMask & Suite::Enterprise)
					   sprintf_s(VersionStr, "%s%s", VersionStr, " Enterprise Edition");
					else if (OSVersion.wSuiteMask & Suite::Blade)
					   sprintf_s(VersionStr, "%s%s", VersionStr, " Web Edition");
					else sprintf_s(VersionStr, "%s%s", VersionStr, " Standard Edition");
				}
			}
		}

		else if (OSVersion.dwMajorVersion == 5 && OSVersion.dwMinorVersion == 1)
		{
			sprintf_s(VersionStr, "%s%s", VersionStr, "XP");
			if (OSVersion.wSuiteMask & Suite::Personal)
				sprintf_s(VersionStr, "%s%s", VersionStr, " Home Edition");
			else
				sprintf_s(VersionStr, "%s%s", VersionStr, " Professional");

			if(GetSystemMetrics(SM_MEDIACENTER))
				sprintf_s(VersionStr, "%s%s", VersionStr, " Media Center Edition");
			else if(GetSystemMetrics(SM_STARTER))
				sprintf_s(VersionStr, "%s%s", VersionStr, " Starter Edition");
			else if(GetSystemMetrics(SM_TABLETPC))
				sprintf_s(VersionStr, "%s%s", VersionStr, " Tablet PC Edition");
		}

		else if (OSVersion.dwMajorVersion == 5 && OSVersion.dwMinorVersion == 0)
		{
			sprintf_s(VersionStr, "%s%s", VersionStr, "2000");
			if (OSVersion.wProductType == NTVersion::Workstation)
				sprintf_s(VersionStr, "%s%s", VersionStr, " Professional");
			else
			{
				if(OSVersion.wSuiteMask & Suite::DataCenter)
					sprintf_s(VersionStr, "%s%s", VersionStr, " Datacenter Server");
				else if(OSVersion.wSuiteMask & Suite::Enterprise)
					sprintf_s(VersionStr, "%s%s", VersionStr, " Advanced Server");
				else sprintf_s(VersionStr, "%s%s", VersionStr, " Server");
			}
		}

		if(strlen(OSVersion.szCSDVersion) > 0)
		{
			sprintf_s(VersionStr, "%s%s%s", VersionStr, " ", OSVersion.szCSDVersion);
		}

		sprintf_s(VersionStr, "%s%s%d", VersionStr, " build ", OSVersion.dwBuildNumber);

		if (SysInfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_AMD64)
			sprintf_s(VersionStr, "%s%s", VersionStr, " 64-bit");
		else if (SysInfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_INTEL)
			sprintf_s(VersionStr, "%s%s", VersionStr, " 32-bit");
	}
	else if (OSVersion.dwPlatformId == VER_PLATFORM_WIN32_NT)
	{
		if (OSVersion.dwMajorVersion == 4 && OSVersion.dwMinorVersion == 0)
		{
			sprintf_s(VersionStr, "%s", "Microsoft Windows NT4");
		}
		if (OSVersion.dwMajorVersion == 3 && OSVersion.dwMinorVersion == 51)
		{
			sprintf_s(VersionStr, "%s", "Microsoft Windows NT 3.51");
		}
	}
	else if (OSVersion.dwPlatformId == Platform::Win32Windows && OSVersion.dwMajorVersion == 4)
	{
		if (OSVersion.dwMinorVersion == 90)
		{
			sprintf_s(VersionStr, "%s", "Microsoft Windows Millennium Edition");
		}
		if (OSVersion.dwMinorVersion == 10 && OSVersion.dwBuildNumber >= 2183)
		{
			sprintf_s(VersionStr, "%s", "Microsoft Windows 98 Second Edition");
		}
		if (OSVersion.dwMinorVersion == 10)
		{
			sprintf_s(VersionStr, "%s", "Microsoft Windows 98");
		}
		if (OSVersion.dwMinorVersion < 10 && OSVersion.dwBuildNumber > 1080)
		{
			sprintf_s(VersionStr, "%s", "Microsoft Windows 95 OEM Service Release 2");
		}
		if (OSVersion.dwMajorVersion == 4 && OSVersion.dwMinorVersion == 0)
		{
			sprintf_s(VersionStr, "%s", "Microsoft Windows 95");
		}
	}
	else if (OSVersion.dwPlatformId == Platform::Win32CE)
	{
		sprintf_s(VersionStr, "%s%d%s%d", "Microsoft Windows CE ", OSVersion.dwMajorVersion, ".", OSVersion.dwMinorVersion);
	}
	else
	{
		sprintf_s(VersionStr, "%s", "unknown OS");
	}

	return VersionStr;
}

const char * System::Info::GetProcessorFeaturesStr()
{
	if (!Initialized && !Initialize())
		return "";

	if (IsProcessorFeaturePresent(PF_3DNOW_INSTRUCTIONS_AVAILABLE)) sprintf_s(FeaturesStr, "%s%s ", FeaturesStr, "3D-Now");
	if (IsProcessorFeaturePresent(PF_MMX_INSTRUCTIONS_AVAILABLE)) sprintf_s(FeaturesStr, "%s%s ", FeaturesStr, "MMX");
	if (IsProcessorFeaturePresent(PF_NX_ENABLED)) sprintf_s(FeaturesStr, "%s%s ", FeaturesStr, "DEP");
	if (IsProcessorFeaturePresent(PF_PAE_ENABLED)) sprintf_s(FeaturesStr, "%s%s ", FeaturesStr, "PAE");
	if (IsProcessorFeaturePresent(PF_XMMI_INSTRUCTIONS_AVAILABLE)) sprintf_s(FeaturesStr, "%s%s ", FeaturesStr, "SSE");
	if (IsProcessorFeaturePresent(PF_XMMI64_INSTRUCTIONS_AVAILABLE)) sprintf_s(FeaturesStr, "%s%s ", FeaturesStr, "SSE2");
	if (IsProcessorFeaturePresent(PF_SSE3_INSTRUCTIONS_AVAILABLE)) sprintf_s(FeaturesStr, "%s%s ", FeaturesStr, "SSE3");

	return FeaturesStr;
}

bool System::Info::IsWow64()
{
	if (!Initialized && !Initialize())
		return false;

	if (fnIsWow64Process)
	{
		BOOL bIsWow64 = FALSE;
		if (fnIsWow64Process(GetCurrentProcess(), &bIsWow64))
		{
			return bIsWow64 == TRUE;
		}
	}

	return false;
}

void System::Info::DumpSystemInfo()
{	
	if (!Initialized && !Initialize())
		return;

	System::Log::Message(Subsystem, "OS Version: %s", GetVersionStr());
	System::Log::Message(Subsystem, "Processors count: %d", SysInfo.dwNumberOfProcessors);
	System::Log::Message(Subsystem, "Processors features: %s", GetProcessorFeaturesStr());
	System::Log::Message(Subsystem, "Running in %s mode", IsWow64() ? "WOW64" : "native");
}