#pragma once

#include <windows.h>

#include <System/Log/Log.h>
#include <Types/Property/Property.h>

namespace System
{
	class Info
	{
	private:
		static Types::Property<const char *> Subsystem;

		static OSVERSIONINFOEX OSVersion;
		static SYSTEM_INFO SysInfo;

		typedef BOOL (WINAPI *LPFN_ISWOW64PROCESS) (HANDLE, PBOOL);
		typedef void (WINAPI *PGNSI)(LPSYSTEM_INFO);
		typedef BOOL (WINAPI *PGPI)(DWORD, DWORD, DWORD, DWORD, PDWORD);

		static PGPI pGPI;
		static LPFN_ISWOW64PROCESS fnIsWow64Process;

		static char VersionStr[1024];
		static char FeaturesStr[512];

		static bool Initialized;

		static bool Initialize();
	public:
		class NTVersion
		{
		public:
			enum
			{
				Workstation	= 0x0000001,
				Server		= 0x0000003,
			};
		};

		class Suite
		{
		public:
			enum
			{
				SmallBusiness			= 0x00000001,
				Enterprise				= 0x00000002,
				BackOffice				= 0x00000004,
				Communications			= 0x00000008,
				Terminal				= 0x00000010,
				SmallBusinessRestricted	= 0x00000020,
				EmbeddedNT				= 0x00000040,
				DataCenter				= 0x00000080,
				SingleUserTS			= 0x00000100,
				Personal				= 0x00000200,
				Blade					= 0x00000400,
				EmbeddedRestricted		= 0x00000800,
				SecurityAppliance		= 0x00001000,
				StorageServer			= 0x00002000,
				ComputeServer			= 0x00004000,
				WHServer				= 0x00008000,
			};
		};

		class ProductType
		{
		public:
			enum
			{
				Undefined						= 0x00000000,
				Ultimate						= 0x00000001,
				HomeBasic						= 0x00000002,
				HomePremium						= 0x00000003,
				Enterprise						= 0x00000004,
				HomeBasicN						= 0x00000005,
				Business						= 0x00000006,
				StandartServer					= 0x00000007,
				DataCenterServer				= 0x00000008,
				SmallBusinessServer				= 0x00000009,
				EnterpriseServer				= 0x0000000A,
				Starter							= 0x0000000B,
				DataCenterServerCore			= 0x0000000C,
				StandartServerCore				= 0x0000000D,
				EnterpriseServerCore			= 0x0000000E,
				EnterpriseServerIA64			= 0x0000000F,
				BusinessN						= 0x00000010,
				WebServer						= 0x00000011,
				ClusterServer					= 0x00000012,
				HomeServer						= 0x00000013,
				StorageExpressServer			= 0x00000014,		
				StorageStandartServer			= 0x00000015,
				StorageWorkgroupServer			= 0x00000016,
				StorageEnterpriseServer			= 0x00000017,
				ServerForSmallBusiness			= 0x00000018,
				SmallBusinessServerPremium		= 0x00000019,
				HomePremiumN					= 0x0000001A,
				EnterpriseN						= 0x0000001B,
				UltimateN						= 0x0000001C,
				WebServerCore					= 0x0000001D,
				MediumBusinessServeManagement	= 0x0000001E,
				MediumBusinessServerSecurity	= 0x0000001F,
				MediumBusinessServerMessaging	= 0x00000020,
				ServerFoundation				= 0x00000021,
				HomePremiumServer				= 0x00000022,
				ServerForSmallBusinessV			= 0x00000023,
				StandartServerV					= 0x00000024,
				DataCenterServerV				= 0x00000025,
				EnterpriseServerV				= 0x00000026,
				DataCenterServerCoreV			= 0x00000027,
				StandartServerCoreV				= 0x00000028,
				EnterpriseServerCoreV			= 0x00000029,
				HyperV							= 0x0000002A,
				StorageExpressServerCore		= 0x0000002B,
				StorageStandartServerCore		= 0x0000002C,
				StorageWorkgroupServerCore		= 0x0000002D,
				StorageEnterpriseServerCore		= 0x0000002E,
				StarterN						= 0x0000002F,
				Professional					= 0x00000030,
				ProfessionalN					= 0x00000031,
				SBSolutionServer				= 0x00000032,
				ServerForSBSolutions			= 0x00000033,
				StandartServerSolutions			= 0x00000034,
				StandartServerSolutionsCore		= 0x00000035,
				SBSolutionServerEM				= 0x00000036,
				ServerForSBSolutionsEM			= 0x00000037,
				SolutionEmbeddedServer			= 0x00000038,
				SolutionEmbeddedServerCore		= 0x00000039,
				SmallBusinessServerPremiumCore	= 0x0000003F,
				EssentialBusinessServerMGMT		= 0x0000003B,
				EssentialBusinessServerADDL		= 0x0000003C,
				EssentialBusinessServerMGMTSVC	= 0x0000003D,
				EssentialBusinessServerADDLSVC	= 0x0000003E,
				ClusterServerV					= 0x00000040,
				Embedded						= 0x00000041,
				StarterE						= 0x00000042,
				HomeBasicE						= 0x00000043,
				HomePremiumE					= 0x00000044,
				ProfessionalE					= 0x00000045,
				EnterpriseE						= 0x00000046,
				UltimateE						= 0x00000047,
			};
		};

		class Edition
		{
		public:
			enum
			{
				TabletPC	= 0x00000086,
				MediaCenter	= 0x00000087,
				Starter		= 0x00000088,
				ServerR2	= 0x00000089,
			};
		};

		class Platform
		{
		public:
			enum
			{
				Win32			= 0,
				Win32Windows	= 1,
				Win32NT			= 2,
				Win32CE			= 3,
			};
		};

		Info();

		static const char * GetVersionStr();
		static const char * GetProcessorFeaturesStr();
		static bool IsWow64();
		static void DumpSystemInfo();
	};
};