#include "CriticalSection.h"

#include <System/Log/Log.h>

namespace System
{
	namespace Threads
	{
		Types::Property<const char *> CriticalSection::Subsystem("System/CriticalSection", true);
	};
};

System::Threads::CriticalSection::CriticalSection()
{
	InitializeCriticalSection(&Section);
}

System::Threads::CriticalSection::CriticalSection(DWORD SpinCount)
{
	if (!InitializeCriticalSectionAndSpinCount(&Section, SpinCount))
	{
		System::Log::Warning(Subsystem, "Cannot initialize critical section with spin count");
		InitializeCriticalSection(&Section);
	}
}

System::Threads::CriticalSection::~CriticalSection()
{
	DeleteCriticalSection(&Section);
}

void System::Threads::CriticalSection::Enter()
{
	EnterCriticalSection(&Section);
}

void System::Threads::CriticalSection::Leave()
{
	LeaveCriticalSection(&Section);
}

bool System::Threads::CriticalSection::SetSpinCount(DWORD SpinCount)
{
	return SetCriticalSectionSpinCount(&Section, SpinCount) == TRUE;
}