#include "Thread.h"

namespace System
{
	namespace Threads
	{
		Types::Property<const char *> Thread::Subsystem("System/Threads/Thread", true);
	};
};

DWORD WINAPI System::Threads::Thread::System_Threads_Thread_ThreadProc(LPVOID lpThreadParameter)
{
	System_Threads_Thread_ThreadProc_Data * Data = (System_Threads_Thread_ThreadProc_Data *)lpThreadParameter;
	DWORD ReturnCode = 0;
	__try
	{
		if (Data != nullptr && Data->ThreadProc != nullptr)
		{
			ReturnCode = (*Data->ThreadProc)(Data->Data);
		}
	}
	__except(EXCEPTION_EXECUTE_HANDLER)
	{
		System::Log::Error(Subsystem, "Error executing thread");
	}
	return ReturnCode;
}

System::Threads::Thread::Thread(System::Threads::Thread::ThreadProc & Proc, void * Data)
{
	RealThreadProcData.ThreadProc = &Proc;
	RealThreadProcData.Data = Data;

	if (::CreateThread(nullptr, 0, System_Threads_Thread_ThreadProc, &RealThreadProcData, 0, &Handle) == INVALID_HANDLE_VALUE)
	{
		System::Log::Error(Subsystem, "Cannot create thread (error code %d)", GetLastError());
	}
}

System::Threads::Thread::~Thread()
{
	TerminateThread((HANDLE)Handle, 0);
}