#pragma once

namespace Types
{
	template <typename T> class Vector2
	{
	public:
		union { T X, U; };
		union { T Y, V; };

		Vector2()
		{
			X = 0;
			Y = 0;
		}

		Vector2(T X, T Y)
		{
			this->X = X;
			this->Y = Y;
		}

		Vector2<T> operator -() const
		{
			return Vector2<T>(-X, -Y);
		}

		Vector2<T> operator -(const Vector2<T> & V) const
		{
			return Vector2<T>(X - V.X, Y - V.Y);
		}

		Vector2<T> operator +(const Vector2<T> & V) const
		{
			return Vector2<T>(X + V.X, Y + V.Y);
		}

		Vector2<T> operator *(const T V) const
		{
			return Vector2<T>(X * V, Y * V);
		}

		Vector2<T> operator /(const T V) const
		{
			return Vector2<T>(X / V, Y / V);
		}

		const Vector2<T> & operator =(const Vector2<T> & V)
		{
			X = V.X;
			Y = V.Y;
			return *this;
		}

		const Vector2<T> & operator +=(const Vector2<T> & V)
		{
			X += V.X;
			Y += V.Y;
			return *this;
		}

		const Vector2<T> & operator -=(const Vector2<T> & V)
		{
			X -= V.X;
			Y -= V.Y;
			return *this;
		}

		const Vector2<T> & operator *=(const T V)
		{
			X *= V;
			Y *= V;
			return *this;
		}

		const Vector2<T> & operator /=(const T V)
		{
			X /= V;
			Y /= V;
			return *this;
		}

		bool operator ==(const Vector2<T> & V) const
		{
			return X == V.X && Y == V.Y;
		}

		float Length()
		{
			return sqrtf(X * X + Y * Y);
		}

		T Distance(T & V)
		{
			return (*this - V).Length();
		}

		const Vector2<T> & Normalize()
		{
			float Length = this->Length();
			return Length == 0 ? Vector2<T>(0, 0) : (*this / Length);
		}

		// angle in radians
		float Angle()
		{
			if (X == 0 && Y == 0)
				return 0.0f;
			float a = atanf(Y / X);
			if (X < 0)
				a += Consts::Pi;
			return a;
		}

		const Vector2<T> & Rotate(float Angle)
		{
			if (X == 0 && Y == 0)
				return Vector2<T>(0.0f, 0.0f);
			return Vector2<T>(X * cosf(Angle) - Y * sinf(Angle), Y * cosf(Angle) + X * sinf(Angle));
		}
	};

	typedef Vector2<float> Vector2f;
	typedef Vector2<int> Vector2i;
};