#pragma once

namespace Types
{
	template <typename T> class Vector3
	{
	public:
		union { T X, U, R; };
		union { T Y, V, G; };
		union { T Z, W, B; };

		Vector3()
		{
			X = 0;
			Y = 0;
			Z = 0;
		}

		Vector3(T X, T Y, T Z)
		{
			this->X = X;
			this->Y = Y;
			this->Z = Z;
		}

		Vector3<T> operator -() const
		{
			return Vector3<T>(-X, -Y, -Z);
		}

		Vector3<T> operator -(const Vector3<T> & V) const
		{
			return Vector3<T>(X - V.X, Y - V.Y, Z - V.Z);
		}

		Vector3<T> operator +(const Vector3<T> & V) const
		{
			return Vector3<T>(X + V.X, Y + V.Y, Z + V.Z);
		}

		Vector3<T> operator *(const T V) const
		{
			return Vector3<T>(X * V, Y * V, Z * V);
		}

		Vector3<T> operator /(const T V) const
		{
			return Vector3<T>(X / V, Y / V, Z / V);
		}

		const Vector3<T> & operator =(const Vector3<T> & V)
		{
			X = V.X;
			Y = V.Y;
			Z = V.Z;
			return *this;
		}

		const Vector3<T> & operator +=(const Vector3<T> & V)
		{
			X += V.X;
			Y += V.Y;
			Z += V.Z;
			return *this;
		}

		const Vector3<T> & operator -=(const Vector3<T> & V)
		{
			X -= V.X;
			Y -= V.Y;
			Z -= V.Z;
			return *this;
		}

		const Vector3<T> & operator *=(const T V)
		{
			X *= V;
			Y *= V;
			Z *= V;
			return *this;
		}

		const Vector3<T> & operator /=(const T V)
		{
			X /= V;
			Y /= V;
			Z /= V;
			return *this;
		}

		bool operator ==(const Vector3<T> & V) const
		{
			return X == V.X && Y == V.Y && Z == V.Z;
		}

		float Length()
		{
			return sqrtf(X * X + Y * Y + Z * Z);
		}

		T Distance(T & V)
		{
			return (*this - V).Length();
		}

		const Vector3<T> & Normalize()
		{
			float Length = this->Length();
			return Length == 0 ? Vector3<T>(0, 0, 0) : (*this / Length);
		}
	};

	typedef Vector3<float> Vector3f;
	typedef Vector3<int> Vector3i;
};