#pragma once

namespace Types
{
	template <typename T> class Vector4
	{
	public:
		union { T R; };
		union { T G; };
		union { T B; };
		union { T A; };

		Vector4()
		{
			R = 0;
			G = 0;
			B = 0;
			A = 0;
		}

		Vector4(T R, T G, T B, T A)
		{
			this->R = R;
			this->G = G;
			this->B = B;
			this->A = A;
		}

		Vector4<T> operator -() const
		{
			return Vector4<T>(-R, -G, -B, -A);
		}

		Vector4<T> operator -(const Vector4<T> & V) const
		{
			return Vector4<T>(R - V.R, G - V.G, B - V.B, A - V.A);
		}

		Vector4<T> operator +(const Vector4<T> & V) const
		{
			return Vector4<T>(R + V.R, G + V.G, B + V.B, A + V.A);
		}

		Vector4<T> operator *(const T V) const
		{
			return Vector4<T>(R * V, G * V, B * V, A * V);
		}

		Vector4<T> operator /(const T V) const
		{
			return Vector4<T>(R / V, G / V, B / V, A / V);
		}

		const Vector4<T> & operator =(const Vector4<T> & V)
		{
			R = V.R;
			G = V.G;
			B = V.B;
			A = V.A;
			return *this;
		}

		const Vector4<T> & operator +=(const Vector4<T> & V)
		{
			R += V.R;
			G += V.G;
			B += V.B;
			A += V.A;
			return *this;
		}

		const Vector4<T> & operator -=(const Vector4<T> & V)
		{
			R -= V.R;
			G -= V.G;
			B -= V.B;
			A -= V.A;
			return *this;
		}

		const Vector4<T> & operator *=(const T V)
		{
			R *= V;
			G *= V;
			B *= V;
			A *= V;
			return *this;
		}

		const Vector4<T> & operator /=(const T V)
		{
			R /= V;
			G /= V;
			B /= V;
			A /= V;
			return *this;
		}

		bool operator ==(const Vector4<T> & V) const
		{
			return R == V.R && G == V.G && B == V.B && A == V.A;
		}

		float Length()
		{
			return sqrtf(R * R + G * G + B * B + A * A);
		}

		T Distance(T & V)
		{
			return (*this - V).Length();
		}

		const Vector4<T> & Normalize()
		{
			float Length = this->Length();
			return Length == 0 ? Vector4<T>(0, 0, 0, 0) : (*this / Length);
		}
	};

	typedef Vector4<float> Vector4f;
	typedef Vector4<int> Vector4i;
};