#include "Block.h"

Game::Block::Block()
{
	Points.clear();
	Pos = Vector2f(0.0f, 0.0f);
	Color = Vector4f(1.0f, 1.0f, 1.0f, 1.0f);
	BorderWidth = 1.0f;

	Fill = true;
	Loop = true;
	Fixed = true;

	Type = 1;

	Mass = 10.0f;
}

Game::Block::~Block()
{
	::World->pWorld->DestroyBody(pBody);

	Points.clear();
	DisplayPoints.clear();
}

void Game::Block::InitPhysics()
{
	if (Points.size() == 0)
		return;

	if (Type != 0)
	{
		bool CopyPoints = DisplayPoints.size() == 0;

		pBodyDef.type = Fixed ? b2_staticBody : b2_dynamicBody;
		pBodyDef.allowSleep = Fixed ? true : false;
		pBodyDef.position.Set(Pos.X, Pos.Y);

		pBody = ::World->pWorld->CreateBody(&pBodyDef);
		pBody->SetUserData(this);

		for (int i = 0; i < Points.size(); i++)
		{
			b2PolygonShape * pShape = new b2PolygonShape();
			pShape->SetAsEdge(b2Vec2(Points[i].X, Points[i].Y),
					b2Vec2(Points[(i + 1) % Points.size()].X, Points[(i + 1) % Points.size()].Y));

			if (Fixed)
			{
				pFixtureDef.shape = pShape;
				pFixtureDef.density = 0.0f;
				pFixtureDef.friction = 0.3f;
				pFixtureDef.restitution = 0.5f;

				pBody->CreateFixture(&pFixtureDef);
			} else {
				pFixtureDef.shape = pShape;
				pFixtureDef.density = 100.0f;
				pFixtureDef.friction = 0.3f;
				pFixtureDef.restitution = 0.5f;

				pBody->CreateFixture(&pFixtureDef);
			}

			if (CopyPoints)
			{
				DisplayPoints.push_back(Points[i]);
			}
		}

		/*b2Vec2 * pVertices = new b2Vec2[Points.size()];
		for (int i = 0; i < Points.size(); i++)
		{
			pVertices[i].Set(Points[i].X, Points[i].Y);
		}

		pShape.Set(pVertices, Points.size());

		delete pVertices;

		if (Fixed)
		{
			pFixtureDef.shape = &pShape;
			pFixtureDef.density = 0.0f;
			pFixtureDef.friction = 0.3f;
			pFixtureDef.restitution = 0.5f;

			pBody->CreateFixture(&pFixtureDef);
		} else {
			pFixtureDef.shape = &pShape;
			pFixtureDef.density = 100.0f;
			pFixtureDef.friction = 0.3f;
			pFixtureDef.restitution = 0.5f;

			pBody->CreateFixture(&pFixtureDef);
		}*/

		b2MassData pMassData;
		pMassData.center = b2Vec2(0, 0);
		pMassData.I = 1.0f;
		pMassData.mass = Mass;

		pBody->SetMassData(&pMassData);
	}
}

void Game::Block::Init()
{
	InitPhysics();
}

void Game::Block::Process(float DeltaTime)
{
	if (Type != 0)
	{
		b2Vec2 NewPos = pBody->GetPosition();
		Pos = Vector2f(NewPos.x, NewPos.y);

		Points.clear();
		for (int i = 0; i < pShape.GetVertexCount(); i++)
		{
			b2Vec2 Point = pShape.GetVertex(i);
			Points.push_back(Vector2f(Point.x, Point.y));
		}
	}
}

void Game::Block::Render(float Zoom)
{
	if (DisplayPoints.size() == 0 || Color.A == 0.0f)
		return;

	glPushMatrix();
	glTranslatef(Pos.X * Zoom, Pos.Y * Zoom, 0);
	glRotatef(pBody->GetAngle() / Consts::Pi * 180.0f, 0.0f, 0.0f, 1.0f);

	if (Fill)
	{
		glColor4f(Color.R, Color.G, Color.B, Color.A);
		glBegin(GL_POLYGON);
			for (int i = 0; i < DisplayPoints.size(); i++)
			{
				glVertex2f(DisplayPoints[i].X * Zoom, DisplayPoints[i].Y * Zoom);
			}
		glEnd();
	}

	glLineWidth(BorderWidth * Zoom);
	glColor4f(0.0f, 0.0f, 0.0f, Color.A);
	glBegin(Loop ? GL_LINE_LOOP : GL_LINE_STRIP);
		for (int i = 0; i < DisplayPoints.size(); i++)
		{
			glVertex2f(DisplayPoints[i].X * Zoom, DisplayPoints[i].Y * Zoom);
		}
	glEnd();

	glPopMatrix();
}

Game::BlockEllipsoid::BlockEllipsoid()
{
	AngleStart = 0;
	AngleEnd = Consts::Pi * 2.0f;
	Radius1 = 0;
	Radius2 = 0;
	qX = 1.0f;
	qY = 1.0f;

	PointsCount = 24;
	DisplayPointsCount = 0;
}

Game::BlockEllipsoid::~BlockEllipsoid()
{
	Points.clear();
	DisplayPoints.clear();
}

void Game::BlockEllipsoid::InitPhysics()
{
	if (Points.size() == 0)
		return;

	if (Type != 0)
	{
		pBodyDef.type = Fixed ? b2_staticBody : b2_dynamicBody;
		pBodyDef.allowSleep = Fixed ? true : false;
		pBodyDef.position.Set(Pos.X, Pos.Y);

		pBody = ::World->pWorld->CreateBody(&pBodyDef);
		pBody->SetUserData(this);

		if (Radius2 == 0 && (Radius1 < Consts::Pi || Radius1 == Consts::Pi * 2.0f))
		{
			b2Vec2 * pVertices = new b2Vec2[Points.size()];
			for (int i = 0; i < Points.size(); i++)
			{
				pVertices[i].Set(Points[i].X, Points[i].Y);
			}

			pShape.Set(pVertices, Points.size());

			delete pVertices;

			if (Fixed)
			{
				pFixtureDef.shape = &pShape;
				pFixtureDef.density = 0.0f;
				pFixtureDef.friction = 0.3f;
				pFixtureDef.restitution = 0.5f;

				pBody->CreateFixture(&pFixtureDef);
			} else {
				pFixtureDef.shape = &pShape;
				pFixtureDef.density = 100.0f;
				pFixtureDef.friction = 0.3f;
				pFixtureDef.restitution = 0.5f;

				pBody->CreateFixture(&pFixtureDef);
			}
		} else {
			for (int i = 0; i < Points.size(); i++)
			{
				b2PolygonShape * pShape = new b2PolygonShape();
				pShape->SetAsEdge(b2Vec2(Points[i].X, Points[i].Y),
					b2Vec2(Points[(i + 1) % Points.size()].X, Points[(i + 1) % Points.size()].Y));

				if (Fixed)
				{
					pFixtureDef.shape = pShape;
					pFixtureDef.density = 0.0f;
					pFixtureDef.friction = 0.3f;
					pFixtureDef.restitution = 0.5f;

					pBody->CreateFixture(&pFixtureDef);
				} else {
					pFixtureDef.shape = pShape;
					pFixtureDef.density = 100.0f;
					pFixtureDef.friction = 0.3f;
					pFixtureDef.restitution = 0.5f;

					pBody->CreateFixture(&pFixtureDef);
				}
			}
		}

		b2MassData pMassData;
		pMassData.center = b2Vec2(0, 0);
		pMassData.I = 1.0f;
		pMassData.mass = Mass;

		pBody->SetMassData(&pMassData);
	}
}

void Game::BlockEllipsoid::Init()
{
	Points.clear();
	DisplayPoints.clear();

	if (Radius2 == 0 && (Radius1 > Consts::Pi && Radius1 < Consts::Pi * 2.0f))
	{
		float Angle = -(AngleEnd - AngleStart) / (PointsCount - 1);

		for (int i = 0; i < PointsCount; i++)
		{
			Points.push_back(Vector2f(
					-sinf(AngleStart + Angle * i) * Radius1 * qX,
					-cosf(AngleStart + Angle * i) * Radius1 * qY
				));
		}

		if (DisplayPointsCount)
		{
			float Angle = -(AngleEnd - AngleStart) / (DisplayPointsCount - 1);

			for (int i = 0; i < DisplayPointsCount; i++)
			{
				DisplayPoints.push_back(Vector2f(
						-sinf(AngleStart + Angle * i) * Radius1 * qX,
						-cosf(AngleStart + Angle * i) * Radius1 * qY
					));
			}
		} else {
			DisplayPointsCount = Points.size();
			for (int i = 0; i < Points.size(); i++)
				DisplayPoints.push_back(Vector2f(Points[i]));
		}
	} else {
		float Angle = -(AngleEnd - AngleStart) / (PointsCount / 2 - 1);

		for (int i = 0; i < PointsCount / 2; i++)
		{
			Points.push_back(Vector2f(
					-sinf(AngleStart + Angle * i) * Radius1 * qX,
					-cosf(AngleStart + Angle * i) * Radius1 * qY
				));
		}

		for (int i = PointsCount / 2 - 1; i >= 0; i--)
		{
			Points.push_back(Vector2f(
					-sinf(AngleStart + Angle * i) * Radius2 * qX,
					-cosf(AngleStart + Angle * i) * Radius2 * qY
				));
		}

		if (DisplayPointsCount)
		{
			float Angle = -(AngleEnd - AngleStart) / (DisplayPointsCount / 2 - 1);

			for (int i = 0; i < DisplayPointsCount / 2; i++)
			{
				DisplayPoints.push_back(Vector2f(
						-sinf(AngleStart + Angle * i) * Radius2 * qX,
						-cosf(AngleStart + Angle * i) * Radius2 * qY
					));
			}

			for (int i = 0; i < DisplayPointsCount / 2; i++)
			{
				DisplayPoints.push_back(Vector2f(
						-sinf(AngleEnd - Angle * i) * Radius1 * qX,
						-cosf(AngleEnd - Angle * i) * Radius1 * qY
					));
			}

			/*DisplayPoints.push_back(Vector2f(
						-sinf(AngleStart) * Radius2 * qX,
						-cosf(AngleStart) * Radius2 * qY
					));*/

			/*for (int i = DisplayPointsCount / 2 - 1; i >= 0; i--)
			{
				DisplayPoints.push_back(Vector2f(
						-sinf(AngleStart + Angle * i) * Radius2 * qX,
						-cosf(AngleStart + Angle * i) * Radius2 * qY
					));
			}

			for (int i = 0; i < DisplayPointsCount / 2; i++)
			{
				DisplayPoints.push_back(Vector2f(
						-sinf(AngleStart + Angle * i) * Radius1 * qX,
						-cosf(AngleStart + Angle * i) * Radius1 * qY
					));
			}*/

			
		} else {
			DisplayPointsCount = Points.size();
			for (int i = 0; i < Points.size(); i++)
				DisplayPoints.push_back(Vector2f(Points[i]));
		}
	}

	InitPhysics();
}

void Game::BlockEllipsoid::Render(float Zoom)
{
	if (DisplayPoints.size() == 0 || Color.A == 0.0f)
		return;

	glPushMatrix();
	glTranslatef(Pos.X * Zoom, Pos.Y * Zoom, 0);
	glRotatef(pBody->GetAngle() / Consts::Pi * 180.0f, 0.0f, 0.0f, 1.0f);

	if (Fill)
	{
		if (Radius2 == 0)
		{
			glColor4f(Color.R, Color.G, Color.B, Color.A);
			glBegin(GL_POLYGON);
				for (int i = 0; i < DisplayPoints.size(); i++)
				{
					glVertex2f(DisplayPoints[i].X * Zoom, DisplayPoints[i].Y * Zoom);
				}
			glEnd();
		} else {
			glColor4f(Color.R, Color.G, Color.B, Color.A);
			glBegin(GL_QUAD_STRIP);
				for (int i = 0; i < DisplayPoints.size() / 2 + 1; i++)
				{
					glVertex2f(DisplayPoints[i].X * Zoom, DisplayPoints[i].Y * Zoom);
					glVertex2f(DisplayPoints[DisplayPoints.size() - i].X * Zoom, DisplayPoints[DisplayPoints.size() - i].Y * Zoom);
				}
			/*for (int i = 0; i < DisplayPoints.size(); i++)
				{
					glColor4f(i * 1.0f / DisplayPoints.size(), i * 1.0f / DisplayPoints.size(), i * 1.0f / DisplayPoints.size(), 1.0f);
					glVertex2f(DisplayPoints[i].X * Zoom, DisplayPoints[i].Y * Zoom);
				//	glVertex2f(DisplayPoints[DisplayPoints.size() - i].X * Zoom, DisplayPoints[DisplayPoints.size() - i].Y * Zoom);
				}*/
			glEnd();
		}
	}

	glLineWidth(BorderWidth * Zoom);
	glColor4f(0.0f, 0.0f, 0.0f, Color.A);
	glBegin(Loop ? GL_LINE_LOOP : GL_LINE_STRIP);
		for (int i = 0; i < DisplayPoints.size(); i++)
		{
			glVertex2f(DisplayPoints[i].X * Zoom, DisplayPoints[i].Y * Zoom);
		}
	glEnd();

	glPopMatrix();
}

Game::BlockPlayer::BlockPlayer()
{
	Type = 3;
}

void Game::BlockPlayer::Render(float Zoom)
{
	if (DisplayPoints.size() == 0 || Color.A == 0.0f)
		return;

	glPushMatrix();
	glTranslatef(Pos.X * Zoom, Pos.Y * Zoom, 0);
	glRotatef(pBody->GetAngle() / Consts::Pi * 180.0f, 0.0f, 0.0f, 1.0f);

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

	glBegin(GL_POLYGON);
		for(int i = 0; i < 32; i++)
		{
			glVertex2f(sinf(i * Consts::Pi * 2.0f / 32.0f) * Radius1 * Zoom, cosf(i * Consts::Pi * 2.0f / 32.0f) * Radius1 * Zoom);
		}
	glEnd();

	glColor4f(0.0f, 0.0f, 0.0f, 1.0f);
	glLineWidth(BorderWidth * Zoom);

	glBegin(GL_LINE_LOOP);
		for(int i = 0; i < 32; i++)
		{
			glVertex2f(sinf(i * Consts::Pi * 2.0f / 32.0f) * Radius1 * Zoom, cosf(i * Consts::Pi * 2.0f / 32.0f) * Radius1 * Zoom);
		}
	glEnd();

	static DWORD KawaiiTick = GetTickCount();
	static DWORD KawaiiTickStop = GetTickCount() + 100;
	
	if (pBody->GetLinearVelocity().Length() < 30.0f &&
		abs(pBody->GetAngularVelocity()) < Consts::Pi * 0.3f)
	{
		for (int j = -1; j <= 1; j += 2)
		{
			glPushMatrix();

			glTranslatef(Radius1 * 0.3f * j * Zoom, -Radius1 * 0.40f * Zoom, 0.0f);

			glBegin(GL_POLYGON);
				for(int i = 0; i < 16; i++)
				{
					glVertex2f(sinf(i * Consts::Pi * 2.0f / 16.0f) * Radius1 * 0.15f * Zoom, cosf(i * Consts::Pi * 2.0f / 16.0f) * Radius1 * 0.2f * Zoom);
				}
			glEnd();

			glPopMatrix();
		}
	} else
	if (GetTickCount() > KawaiiTick)
	{
		glBegin(GL_LINES);
			glVertex2f(Radius1 * 0.2f * Zoom, -Radius1 * 0.30f * Zoom);
			glVertex2f(Radius1 * 0.35f * Zoom, -Radius1 * 0.6f * Zoom);

			glVertex2f(Radius1 * 0.5f * Zoom, -Radius1 * 0.30f * Zoom);
			glVertex2f(Radius1 * 0.35f * Zoom, -Radius1 * 0.6f * Zoom);

			glVertex2f(-Radius1 * 0.2f * Zoom, -Radius1 * 0.30f * Zoom);
			glVertex2f(-Radius1 * 0.35f * Zoom, -Radius1 * 0.6f * Zoom);

			glVertex2f(-Radius1 * 0.5f * Zoom, -Radius1 * 0.30f * Zoom);
			glVertex2f(-Radius1 * 0.35f * Zoom, -Radius1 * 0.6f * Zoom);
		glEnd();

		if (GetTickCount() > KawaiiTickStop)
		{
			KawaiiTick = GetTickCount() + 500 + rand() * 1000 / RAND_MAX;
			KawaiiTickStop = KawaiiTick + 100 + rand() * 1000 / RAND_MAX;
		}
	} else {
		glBegin(GL_LINES);
			glVertex2f(Radius1 * 0.3f * Zoom, -Radius1 * 0.30f * Zoom);
			glVertex2f(Radius1 * 0.6f * Zoom, -Radius1 * 0.6f * Zoom);

			glVertex2f(Radius1 * 0.3f * Zoom, -Radius1 * 0.30f * Zoom);
			glVertex2f(Radius1 * 0.6f * Zoom, -Radius1 * 0.0f * Zoom);

			glVertex2f(-Radius1 * 0.3f * Zoom, -Radius1 * 0.30f * Zoom);
			glVertex2f(-Radius1 * 0.6f * Zoom, -Radius1 * 0.6f * Zoom);

			glVertex2f(-Radius1 * 0.3f * Zoom, -Radius1 * 0.30f * Zoom);
			glVertex2f(-Radius1 * 0.6f * Zoom, -Radius1 * 0.0f * Zoom);
		glEnd();
	}

	glPopMatrix();
}