#pragma once

#include "Common.h"

#include <vector>

using namespace Types;

namespace Game
{
	class Block
	{
	public:
		Vector2f Pos;

		std::vector<Vector2f> Points;
		std::vector<Vector2f> DisplayPoints;
		Vector4f Color;
		float BorderWidth;

		bool Loop;
		bool Fill;
		bool Fixed;

		float Mass; // def 1.0f kg

		// 0 - transparent
		// 1 - solid
		// 2 - danger
		// 3 - player
		int Type;

		b2BodyDef pBodyDef;
		b2Body * pBody;
		b2PolygonShape pShape;
		b2FixtureDef pFixtureDef;

		Block();
		~Block();

		virtual void InitPhysics(); // do not call it manually

		virtual void Init();

		virtual void Process(float DeltaTime);
		virtual void Render(float Zoom = 1.0f);
	};

	class BlockEllipsoid : public Block
	{
	public:
		std::vector<Vector2f> DisplayPoints; // see DisplayPointsCount

		float AngleStart, AngleEnd;
		float Radius1, Radius2;
		float qX, qY;

		int PointsCount; // max = 32, def = 24
		int DisplayPointsCount; // 0 = PointsCount

		BlockEllipsoid();
		~BlockEllipsoid();

		virtual void InitPhysics(); // do not call it manually

		virtual void Init();
		virtual void Render(float Zoom = 1.0f);
	};

	class BlockPlayer : public BlockEllipsoid
	{
	public:
		BlockPlayer();

		virtual void Render(float Zoom = 1.0f);
	};

	/*Block * CreateEllipsoid(float Radius, int PointsCount, float qX = 1.0f, float qY = 1.0f, float AngleStart = 0.0f, float AngleEnd = Consts::Pi * 2.0f);
	Block * CreateEllipsoidBar(float Radius1, float Radius2, int PointsCount, float qX = 1.0f, float qY = 1.0f, float AngleStart = 0.0f, float AngleEnd = Consts::Pi * 2.0f);*/
};
