#include "Common.h"

Window MainWnd;
OpenGL GL;

Texture * Logo;

bool Keys[256];
Vector2i CursorPos;

Game::World * World;
Game::Player * Player;



// stollen from NeHe...

GLuint	base;				// Base Display List For The Font Set

GLvoid BuildFont(GLvoid)								// Build Our Bitmap Font
{
	HFONT	font;										// Windows Font ID
	HFONT	oldfont;									// Used For Good House Keeping

	base = glGenLists(96);								// Storage For 96 Characters

	font = CreateFont(	-18,							// Height Of Font
						0,								// Width Of Font
						0,								// Angle Of Escapement
						0,								// Orientation Angle
						0,						// Font Weight
						FALSE,							// Italic
						FALSE,							// Underline
						FALSE,							// Strikeout
						ANSI_CHARSET,					// Character Set Identifier
						OUT_TT_PRECIS,					// Output Precision
						CLIP_DEFAULT_PRECIS,			// Clipping Precision
						CLEARTYPE_QUALITY,			// Output Quality
						FF_DONTCARE|DEFAULT_PITCH,		// Family And Pitch
						"Tahoma");					// Font Name

	oldfont = (HFONT)SelectObject(GL.GetDC(), font);           // Selects The Font We Want
	wglUseFontBitmaps(GL.GetDC(), 32, 96, base);				// Builds 96 Characters Starting At Character 32
	SelectObject(GL.GetDC(), oldfont);							// Selects The Font We Want
	DeleteObject(font);									// Delete The Font
}

GLvoid KillFont(GLvoid)									// Delete The Font List
{
	glDeleteLists(base, 96);							// Delete All 96 Characters
}

GLvoid glPrint(const char *fmt, ...)					// Custom GL "Print" Routine
{
	char		text[256];								// Holds Our String
	va_list		ap;										// Pointer To List Of Arguments

	if (fmt == NULL)									// If There's No Text
		return;											// Do Nothing

	va_start(ap, fmt);									// Parses The String For Variables
	    vsprintf(text, fmt, ap);						// And Converts Symbols To Actual Numbers
	va_end(ap);											// Results Are Stored In Text

	glPushAttrib(GL_LIST_BIT);							// Pushes The Display List Bits
	glListBase(base - 32);								// Sets The Base Character to 32
	glCallLists(strlen(text), GL_UNSIGNED_BYTE, text);	// Draws The Display List Text
	glPopAttrib();										// Pops The Display List Bits
}


void Process(float DeltaTime)
{
	if (Keys[VK_LEFT])
	{
		World->Down = World->Down.Rotate(DeltaTime * 1.5f);
	}

	if (Keys[VK_RIGHT])
	{
		World->Down = World->Down.Rotate(-DeltaTime * 1.5f);
	}

	if (Keys[VK_UP])
	{
		World->Zoom += 1.0f * DeltaTime;
		if (World->Zoom > 3.0f)
			World->Zoom = 3.0f;
	}

	if (Keys[VK_DOWN])
	{
		World->Zoom -= 1.0f * DeltaTime;
		if (World->Zoom < 0.1f)
			World->Zoom = 0.1f;
	}

	if (Keys[VK_RETURN])
	{
		Keys[VK_RETURN] = false;
		Player->Reset();
	}

	GL.Clear();
	glLoadIdentity();

	World->Process(DeltaTime);

	glPushMatrix();
	//glTranslatef(50.0f, 150.0f, 0);
	glRasterPos2f(50.0f, 50.0f);
	glColor4f(0.2f, 0.2f, 0.2f, 0.5f);
	glPrint("Level %d: %s", World->CurrLevel + 1, World->Levels[World->CurrLevel]->Name);
	glPopMatrix();

	glPushMatrix();
	glTranslatef(MainWnd.GetClientWidth() * 0.5f, MainWnd.GetHeight() - 50.0f, 0);
	glRotatef(-World->Down.Angle() * 180.0f / Consts::Pi - 90.0f, 0, 0, 1);
	glLineWidth(2.0f);
	glColor4f(0.2f, 0.2f, 0.2f, 0.5f);
	glBegin(GL_LINES);
		glVertex2f(0.0f, -15.0f);
		glVertex2f(0.0f, 15.0f);

		glVertex2f(0.0f, -15.0f);
		glVertex2f(7.0f, -8.0f);

		glVertex2f(0.0f, -15.0f);
		glVertex2f(-7.0f, -8.0f);
	glEnd();
	glPopMatrix();

	GL.SwapBuffers();
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
	switch(Message)
	{
	case WM_DESTROY:
		Application::Terminate();
        break;
	case WM_KEYDOWN:
		Keys[wParam] = true;
		break;
	case WM_KEYUP:
		Keys[wParam] = false;
		break;
	case WM_LBUTTONDOWN:
		Keys[VK_LBUTTON] = true;
		break;
	case WM_LBUTTONUP:
		Keys[VK_LBUTTON] = false;
		break;
	case WM_RBUTTONDOWN:
		Keys[VK_RBUTTON] = true;
		break;
	case WM_RBUTTONUP:
		Keys[VK_RBUTTON] = false;
		break;
	case WM_MOUSEMOVE:
		CursorPos.X = LOWORD(lParam);
		CursorPos.Y = HIWORD(lParam);
		break;
	default:
        return DefWindowProc(hWnd, Message, wParam, lParam);
        break;
	}

	return 0;
}

void Initialize()
{
	MainWnd.Init(800, 600, true, "Rototto", WndProc);
	GL.Init(MainWnd);

	BuildFont();

	glClearColor(0.5f, 0.75f, 1.0f, 1.0f);

	World = new Game::World();
	World->AddLevel(Level1Name, Level1);
	World->AddLevel(Level2Name, Level2);
	World->AddLevel(Level3Name, Level3);
	World->AddLevel(Level4Name, Level4);

	World->Init();
}

void Finalize()
{
	delete World;
	KillFont();
}

void Run()
{
	MSG Msg;
	DWORD LastTick = GetTickCount();
	while (Application::IsRunning())
	{
		if (PeekMessage(&Msg, 0, 0, 0, PM_REMOVE))
		{
			TranslateMessage(&Msg);
			DispatchMessage(&Msg);
		}

		DWORD CurrTick = GetTickCount();
		Process((CurrTick - LastTick) / 1000.0f);
		LastTick = CurrTick;

		Sleep(1);
	}
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
	// assign log file output
	Log::AssignLogFile("Rototto.log");

	// dump system info
	Application::AddInitializator(Info::DumpSystemInfo);

	// game core initialization
	Application::AddInitializator(Initialize);

	// dump OpenGL info
	Application::AddInitializator(OpenGL::DumpInfo);

	// game core finalization
	Application::AddFinalizator(Finalize);

	// run application
	Application::Run(Run);
}