#include "World.h"

Game::World::World()
{
	Down = Vector2f(0, 7.0f);
	Zoom = 1.0f;

	pWorld = new b2World(b2Vec2(Down.X, Down.Y), true);

	Blocks.clear();
}

Game::World::~World()
{
	ClearBlocks(true);

	delete pWorld;
}

void Game::World::Init()
{
	if (!::Player)
		::Player = new Game::Player();
	CurrLevel = 0;
	(*Levels[CurrLevel]->InitFunc)();
}

void Game::World::ClearBlocks(bool WithPlayerBlock)
{
	if (Blocks.size() == 0)
		return;

	for (int i = 0; i < Blocks.size(); i++)
	{
		if (!WithPlayerBlock && Blocks[i] == ::Player->Block)
			continue;

		__try
		{
			delete Blocks[i];
			Blocks[i] = nullptr;
		}
		__except(EXCEPTION_EXECUTE_HANDLER)
		{
		}
	}

	Blocks.clear();
}

void Game::World::InitBlocks()
{
	if (Blocks.size() == 0)
		return;

	for (int i = 0; i < Blocks.size(); i++)
	{
		__try
		{
			Blocks[i]->Init();
		}
		__except(EXCEPTION_EXECUTE_HANDLER)
		{
		}
	}
}

void Game::World::ProcessBlocks(float DeltaTime)
{
	if (Blocks.size() == 0)
		return;

	for (int i = 0; i < Blocks.size(); i++)
	{
		__try
		{
			Blocks[i]->Process(DeltaTime);
		}
		__except(EXCEPTION_EXECUTE_HANDLER)
		{
		}
	}
}

void Game::World::RenderBlocks()
{
	if (Blocks.size() == 0)
		return;

	for (int i = 0; i < Blocks.size(); i++)
	{
		__try
		{
			Blocks[i]->Render(Zoom);
		}
		__except(EXCEPTION_EXECUTE_HANDLER)
		{
		}
	}
}

void Game::World::Process(float DeltaTime)
{
	if (!::Player)
		Init();

	pWorld->SetGravity(b2Vec2(Down.X, Down.Y));
	for (int i = 0; i < 10; i++)
	{
		pWorld->Step(DeltaTime * 10.0f / 10.0f, 5, 5);
		pWorld->ClearForces();
	}
	ProcessBlocks(DeltaTime);

	if (::Player)
		::Player->Process(DeltaTime);

	glPushMatrix();
	glTranslatef(400.0f, 300.0f, 0.0f);
	glRotatef(-Down.Angle() * 180.0f / Consts::Pi + 90.0f, 0.0f, 0.0f, 1.0f);
	if (::Player)
		glTranslatef(-::Player->Block->Pos.X * Zoom, -::Player->Block->Pos.Y * Zoom, 0);

	RenderBlocks();

	glPopMatrix();
}

void Game::World::AddLevel(const char * Name, LevelInitFunc & InitFunc)
{
	Level * L = new Level;
	L->Name = Name;
	L->InitFunc = &InitFunc;
	Levels.push_back(L);
}

void Game::World::NextLevel()
{
	CurrLevel++;
	if (CurrLevel >= Levels.size())
		CurrLevel = 0;
	(*Levels[CurrLevel]->InitFunc)();
}