unit u_Game;

interface
uses
  u_Size, u_Button, u_Map, u_Win;

type

  T_Player = record
    Enabled,
    MD   : Boolean;
    X, Y : Single;
  end;

  T_ButtonBreeze = class(t_Button)
    procedure Draw;
  end;
  
  T_Game = class
    Point : Single;
    MyLevel,
    Xod,
    MyFish : Integer;
    BReset,
    BMenu   : T_Button;
    BBreeze : array[1..4] of T_ButtonBreeze;

    Map : t_Map;
    WiN,
    Wait : Boolean;

    WinW : t_Win;
    constructor Create;
    procedure Start(Level : Integer);
    procedure Load;
    procedure Draw;
    procedure Work;
  end;

var
  InfoS : array[1..10] of string;

implementation
uses
  GSOUND,
  u_Player, u_Main, u_Ship,
  u_SceneM, u_Cursor, u_Material,
  OMEGA, LexUtils, Setup;

procedure T_ButtonBreeze.Draw;
begin
  SSPrite_Draw(m_Cone[T, St], X, Y, W, H);
end;

constructor T_Game.Create;
begin
  Map := t_Map.Create;
  WinW := t_Win.Create;

  BReset := T_Button.Create('', 1, 20, WinH - c_TileS + 16, 180, 48, 0.6);
  BMenu  := T_Button.Create('',    1, 20, WinH - c_TileS + 72, 180, 48, 0.6);

  BBreeze[1] := T_ButtonBreeze.Create('', 1, 220 + 68,  WinH + 4 - 128, 64, 64);
  BBreeze[2] := T_ButtonBreeze.Create('', 2, 220 + 68,  WinH + 4 - 64,  64, 64);
  BBreeze[3] := T_ButtonBreeze.Create('', 3, 220,       WinH + 4 - 96,  64, 64);
  BBreeze[4] := T_ButtonBreeze.Create('', 4, 220 + 136, WinH + 4 - 96,  64, 64);
end;

procedure T_Game.Start(Level : Integer);
begin
  MyLevel := Level;
// if Main.Scene <> tg_game then
    Main.SetScene(tg_game);
end;

procedure T_Game.Load;
begin
  Map.Load(Menu.M_Level.LevelS[MyLevel, 2]);
  WiN  := False;
  Wait := False;

  Point := 100;
  Xod   := 0;
end;


procedure Info;
var
  I : Integer;
begin
  Omega.Pr2D_Rect( 8,  8, 156, 126,      15, 120, 1);
  Omega.Pr2D_Rect(11, 11, 150, 120, $FFFFFF, 240, 0);
  Omega.Text_Draw(FontA1, 16, 16, PChar('FPS : ' + Int2S(Omega.OE_GetFPS)), 0.4);
  for I := 1 to 3 do
  Omega.Text_Draw(FontA1, 16, 25 + I * 24, PChar(InfoS[I]), 0.4);

end;

const
  LLine = 170;
  ALine = 120;
  CLine = 4855745;

procedure T_Game.Draw;
var
  I : Integer;
  S : string;
begin
  Map.Draw;

  SSprite_Draw(m_Board, 0, WinH - 128, 1024, 128);
  BReset.Draw;
  BMenu.Draw;

  for I := 1 to 4 do
    BBreeze[I].Draw;

//  Info;

  S := ': ' + Int2S(Trunc(Point));
  Text_Draw(FontA1, 512 + 26, WinH - 40, PChar(S), 0.5);
  S := ': ' + Int2S(Xod);
  Text_Draw(FontA1, 512 + 200, WinH - 40, PChar(S), 0.5);

  if Win then
    WinW.Draw;
end;

procedure T_Game.Work;

  procedure Breeze(pX, pY : Integer);
  var
    Z, IX, IY : Integer;
  begin
    PlaySound(1);
    Map.Water.MP := Set2DRealPoint(pX * 5, pY * 5);
    Wait := True;

    for IX := 0 to 7 do
    for IY := 0 to 4 do
    if Map.Tile[IX, IY].T > 2 then
      Map.Tile[IX, IY].T := 0;

    for Z := 0 to Length(Map.Ship) - 1 do
      ShipMove(Map.Ship[Z], pX, pY);
    for Z := 0 to Length(Map.Bord) - 1 do
      ShipMove(Map.Bord[Z], pX, pY);

    Point := Point - 5;
    Xod := Xod + 1;
  end;

var
	I, Y : Integer;
  ShipOnFish,
  NoWait : Boolean;
begin
  if (BMenu.Work)
  or (Key_Press(k_ESCAPE)) then
    Main.SetScene(tg_menu);

  if (BReset.Work)
  or (Key_Press(k_R)) then
    Main.SetScene(tg_game);
  //

  //
  if Win then
  begin
    WinW.Work;
    Exit;
  end;

  for I := 0 to Length(Map.Ship) - 1 do
  with Map.Ship[I] do
  if trunc(A) <> trunc(DA) then
    begin
      mA := Abs(dA - A) / 10;

      if (A<-90)
      and (DA>90)
      then
        mA:= -mA
      else
      if (A>90)
      and (DA<-90)
      then
        mA:= +mA
      else
      if  (A>DA)
      then
        mA:= -mA
      else
      if A<DA then
        mA:= +mA;

      A := A + MA;
      NormAngle(A);
    end;

  if Wait then
  begin
    for I := 0 to Length(Map.Ship) - 1 do
      ShipWork(Map.Ship[I]);
    for I := 0 to Length(Map.Bord) - 1 do
      ShipWork(Map.Bord[I]);

    NoWait := True;

    for I := 0 to Length(Map.Ship) - 1 do
    if Map.Ship[I].Move then
      NoWait := False;

    for I := 0 to Length(Map.Bord) - 1 do
    if Map.Bord[I].Move then
      NoWait := False;

    if NoWait then Wait := False;
  end
  else
  begin
    if (BBreeze[1].Work)
    or (Key_Down(K_W))
    or (Key_Down(K_UP)) then
      Breeze(0, -1)
    else
    if (BBreeze[2].Work)
    or (Key_Down(K_S))
    or (Key_Down(K_Down)) then
      Breeze(0, +1)
    else
    if (BBreeze[3].Work)
    or (Key_Down(K_A))
    or (Key_Down(K_LEFT)) then
      Breeze(-1, 0)
    else
    if (BBreeze[4].Work)
    or (Key_Down(K_D))
    or (Key_Down(K_RIGHT)) then
      Breeze(+1, 0);
  end;

  Map.Work;

	MyFish := 0;
	for I := 0 to Length(Map.Fish) - 1 do
	begin
		for Y := 0 to Length(Map.Ship) - 1 do
		if  (Map.Fish[I].EP.X = Map.Ship[Y].EP.X)
		and (Map.Fish[I].EP.Y = Map.Ship[Y].EP.Y)  then
		begin
			Inc(MyFish);
			Break;
		end;
	end;

  if MyFish >= Length(Map.Fish) then
  begin
    WiN := True;
    PlaySound(3);
    Rec.NewRec(Menu.M_Level.LevelS[MyLevel, 2]);

    Menu.M_Level.LevelS[MyLevel, 3] := Int2S(Rec.RecTable.R[1].P ) + '  (' +
                                                  Rec.RecTable.R[1].N + ')';

    if Game.MyLevel < Menu.M_Level.MofLevel then
      WinW.S[0] := ' !'
    else
      WinW.S[0] := ' !';

    WinW.S[1] := ' : ' + Int2S(Game.Xod);
    WinW.S[2] := ' : ' + Int2S(Game.MyFish);
    WinW.S[3] := ' : ' + (Menu.M_Level.LevelS[MyLevel, 3]);
  end
  else
    WiN := False;

  if Point < 0 then
    Point := 0
  else
    Point := Point - 0.01;
end;


end.
