unit u_Map;

interface
uses
  LexUtils, u_Water, u_Ship;

type
  t_Tile = record
    A : Integer;
    H,
    M,
    T : Byte;
  end;



  t_Fishs = record
    P : TM2DRealPoint;
    A : Integer;
    EP : TM2DPoint;
  end;

  T_TileMap = array[0..7, 0..4] of t_Tile;
  t_Map = class
    Water : t_Water;
    Tile  : T_TileMap;
    Bord,
    Ship  : array of t_Ship;
    Fish : array of t_Fishs;
    constructor Create;
    procedure Work;
    procedure Draw;

    procedure Save(FN : string);
    procedure Open(FN : string);
    procedure Load(FN : string);
  end;
  
implementation

uses
  Omega, u_Size, u_Material;
  
constructor t_Map.Create;
begin
  Water := t_Water.Create;
end;

procedure t_Map.Work;
begin
  Water.Work;
end;

procedure t_Map.Draw;
var
  I, IX, IY : Integer;
const
  Shadow = 150;
begin
  STile_Draw(m_W1, 0, 0, c_TileS, c_TileS, k_TileX, k_TileY );

  for I := 0 to Length(Fish) - 1 do
  with Fish[I] do
    SSprite_Draw(m_Et[6, 1], P.X, P.Y, c_TileS, c_TileS, $FF, A);

// 
  for I := 0 to Length(Ship) - 1 do
  with Ship[I] do
    SSprite_Draw(m_Et[7, 2], P.X, P.Y, c_TileS + 8, c_TileS + 6, Shadow, A);

  for I := 0 to Length(Bord) - 1 do
  with Bord[I] do
    SSprite_Draw(m_Et[8, 2], P.X, P.Y, c_TileS + 8, c_TileS + 8, Shadow);

  for IX := 0 to 7 do
  for IY := 0 to 4 do
  with Tile[IX, IY] do
  if T = 2 then
     SSprite_Draw(m_Et[M, 2], IX * c_TileS + 8,
                              IY * c_TileS + 8,
                              c_TileS, c_TileS, Shadow, A);

//
  Water.Draw;

  for I := 0 to Length(Ship) - 1 do
  with Ship[I] do
  begin
    SSprite_Draw(m_Et[7, 1], P.X, P.Y, c_TileS, c_TileS, $FF, A);
   { Text_Draw(FontA1, P.X, P.Y, PChar(Int2S(trunc(A))), 0.3, 0, 225, $FFFFFF);
    Text_Draw(FontA1, P.X, P.Y + 15, PChar(Int2S(trunc(DA))), 0.3, 0, 225, $FFFFFF);
    Text_Draw(FontA1, P.X, P.Y + 30, PChar(Int2S(trunc(MA))), 0.3, 0, 225, $FFFFFF);
    }
  end;

  for I := 0 to Length(Bord) - 1 do
  with Bord[I] do
    SSprite_Draw(m_Et[8, 1], P.X, P.Y, c_TileS, c_TileS);
//
  for IX := 0 to 7 do
  for IY := 0 to 4 do
  with Tile[IX, IY] do
  if T = 2 then
     SSprite_Draw(m_Et[M, 1], IX * c_TileS,
                              IY * c_TileS,
                              c_TileS, c_TileS, $FF, A);

end;
procedure t_Map.Save(FN : string);
var
  F : file;
begin
  AssignFile(F, 'Maps\' + FN);
  ReWrite(F, 1);
  BlockWrite(F, Tile, SizeOf(t_TileMap));
  CloseFile(F);
end;

procedure t_Map.Open(FN : string);
var
  F : file;
begin
  AssignFile(F, 'Maps\' + FN);
  Reset(F, 1);
  BlockRead(F, Tile, SizeOf(t_TileMap));
  CloseFile(F);
end;

procedure t_Map.Load(FN : string);
var
  I, IX, IY : Integer;
begin
  SetLength(Fish, 0);
  SetLength(Ship, 0);
  SetLength(Bord, 0);

  Open(FN);
  for IX := 0 to 7 do
  for IY := 0 to 4 do
  case Tile[IX, IY].M of
    6 :
    begin
      I := Length(Fish);
      SetLength(Fish, I + 1);
      Fish[I].A := Tile[IX, IY].A;
      Fish[I].P := Set2DRealPoint(IX * c_TileS, IY * c_TileS);
      Fish[I].EP := Set2DPoint(IX, IY);
    end;
    7 :
    begin
      I := Length(Ship);
      SetLength(Ship, I + 1);
      Ship[I].dA := Tile[IX, IY].A;
      Ship[I].A := Ship[I].dA - 30;
      Ship[I].P := Set2DRealPoint(IX * c_TileS, IY * c_TileS);
      Ship[I].MP := Ship[I].P;
      Ship[I].EP := Set2DPoint(IX, IY);
      Ship[I].Move := False;
    end;
    8 :
    begin
      I := Length(Bord);
      SetLength(Bord, I + 1);
      Bord[I].A := Tile[IX, IY].A;
      Bord[I].P := Set2DRealPoint(IX * c_TileS, IY * c_TileS);
      Bord[I].MP := Bord[I].P;
      Bord[I].EP := Set2DPoint(IX, IY);
      Bord[I].Move := False;
    end;
  end;
end;

end.
