unit myGUI;

interface

uses
  Windows,
  OpenGL,
  uSysUtils,
  eXgine;

type
  THint   = object
    x,y   : single;
    w,h   : word;
    stngs : byte;
    Text  : array [1..256] of String;
  public
    procedure Render;
  end;

  TButton = object
    X,Y : single;
    w,h : word;
    Over    : Boolean;
    Down    : Boolean;
    Color   : array[0..3] of Single;
    Caption : PChar;
    Visible : Boolean;
    OnClick : procedure;
    private
      IsTex   : boolean;
      rotZ    : single;
      texture : TTexture;
      TexCoord: array [1..2] of Single;
      ShowHint: boolean;
      Hint    : THint;
    public
      procedure Render;
      procedure Update;
{    .
     4  :
  - 
  -   
  - 
  - 
         .
}
      procedure SetTexture(tx: TTexture);
      procedure SetHint(_H: THint);
  end;

  TGCursor = object
     Visible: boolean;
     Color   : array[0..3] of Single;
     w, h    : word;
     private
       IsTex   :  boolean;
       texture : TTexture;
     public
       procedure Render;
       procedure SetTexture(tx: TTexture);
  end;


var
  CPos: TPoint;
implementation

procedure THint.Render;
var i : integer;
begin
  glPushMatrix;
  glTranslatef(x, y, 0);
  //  Ogl.Blend(BT_SUB);
         glColor3f(1, 1 ,1);
         glBegin(GL_QUADS);
           glVertex2f(-w/2, +h/2);
           glVertex2f(+w/2, +h/2);
           glVertex2f(+w/2, -h/2);
           glVertex2f(-w/2, -h/2);
         glEnd;
  //  Ogl.Blend(BT_NONE);
    glColor3f(0, 0, 0);
    for I := 1 to stngs do
      ogl.TextOut(0, x-w/2+5, x-w/2+5+(i-1)*10, PChar(Text[i]));
      
  glPopMatrix;
end;

{$REGION 'Button'}
procedure TButton.Render;
begin
  if not Visible then Exit;
  glPushMatrix;
  glTranslatef(x, y, 0);
    Ogl.Blend(BT_SUB);
    if not IsTex then
       begin
         glColor4fv(@Color);
         glBegin(GL_QUADS);
           glVertex2f(-w/2, +h/2);
           glVertex2f(+w/2, +h/2);
           glVertex2f(+w/2, -h/2);
           glVertex2f(-w/2, -h/2);
         glEnd;
       end else
       begin
         glColor4f(1,1,1,1);
         tex.Enable(texture);
         glBegin(GL_QUADS);
           glTexCoord2f(0, TexCoord[2]);
           glVertex2f(-w/2, +h/2);
           glTexCoord2f(1, TexCoord[2]);
           glVertex2f(+w/2, +h/2);
           glTexCoord2f(1, TexCoord[1]);
           glVertex2f(+w/2, -h/2);
           glTexCoord2f(0, TexCoord[1]);
           glVertex2f(-w/2, -h/2);
         glEnd;
         tex.Disable();
       end;
    if Over and ShowHint then
      begin
        GetCursorPos(CPos);
        hint.x:=CPos.X-hint.w/2;
        hint.y:=CPos.Y+Hint.h/2;
        hint.Render;
      end;
    
    Ogl.Blend(BT_NONE);
    glColor3f(0, 0, 0);
    Ogl.TextOut(0, -w/2+5, -h/2+4, Caption);
  glPopMatrix;
end;

procedure TButton.Update;
   procedure UpdateFrame(Frame: integer);
     begin
       TexCoord[1]:=Frame*0.25;
       TexCoord[2]:=(Frame+1)*0.25;
     end;
begin
  GetCursorPos(CPos);
  if not Visible then Exit;
  if (CPos.X > x-w/2) and (CPos.X < x+w/2) and
     (CPos.Y > y-h/2) and (CPos.Y < y+h/2) then Over:=True else Over:=False;
  if Over then
  begin
    Color[0] := 1;
    Color[1] := 1;
    Color[2] := 1;
  end else
  begin
    Color[0] := 0.8;
    Color[1] := 0.8;
    Color[2] := 0.8;
  end;

  if Over and Inp.Down(M_BTN_1) then
  begin
    Down:=True;
  end;

  if (not Inp.Down(M_BTN_1)) and Down then
  begin
    Down:=False;
    if Assigned(OnClick) then OnClick;
  end;

  if Down and (not Over) then Down:=False;

  UpdateFrame(0);
  if Down then UpdateFrame(2)
     else if Over then UpdateFrame(1);

end;

procedure TButton.SetTexture(tx: Integer);
begin
  texture:=tx;
  istex:=true;
end;

procedure TButton.SetHint(_H: THint);
begin
  Hint:=_H;
  ShowHint:=true;
end;

{$ENDREGION}

{$REGION 'Cursor'}
procedure TGCursor.Render;
begin
  GetCursorPos(CPos);
  if not Visible then exit;
  glPushMatrix;
  glTranslatef(Cpos.x, CPos.y, 0);
  if not IsTex then
     begin
       glColor4fv(@Color);
       glBegin(GL_TRIANGLES);
         glVertex2f(0.00, 0.00);
         glVertex2f(0.10, 0.05);
         glVertex2f(0.05, 0.10);
       glEnd;
     end else
     begin
       glColor4f(1,1,1,1);
         tex.Enable(texture);
         glBegin(GL_QUADS);
           glTexCoord2f(0, 1);
           glVertex2f(-w/2, +h/2);
           glTexCoord2f(1, 1);
           glVertex2f(+w/2, +h/2);
           glTexCoord2f(1, 0);
           glVertex2f(+w/2, -h/2);
           glTexCoord2f(0, 0);
           glVertex2f(-w/2, -h/2);
         glEnd;
         tex.Disable();
     end;
  glPopMatrix;
end;

procedure TGCursor.SetTexture(tx: Integer);
begin
  texture:=tx;
  istex:=true;
end;

{$ENDREGION}
end.
