unit uSprite;

interface

uses Windows, OpenGL, eXgine;

type
  TAnimatedTexture = record
    W,H     : integer;
    Texture : TTexture;
  end;

  TAnimation = record
    Name : PChar;
    FPS, FrameCount : integer;
    fWidth, fHeight : word;
    Atex : TAnimatedTexture;
  end;

  TSprite = class
    constructor Create(AnimCount : cardinal; _w, _h: word);
    private
      Anim: array of Tanimation;

      AnimUsed, CurrAnim, CurrFrame, upds: cardinal;
      Play : boolean;

      tx: array [1..4, 1..2] of real;
      lastFrame: cardinal;

      DoMove      : procedure;
      DoCollision : procedure;

      procedure CalculateTexCoords;
    public
      pos: TVector;
      w,h: word;

      procedure AddAnimation(FAnim: TAnimation);

      procedure PlayAnim(AnimName: string);
      procedure StopAnim;

      procedure Render;
      procedure Update;
  end;

 { TSpriteEngine = class
    constructor Create;
    private
      SpriteList   : array of TSprite;
      AnimationList: array of TAnimation;
      TextureList  : array of TAnimatedTexture;

      SpriteCount, AniCount, TexCount: cardinal;

    public
      procedure AddSprite(_w,_h: word);
  end;    }

implementation

{$REGION 'TSprite'}

constructor TSprite.Create(AnimCount: Cardinal; _w, _h: word);
begin
  SetLength(Anim, AnimCount);
  CurrAnim:=1;
  w:= _w;
  h:= _h;
end;



procedure TSprite.AddAnimation(FAnim: TAnimation);
begin
  inc(AnimUsed);
  Anim[AnimUsed]:=FAnim;
  CalculateTexCoords;
end;

 procedure TSprite.CalculateTexCoords;
     var wfCount, hfCount: cardinal;
         wStep, hStep : real;
         _CurrFrame : cardinal;
   begin
     wfCount:=Anim[CurrAnim].Atex.W div Anim[CurrAnim].fWidth;
     hfCount:=Anim[CurrAnim].Atex.H div Anim[CurrAnim].fHeight;
     wStep:=1/wfCount;
     hStep:=1/hfCount;

     //    ,     
     //  1...

     _CurrFrame:=CurrFrame-1;
     if _CurrFrame<1 then _CurrFrame:=Anim[CurrAnim].FrameCount;
     


     tx[1, 1]:=(_CurrFrame/wfCount)-
               (_CurrFrame div wfCount);
     tx[1, 2]:=(_CurrFrame div wfCount)*hStep;

     tx[2, 1]:=(_CurrFrame/wfCount)-
               (_CurrFrame div wfCount)+wstep;
     tx[2, 2]:=(_CurrFrame div wfCount)*hStep;

     tx[3, 1]:=(_CurrFrame/wfCount)-
               (_CurrFrame div wfCount)+wStep;
     tx[3, 2]:=(_CurrFrame div wfCount)*hStep+hStep;

     tx[4, 1]:=(_CurrFrame/wfCount)-
               (_CurrFrame div wfCount);
     tx[4, 2]:=(_CurrFrame div wfCount)*hStep+hStep;
   end;

Procedure TSprite.Render;
begin
  glPushMatrix;
  glTranslatef(pos.X, pos.Y, 0);
  glRotatef(180,0,0,1);
  glRotatef(180,0,1,0);
    Ogl.Blend(BT_SUB);
         glColor4f(1,1,1,1);
         tex.Enable(Anim[CurrAnim].Atex.Texture);
         //   
         //    , 
         if lastFrame<>CurrFrame then
         begin
           CalculateTexCoords;
           lastFrame:=CurrFrame;
         end;

         glBegin(GL_QUADS);
           glTexCoord2f(tx[1,1], Tx[1,2]);
           glVertex2f(-w/2, +h/2);
           glTexCoord2f(tx[2,1], Tx[2,2]);
           glVertex2f(+w/2, +h/2);
           glTexCoord2f(tx[3,1], Tx[3,2]);
           glVertex2f(+w/2, -h/2);
           glTexCoord2f(tx[4,1], Tx[4,2]);
           glVertex2f(-w/2, -h/2);
          { glTexCoord2f(0, 0);
           glVertex2f(-w/2, +h/2);
           glTexCoord2f(1, 0);
           glVertex2f(+w/2, +h/2);
           glTexCoord2f(1, 1);
           glVertex2f(+w/2, -h/2);
           glTexCoord2f(0, 1);
           glVertex2f(-w/2, -h/2);    }
         glEnd;
         tex.Disable();
    Ogl.Blend(BT_NONE);
  glPopMatrix;
  ogl.Set2D(0, 0, wnd.Width, wnd.Height);
  glColor3f(1, 0, 0);
  ogl.TextOut(0, 5, 35, PChar('CurrFrame: ' + IntToStr(CurrFrame)));
end;

procedure TSprite.Update;
begin
  //DoMove;
  //DoCollision;

  if Play then inc(upds);
  if upds>(50/Anim[CurrAnim].FPS) then
  begin
     inc(CurrFrame);
     upds:=0;
  end;

  if CurrFrame>Anim[CurrAnim].FrameCount then CurrFrame:=1;
end;

procedure TSprite.PlayAnim(AnimName: string);
var i: integer;
    f: boolean;
begin
  for i:=0 to AnimUsed do
    if Anim[i].Name=AnimName then
    begin
      CurrFrame:=1;
      CurrAnim:=i;
      Play:=true;
      upds:=0;
      f:=true;
    end;
  if not f then
    begin
      CurrAnim:=1;
      CurrAnim:=1;
      Play:=false;
      upds:=0;
      log.Print(PChar('   '+AnimName));
    end;
end;

procedure TSprite.StopAnim;
begin
  Play:=false;
end;

{$ENDREGION}

end.
